# -*- coding: utf-8 -*-
""" CapToMusic -- Copyright (c) 2006-2017 Bernd Jungmann
>>> capella-MusicXML-Export
    Version 1.41 vom 6.11.2017||

    Dieses Skript erzeugt aus der gerade geöffneten Capella-Datei
    eine MusicXML-Datei, die in Finale oder Sibelius importiert
    werden kann.||
    Einige Optimierungen für die verwendete Version von Finale
    bzw. Sibelius können eingestellt werden mit Hilfe des
    weiteren Skripts SetupC2M.py.||
    
<<<
    Kann auch ohne capella gestartet werden.
    Aufruf von der Kommandozeile:
    python CapToMusic.py -l# -f DateiOhneErweiterung -o ZielDatei
    Aufruf von der Python-Shell (erfragt Dateinamen per Dialogbox):
    >>> import CapToMusic
    >>> doit()
    Aufruf aus dem Idle-Fenster:
    Run - Run Module (F5)

    Aus DateiOhneErweiterung.capx wird ZielDatei.xml
    erzeugt. Ggfls. wird eine vorhandene Datei gleichen Namens ohne
    Rückfrage überschrieben.
    Falls die Angabe ZielDatei fehlt, wird DateiOhneErweiterung als
    Name der Zieldatei benutzt.
    -l# bezeichnet die Sprache für Meldungen des Programms.
        Bisher unterstützt:
        5   EnglischUS
        1   Deutsch

    Korrektur für Mehrtaktpausen in mehrstimmigen Zeilen 6.11.17 B. Jungmann
    Korrektur für punktierte Ganze und Doppelganze 14.11.2016
    Sprachunterstützung Französisch 10.10.16
    Vermeidung von redundantem Tempoexport abermals korrigiert. 23.6.16 B. Jungmann
    Korrekturen für Metrumangaben am Zeilenende 13.1.16 B. Jungmann
    Korrekturen für im System nicht vorhandene Mustersystemzeilen 13.1.16 B. Jungmann
    Schriftartenexport bei Seitenobjekten präzisiert 13.1.16 B. Jungmann
    Unterstützung für Doppeltaktfaulenzer an festen Taktstrichen 9.1.16 B. Jungmann
    Mehrtaktpausen jetzt nicht mehr über längere duration, sondern per
    measure-style multiple-rest exportiert 9.1.16 B. Jungmann
    Korrektur für Voltenklammer-Bezeichnungen 6.1.16 B. Jungmann
    Unterstützung des measure-Attributes für Ganztaktpausen 6.11.15 B. Jungmann
    Unterstützung für rechten Einzug bei Systemen 6.11.15 B. Jungmann
    Korrektur für Voltenklammern an festen Taktstrichen 3.11.15 B. Jungmann
    Korrektur für Mehrtaktpausen 20.7.15 B. Jungmann
    Korrektur für Schlüssel mit Oktavierungen 20.7.15 B. Jungmann
    Korrektur für Noten im RichText (unorthodoxe Tempobezeichnungen
    oder Rhythmusangaben) 10.6.2015 B. Jungmann
"""

# Vielen Dank an Loic Fejoz für seinen RTF-Parser Rtf2TxtAndFont
# http://sourceforge.net/projects/pyrtflib

globalVersionString = "CapToMusic.py CapXML to MusicXML converter version 1.41"
globalFinaleDolet33 = True
# Falls globalFinaleDolet33 == True:
# Akkordsymbole werden anders exportiert, mehrzeilige Einfachtexte in einem einzigen Kästchen.
# Sonst (Export nach Sibelius oder nach Finale mit Dolet vor Version 3.3): Mehrzeilige Einfachtexte
# werden mit einem Kästchen für jede Zeile exportiert.
globalChordCaseMatters = True
# Falls globalChordCaseMatters = True:
# Akkordsymbole mit kleinen Anfangsbuchstaben werden als Moll-Akkorde interpretiert.
globalExportToSibelius = False
# Falls globalExportToSibelius == True:
# vollständig leere Takte führen bei Sibelius 4 zu "error opening the file" und Importabbruch.
# Das wird durch Export einer Viertelpause vermieden
# Sibelius 4 unterstützt keine Unsichtbarkeit bei Noten oder Pausen. Deshalb wird dann
# ein forward-Element eingefügt (das erscheint in Finale als unsichtbare Pause)
globalPagesizeXmm = 210.0 # A4
globalPagesizeYmm = 297.0 # A4

language = 1 # default: deutsch
# L_German=1,L_Finnish=3,L_Swedish=4,L_EnglishUS=5,L_EnglishGB=6,L_Dutch=7,
#         L_Czech=8,L_Polish=9

def fillTextbasket(language):
    # unabhängig von _Localizer aus cap-obj.py!
    global textbasket
    textbasket = {}
    if language == 1: # deutsch
        textbasket["errorXMLnode"] = "Fehler beim Analysieren des XML-Knotens...\n"
        textbasket["elementOf"] = ". "" Element von...\n"
        textbasket["warnGeneral"] = "CapToMusic Warnung"
        textbasket["warnLayout"] = "Fehler in Layoutanalyse "
        textbasket["braceExpected"] = ": Geschweifte Klammer erwartet"
        textbasket["fileFilter"] = "MusicXML-Dateien (*.xml)"
        textbasket["errorConversionTitle"] = "CapToMusic Fehler beim Konvertieren"
        textbasket["errorConversionLocation"] = "System %d Takt %d Zeile %s"
        textbasket["errorReadingTitle"] = "CapToMusic Fehler beim Lesen"
        textbasket["errorReadingDetails"] = " von %s\n%s"
        textbasket["truncated"] = "Konvertierung abgebrochen"
        textbasket["errorCommandline"] = "Fehlerhafte Kommandozeile: "
        textbasket["usage"] = "Erwartet: CapToMusic.py -l Sprache -f DateinameOhneErweiterung -o ZielDatei\
            (Sprachen: 11 Französisch - 5 Englisch (US) - 1 Deutsch)"
        textbasket["srcFileType"] = "CapXML Dateien"
    elif language == 11: # französisch
        textbasket["errorXMLnode"] = "Erreur dans l'analyse du nœud xml ...\n"
        textbasket["elementOf"] = ". "" élément du...\n"
        textbasket["warnGeneral"] = "Alerte CapToMusic"
        textbasket["warnLayout"] = "Erreur dans l'analyse de modèle de système "
        textbasket["braceExpected"] = ": Accolade systeme attendue"
        textbasket["fileFilter"] = "Fichiers MusicXML (*.xml)"
        textbasket["errorConversionTitle"] = "CapToMusic erreur de conversion"
        textbasket["errorConversionLocation"] = "Système %d mesure %d portée %s"
        textbasket["errorReadingTitle"] = "CapToMusic erreur de lecture"
        textbasket["errorReadingDetails"] = " du %s\n%s"
        textbasket["truncated"] = "Conversion inachevée"
        textbasket["errorCommandline"] = "Ligne de commande erronée: "
        textbasket["usage"] = "Attendu: CapToMusic.py -l langue -f NomDeFichierSansSuffix -o FichierDestination\
             (Langues: 11 français - 5 anglais (US) - 1 allemand)"
        textbasket["srcFileType"] = "Fichiers CapXML"
    else: # englisch (US)
        textbasket["errorXMLnode"] = "error parsing xml node...\n"
        textbasket["elementOf"] = ". "" element of...\n"
        textbasket["warnGeneral"] = "CapToMusic warning"
        textbasket["warnLayout"] = "Error in layout analysis "
        textbasket["braceExpected"] = ": Brace expected"
        textbasket["fileFilter"] = "MusicXML files (*.xml)"
        textbasket["errorConversionTitle"] = "CapToMusic conversion error"
        textbasket["errorConversionLocation"] = "in system %d measure %d staff %s"
        textbasket["errorReadingTitle"] = "CapToMusic error reading file"
        textbasket["errorReadingDetails"] = " %s\n%s"
        textbasket["truncated"] = "output file truncated"
        textbasket["errorCommandline"] = "Wrong command line: "
        textbasket["usage"] = "Expected: CapToMusic.py -l Language -f FilenameWithoutExtension -o DestinationFile\
             (Languages: 11 French - 5 English (US) - 1 German)"
        textbasket["srcFileType"] = "CapXML files"
    
def myTr(key):
    global textbasket
    return textbasket[key]

fillTextbasket(language)

import caplib.capDOM, zipfile, tempfile, sys, time, xml.dom.minidom, copy
try:
    import tkMessageBox, tkFileDialog
    doitwithTK = True
except:
    doitwithTK = False
from caplib.rational import Rational
import Rtf2TxtAndFont, Rtf2Txt, array
import os
import locale

class MyScriptOptions (object):
    def __init__(self):
        try:
            import _winreg
            self.mykey = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER,r"Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders")
            userDataPath,type = _winreg.QueryValueEx(self.mykey,"AppData")
            userDataPath += r"\capella-software\CapToMusic"
            try:
                os.makedirs(userDataPath)
            except OSError, (errno, strerror):
                if errno == 17: # file exists
                    pass
                else:
                    raise
            self._optionsFile = os.path.join(userDataPath,"CapToMusic.opt")
        except:
            dir, file = os.path.split(sys.argv[0])
            file = os.path.splitext(file)[0] + '.opt'
            self._optionsFile = os.path.join(dir, file)

    def get(self):
        opt = dict()
        try:                                          # Beispiel
            l = file(self._optionsFile).readlines()   # ['a = 7', 'b = true']
            for x in l:
                a = [s.strip() for s in x.split('=')] # ['a', '7']
                if len(a) == 2:
                    opt[a[0]] = a[1]
        except: pass
        return opt

class KGV (object):
    def __init__(self):
        self.primes = [2,3,5,7,11,13,17]
        self.primeExponents = [0,0,0,0,0,0,0]
    def add(self, nn):
        n = nn
        for i in range(len(self.primes)):
            exp = self.primeExponents[i]
            prime = self.primes[i]
            while n != 1 and n % prime == 0:
                exp -= 1
                n = n / prime
            if exp < 0:
                self.primeExponents[i] -= exp
        if n != 1:
            raise "illegal note length ?/"+str(nn)
    def getkgv(self):
        kgv = 1
        for i in range(len(self.primes)):
            kgv *= self.primes[i] ** self.primeExponents[i]
        return kgv

def utf8(s):
    """ geändert 2005-05-06:
        Jetzt kombiniert mit Ersetzung der Zeichen
        & < " durch Entity-Referenzen.
    """
    return '&amp;'.join([x.replace('"', '&quot;').replace('<','&lt;') for x in s.split('&')]).encode('utf-8')
                         
def latin1simple(s):
    return s.encode('Latin-1')

def myNext(iter):
    try:
        return iter.next()
    except StopIteration:
        return None

def LastIndexLowerEqual(array, Value):
    Sum = 0
    Index = 0
    while Index < len(array):
        Sum += array[Index]
        if Sum > Value:
            break
        Index += 1
    return Index
#-----------------------------------------------------------------------
    
#-----------------------------------------------------------------------

def pagesizeMm():
    global globalPagesizeXmm
    global globalPagesizeYmm
    return globalPagesizeXmm, globalPagesizeYmm

def typeFromDenominator(durRat, denom):
    numerator = durRat.p
    type = ""
    if denom == 4:
        type = "quarter"
    elif denom == 8:
        type = "eighth"
    elif denom == 16:
        type = "16th"
    elif denom == 2:
        type = "half"
    elif denom == 32:
        type = "32nd"
    elif denom == 64:
        type = "64th"
    elif denom == 1:
        if durRat < 2:
            type = "whole"
        elif durRat < 4:
            type = "breve"
    elif denom == 128:
        type = "128th"
    elif denom == 256:
        type = "256th"
    return type

def beamsFromDenominator(denom):
    beams = 0
    if denom == 8:
        beams = 1
    elif denom == 16:
        beams = 2
    elif denom == 32:
        beams = 3
    elif denom == 64:
        beams = 4
    elif denom == 128:
        beams = 5
    elif denom == 256:
        beams = 6
    return beams

def getMXMLdynamics(capella3str):
    if capella3str == "f":
        return "<f/>"
    elif capella3str == "g":
        return "<ff/>"
    elif capella3str == "h":
        return "<fff/>"
    elif capella3str == "j":
        return "<mf/>"
    elif capella3str == "p":
        return "<p/>"
    elif capella3str == "q":
        return "<pp/>"
    elif capella3str == "r":
        return "<ppp/>"
    elif capella3str == "i":
        return "<mp/>"
    elif capella3str == "s":
        return "<sf/>"
    elif capella3str == "z":
        return "<sfz/>"
    elif capella3str == "{":
        return "<fz/>"
    elif capella3str == "|":
        return "<fp/>"
    return ""

MusicXMLDynamicsVals = {"<f/>":"111","<ff/>":"122","<fff/>":"133",
                        "<mf/>":"100","<p/>":"78","<pp/>":"67",
                        "<ppp/>":"56","<mp/>":"89",
                        "<sf/>":"","<sfz/>":"","<fz/>":"","<fp/>":""}

def isSegno1(capella3str):
    return capella3str == "y"
    
def isSegno2(capella3str):
    return capella3str == "$"
    
def isCodaLarge(capella3str):
    return capella3str == "n"
    
def isCodaSmall(capella3str):
    return capella3str == "o"

def isPedalStart(capella3str):
    return capella3str == "a"

def isPedalStop(capella3str):
    return capella3str == "b"

def isFermataAbove(capella3str):
    return capella3str == "u"

def isFermataBelow(capella3str):
    return capella3str == "k"

def isAufstrich(capella3str):
    return capella3str == "Z"

def isAbstrich(capella3str):
    return capella3str == "Y"

def isMXMLornament(capella3str):       # tr
    return capella3str == "t" or capella3str == "l" or capella3str == "x" or capella3str == "w"

def isTrill(capella3str):       # tr
    return capella3str == "t"

def isPrallTrill(capella3str):  # inverted-mordent
    return capella3str == "l"

def isMordent(capella3str):
    return capella3str == "x"

def isDoppelschlag(capella3str):    # turn
    return capella3str == "w"

def isFaulenzer1(capella3str):
    return capella3str == "W"

def isFaulenzer2(capella3str):
    return capella3str == "X"

def getNtabs(N):
    retstr = ""
    for i in range(N):
        retstr += "\t"
    return retstr

def isNoteType(capella3str):
    capella3str = capella3str.strip()
    if capella3str == u"\246": # 16telnote Hals oben
        return "16th"
    if capella3str == u"\244": # Achtelnote Hals oben
        return "eighth"
    if capella3str == u"\242": # Viertelnote Hals oben
        return "quarter"
    if capella3str == u"\240" or capella3str == u"\256": # Halbe Note Hals oben
        return "half"
    if capella3str == u"\343": # Ganze Note
        return "whole"
    return ""

def Capella3ToMaestro(string):
    retstr = ""
    for char in string:
        if char == u"\246": # 16telnote Hals oben
            retstr += "x"
        elif char == u"\244": # Achtelnote Hals oben
            retstr += "e"
        elif char == u"\242": # Viertelnote Hals oben
            retstr += "q"
        elif char == u"\240" or char == u"\256": # Halbe Note Hals oben
            retstr += "h"
        elif char == u"\343": # Ganze Note
            retstr += "w"
        elif isSegno1(char):
            retstr += "%"
        else:
            retstr += char
    return retstr

def isTranslatable(TextContent):
    for char in TextContent:
        if 0 <= "0123456789=QS#de".find(char):
            return True
    return False

def Capella3ToText(string): # Für Akkordsymbole
    retstr = ""
    for char in string:
        if char == "Q": 
            retstr += "b"
        elif char == "S":
            retstr += "#"
        elif char == "d":
            retstr += "D.C."
        elif char == "e":
            retstr += "D.S."
        else:
            retstr += char
    return retstr
        
def TextWidthMXML(fontsize, bold, text, fontCharSet):
    # todo: distinguish uppercase/lowercase
    if fontCharSet == "204":
        if bold:
            fac = 1.15
        else:
            fac = 1.00
    else:
        if bold:
            fac = 0.97
        else:
            fac = 0.88
    return int(len(text) * fontsize * fac)

def TextHeightMXML(fontsize):
    return int(fontsize * 10 / 4)

def cShape2mShape(shape, long):
    if shape == "diamond":
        return "diamond",""
    elif shape == "crossCircle":
        if long:
            return "circle-x",""
        else:
            return "x",""
    elif shape == "crossDiamond":
        if long:
            return "diamond",""
        else:
            return "x",""
    elif shape == "triangle":
        return "triangle",""
    elif shape == "square":
        return "square",""
    elif shape == "none":
        return "none",""
    elif shape == "openDiamond":
        if long:
            return "diamond",""
        else:
            return "diamond","filled=\"no\""
    elif shape == "crossCircle2":
        return "circle-x",""
    elif shape == "openTriangle":
        if long:
            return "triangle",""
        else:
            return "triangle","filled=\"no\""
    elif shape == "openSquare":
        if long:
            return "square",""
        else:
            return "square","filled=\"no\""
    elif shape == "thickCross":
        return "cross",""
        
        
#---------------------------------------------------------------
def expatbuilderStateString(builder):
    str = ""
    n = builder.curNode
    if n.nodeType == n.ELEMENT_NODE:
        str = myTr("errorXMLnode")
    return str + parentNodeStateString(n)

def parentNodeStateString(n):
    if n.parentNode:                
        strg = ""
        i = 0
        if n.nodeType == n.ELEMENT_NODE:
            strg += n.tagName
        else:
            strg += str(n.nodeType)
        for sibl in n.parentNode.childNodes:
            if sibl.nodeType == sibl.ELEMENT_NODE:
                if sibl == n:
                    break
                i += 1
        strg += ", "+str(i+1)+myTr("elementOf")
        return strg + parentNodeStateString(n.parentNode)
    else:
        return "root"

class MusicXMLExport (caplib.capDOM.ScoreExport):
    def __init__(self, inputFile, outputFile):
        # überschrieben, um self.zipfile verfügbar zu machen
        time1 = time.time()
        self.success = 0
        
        try:
            self.zipfile = zipfile.ZipFile(inputFile, 'r')
            text = self.zipfile.read('score.xml')
            time2 = time.time()
#            self.doc = xml.dom.minidom.parseString(text)
#   wir machen dasselbe, wollen aber später das builder-Objekt ansprechen können:
            import xml.dom.expatbuilder
            builder = xml.dom.expatbuilder.ExpatBuilderNS()
            self.doc = builder.parseString(text)
        except:
            self.parseError = expatbuilderStateString(builder)
            self.success = 4
            self.zipfile.close()
            return
        
        time3 = time.time()
        self.LyricsCharset = "0"
        # Initialisierung für Fehlerexit
        self.iSystem = 0
        self.MeasureCount = -1
        self.layoutsInPart = [""]

        try:
            self.f = file(outputFile, 'wt')
            for n in self.doc.childNodes:
                if self.isElement(n, 'score'):
                    xmlns = n.getAttribute("xmlns")
                    self.isCAPX2 = 0 > xmlns.find("1.0")
                    self.handleScore(n)
            self.f.close()
        except:
            self.success = 5

        self.zipfile.close()
        time4 = time.time()

#        myMessage("CapToMusic",outputFile+" written. Read time"+str(time2 - time1)+"sec, parse time "+str(time3-time2)+"sec, translate time "+str(time4-time3)+"sec")


# Klassen-Variablen:
# von außen steuerbar:
# self.ExportLineBreak
# intern:
# self.Standalone
# self.isFirstSystem
# self.System1MarginWritten
# self.iSystem
# self.changedAttrClef[StavesInPart]  (gilt für einen part)
# self.lastPrintState[StavesInPart]  (gilt für einen part)
# self.changedAttrKey  (gilt für einen part)
# self.changedAttrTime  (gilt für ein System)
# self.measureRepeat (Faulenzer)
# self.Clefs[StavesInPart] (sign,line,octaveChange)
# self.arDivisions[nSysteme+1]    480
# self.durationsum[nVoices]     seit dem letzten Taktstrich
# self.durationsumSystem        seit dem letzten Systemumbruch
# self.explicitBar[nVoices]
# self.indentlevel
# self.Key  (fifths,mode)
# self.MeasureCount     (Index des aktuellen Taktes)
# self.MeasureNumber    (Index des aktuellen Taktes der Taktnummern definierenden Stimme)
# self.MeasureNumberSysOffset
# self.HideBarCount
# self.nVoices
# self.part {}
# self.partIndex
# self.ScorePartIDs[]   (dictionary of staffLayout descriptions, part id, transpose, staveLineCount)
# self.TemplateIndex {}
# self.Time[StavesInSystem] (beats,beatType)
# self.beamGrouping
# self.outbeamStarted
# self.staffIndSystem[nVoices]
# self.staffNoPart[nVoices]
# self.DivisionsWritten (gilt für aktuellen part, wird für jedes System kontrolliert)
# self.Divisions (die aktuell gültigen divisions)
# self.StavesInPart (Anzahl der Zeilen, die im aktuellen System vorkommen und zu diesem Part gehören)
# self.layoutsInPart (Für aktuellen MusicXML-part die capella-layouts, die dazugehören)
# self.nextDefaultChannel (nur innerhalb handleLayout())
# self.pagesizeY (in Zehnteln des Linienabstandes)
# self.pagesizeX
# self.lyricsDist = [nVoices]
# self.lyricsFirstLine = [nVoices]
# self.stemDir = [nVoices]
# self.alterPending = [nVoices]
# self.MaxSlurNo = []
# self.SlurPending = [iVoice]
# self.MaxTupletNo = [iVoice]
# self.TupletPending = [iVoice]
# self.MaxWedgeNo = [iVoice]
# self.WedgePending = [iVoice]
# self.MaxBracketNo = [iVoice]
# self.BracketPending = [iVoice]
# self.EndingPending 
# self.EndingNumber: Effektivstring, der in capella unter der Voltenklammer erscheint 
# self.marginSys1
# self.marginShort
# self.marginLong
# self.ydistSys1
# self.StaffDist[StavesInCapellaTemplate] (top,bottom)
# self.StaffDistOffset[StavesInSystem] (top,bottom)   capella system
# self.previousStaffDistOffset[StavesInSystem] (top,bottom)   capella system
# self.StavesInSystem[StavesInSystem] (layoutstring)    
# self.DefaultSystemDistance
# self.BarlineSaved (barline object)
# self.VoltaSaved (drawObj volta)
# self.TimeSignSaved (timeSign object)
# self.LyricsCharset
# self.stavesmissing[Staves in part] (gezählt im capella-Mustersystem)
# self.leftIndent
# self.rightIndent
# self.pageBreakPending
# self.chordstyle
# self.nonControllingSuffix (MeasureNumber, "A")
# self.DefaultDurationsDict {self.iSystem: (DefaultDurationss, DurationSums, RestsOnly, SpecialBarss)}
# self.success
# self.transposeNotWritten
# self.staveLineCountNotWritten
# self.staveLineCount

        
    def getFirstChildElement(self, el, childTag):
        """erstes Element innerhalb el (alle Stufen) mit dem Namen <childTag>"""
        for n in el.childNodes:
            if n.nodeType == n.ELEMENT_NODE and n.tagName == childTag:
                return n
        return None

    def getDirectChildrenByTagName(self, obj, tag):
        list = []
        for n in obj.childNodes:
            if self.isElement(n, tag):
                list.append(n)
        return list

    def getDirectChildrenBy2TagNames(self, obj, tag1, tag2):
        list = []
        for n in obj.childNodes:
            if self.isElement(n, tag1) or self.isElement(n, tag2):
                list.append(n)
        return list

    def decodeFromCharset(self, uString, charset):
        # capella strings erscheinen in Python als unicode, sind aber nach
        # charsets/codepages codiert.
        plainarray = array.array("c")
        for c in uString:
            if ord(c) < 256:
                plainarray.append(chr(ord(c)))
        plainstring = plainarray.tostring()
        return plainstring.decode(self.CodecFromCharset[charset])

    def writeNewline(self, s):
        i=0
        while i<self.indentlevel:
            self.f.write('\t')
            i += 1
        self.f.write(s)

    def writeSimpleElement(self,name,value):
        self.writeNewline("<")
        self.f.write(name)
        self.f.write(">")
        self.f.write(value)
        self.f.write("</")
        self.f.write(name)
        self.f.write(">\n")

    def writeElementWithAttributeAndChild(self,name,attribute,child):
        self.writeNewline("<")
        self.f.write(name)
        if attribute != "":
            self.f.write(" ")
            self.f.write(attribute)
        if child == "":
            self.f.write("/>\n")
        elif child[0:1] == "<":
            # child is element: new line
            self.f.write(">\n")
            self.indentlevel += 1
            self.writeNewline(child + "\n")
            self.indentlevel -= 1
            self.writeNewline("</")
            self.f.write(name)
            self.f.write(">\n")
        else:
            # child is text: same line
            self.f.write(">")
            self.f.write(child)
            self.f.write("</")
            self.f.write(name)
            self.f.write(">\n")

    def writeDirection( self, above, element, attribute, child, iVoice):
        self.writeNewline("<direction")
        if above != "":
            self.f.write(" placement=\""+above+"\"")
        self.f.write(">\n")
        self.indentlevel += 1
        self.writeNewline("<direction-type>\n")
        self.indentlevel += 1
        self.writeElementWithAttributeAndChild(element, attribute, child)
        self.indentlevel -= 1
        self.writeNewline("</direction-type>\n")
        if self.StavesInPart > 1:
            self.writeSimpleElement("staff",str(self.staffNoPart[iVoice]))
        if element == "dynamics":
            val = MusicXMLDynamicsVals[child]
            if val != "":
                self.writeElementWithAttributeAndChild("sound", "dynamics=\""+val+"\"", "")
        self.indentlevel -= 1
        self.writeNewline("</direction>\n")

    def writeDirectionMultiline( self, above, element, attribute1, attribute2, child, iVoice, yStart, iLineHeight):
        self.writeNewline("<direction")
        if above != "":
            self.f.write(" placement=\""+above+"\"")
        self.f.write(">\n")
        self.indentlevel += 1
        self.writeNewline("<direction-type>\n")
        self.indentlevel += 1
        LineIndex = 0
        NewLineIndex = 0
        nextend = child[NewLineIndex:].find("\n")
        while nextend > 0:
            endThisLine = NewLineIndex + nextend
            childThisLine = child[NewLineIndex:endThisLine]
            attributeThisLine = attribute1+str(yStart - LineIndex * iLineHeight)+attribute2
            self.writeElementWithAttributeAndChild(element, attributeThisLine, childThisLine)
            NewLineIndex = endThisLine+1
            nextend = child[NewLineIndex:].find("\n")
            LineIndex += 1
        attributeThisLine = attribute1+str(yStart - LineIndex * iLineHeight)+attribute2
        self.writeElementWithAttributeAndChild(element, attributeThisLine, child[NewLineIndex:])
        self.indentlevel -= 1
        self.writeNewline("</direction-type>\n")
        if self.StavesInPart > 1:
            self.writeSimpleElement("staff",str(self.staffNoPart[iVoice]))
        self.indentlevel -= 1
        self.writeNewline("</direction>\n")

    def writeElementWithAttribute(self,name,attribute,value):
        self.writeNewline("<")
        self.f.write(name)
        self.f.write(" ")
        self.f.write(attribute)
        self.f.write(">")
        self.f.write(value)
        self.f.write("</")
        self.f.write(name)
        self.f.write(">\n")

    def handleSound(self, sound, MusicXMLID, usedChannels):
        midiprog = sound.getAttribute("instr")
        volume = sound.getAttribute("volume")
        channel = sound.getAttribute("preferredChannel")
        self.writeNewline("<score-instrument id=\"")
        MusicXMLInstrumentID = MusicXMLID + "-I1"
        self.f.write(MusicXMLInstrumentID)
        self.f.write("\">\n")
        self.indentlevel += 1
        self.writeSimpleElement("instrument-name","Midi_"+str(int(midiprog)+1))
        self.indentlevel -= 1
        self.writeNewline("</score-instrument>\n")
        self.writeNewline("<midi-instrument id=\"")
        self.f.write(MusicXMLInstrumentID)
        self.f.write("\">\n")
        self.indentlevel += 1
        # Finale will ausdrückliche channel-Angabe:
        if channel != "":
            intchan = int(channel)
            while intchan <= 16 and intchan in usedChannels:
                intchan += 1
            channel = str(intchan)
            usedChannels[intchan] = True
        else:
            intchan = self.nextDefaultChannel
            while intchan <= 16 and intchan in usedChannels:
                intchan += 1
            channel = str(intchan)
            usedChannels[intchan] = True
            if intchan < 16:
                self.nextDefaultChannel = intchan + 1
        # Wenn's keine freien mehr gibt, offen lassen. Spätestens mit MusicXML 2.0 erlaubt.
        if intchan <= 16:
            self.writeSimpleElement("midi-channel",channel)
        self.writeSimpleElement("midi-program",str(int(midiprog)+1))
        self.indentlevel -= 1
        self.writeNewline("</midi-instrument>\n")
        
    def handleLayout(self, layout, firstNameStyle, othersNameStyle):
        self.ScorePartIDs = []
        staves = self.getFirstChildElement(layout,"staves")
        # auskommentierte Variante ist wahrscheinlich ineffektiver
        # staffLayouts = staves.getElementsByTagName("staffLayout")
        staffLayouts = self.getDirectChildrenByTagName(staves, "staffLayout")
        self.writeNewline("<part-list>\n")
        self.indentlevel += 1
        countParts = 0
        countMXMLParts = 0
        countPartGroups = 0
        PartGroupsFrom = {}
        PartGroupsTo = {}
        usedChannels = {}
        self.TemplateIndex = {}
        self.nextDefaultChannel = 1
        brackets = layout.getElementsByTagName("bracket")
        for bracket in brackets:
            countPartGroups += 1
            brfrom = int(bracket.getAttribute("from"))+1
            brto = int(bracket.getAttribute("to"))+1
            # capella zählt ab 0!
            brcurly = bracket.getAttribute("curly")
            # hier fehlt was in capella! Es geht auch noch "line".
            PartGroupsFrom[brfrom] = (countPartGroups, brcurly)
            PartGroupsTo[brto] = (countPartGroups, brcurly)
            
        parts = []
        multistaffPart = False
        for staffLayout in staffLayouts:
            instrument = self.getFirstChildElement(staffLayout,"instrument")
            strID = staffLayout.getAttribute("description")
            parts.append(strID)
            self.TemplateIndex[strID] = countParts
            countParts += 1
            # Systemklammern?
            if countParts in PartGroupsFrom:
                bracketNo, brcurly = PartGroupsFrom[countParts]
                if brcurly == "true":
                    # wir nehmen alle zugehörigen Zeilen in einen part
                    # Als Namen nehmen wir möglichst "pairName" der ersten
                    # Zeile, als sound nehmen wir den von der letzten Zeile
                    multistaffPart = True
                    multistaffpairName = instrument.getAttribute("pairName")
                    multistaffpairAbbrev = instrument.getAttribute("pairAbbrev")
                    if multistaffpairName == "":
                        multistaffpairName = instrument.getAttribute("name")
                    if multistaffpairAbbrev == "":
                        multistaffpairAbbrev = instrument.getAttribute("abbrev")
                else:
                    # wir behandeln alle zugehörigen Zeilen als part-group
                    self.writeNewline("<part-group type=\"start\" number=\"")
                    self.f.write(str(bracketNo))
                    self.f.write("\">\n")
                    self.indentlevel += 1
                    symbol = "bracket"
                    pairName = instrument.getAttribute("pairName")
                    pairAbbrev = instrument.getAttribute("pairAbbrev")
                    if pairName != "":
                        self.writeSimpleElement("group-name",utf8(pairName))
                    if pairAbbrev != "":
                        self.writeSimpleElement("group-abbreviation",utf8(pairAbbrev))
                    self.writeSimpleElement("group-symbol",symbol)
                    self.indentlevel -= 1
                    self.writeNewline("</part-group>\n")
            if multistaffPart:
                if countParts in PartGroupsTo:
                    # Letzte Zeile für diesen part
                    bracketNo, brcurly = PartGroupsTo[countParts]
                    if brcurly != "true":
                        # Fehler in Layoutanalyse
                        myMessage(myTr("warnGeneral"), myTr("warnLayout")+multistaffpairName+myTr("braceExpected"))
                        break
                    partname = multistaffpairName
                    abbrevname = multistaffpairAbbrev
                else:
                    continue
            else:
                partname = instrument.getAttribute("name")
                abbrevname = instrument.getAttribute("abbrev")
            # Schlagzeug?
            lineCount = self.getLineCountFromStaffLayout(staffLayout)
                
            countMXMLParts += 1
            self.writeNewline("<score-part id=\"")
            MusicXMLID = "P"+str(countMXMLParts)
            self.f.write(MusicXMLID)
            self.f.write("\">\n")
            self.indentlevel += 1
            self.writeNewline("<part-name>")
            if firstNameStyle == "long":
                self.f.write(utf8(partname))
            elif firstNameStyle == "short":
                self.f.write(utf8(abbrevname))
            self.f.write("</part-name>\n")
            self.writeNewline("<part-abbreviation>")
            if othersNameStyle == "long":
                self.f.write(utf8(partname))
            elif othersNameStyle == "short":
                self.f.write(utf8(abbrevname))
            self.f.write("</part-abbreviation>\n")
            sound = self.getFirstChildElement(staffLayout,"sound")
            transpose = 0
            if sound != None:
                self.handleSound(sound, MusicXMLID, usedChannels)
                tr = sound.getAttribute("transpose")
                if tr != "":
                    transpose = int(tr)
            self.ScorePartIDs.append((parts,MusicXMLID, transpose, lineCount))
            parts = []
            self.indentlevel -= 1
            self.writeNewline("</score-part>\n")
            if countParts in PartGroupsTo:
                bracketNo, brcurly = PartGroupsTo[countParts]
                if brcurly == "true":
                    multistaffPart = False
                else:
                    self.writeNewline("<part-group type=\"stop\" number=\"")
                    self.f.write(str(bracketNo))
                    self.f.write("\"/>\n")
                
        self.indentlevel -= 1
        self.writeNewline("</part-list>\n")

    def handleDrawObjAsCredit(self, n):
        # return: Es ist ein credit-Element geschrieben worden
        basic = self.getFirstChildElement(n,"basic")
        if basic != None:
            firstPage = basic.getAttribute("firstPage")
            oddPagesExceptFirst = basic.getAttribute("oddPagesExceptFirst")
            evenPages = basic.getAttribute("evenPages")
            if firstPage == "false" or oddPagesExceptFirst == "true" or evenPages == "true":
                return False
        pagetext = self.getFirstChildElement(n, "text")
        if pagetext != None:
            self.handleTextAsCredit(pagetext)
            return True
        pagetext = self.getFirstChildElement(n, "richText")
        if pagetext != None:
            self.handleRichText(pagetext, True, -1)
            return True
        return False

    def handleTextAsCredit(self, pagetext):
        xStr = pagetext.getAttribute("x")
        x = int(float(xStr) * 10.0)
        yStr = pagetext.getAttribute("y")
        y = int(float(yStr) * 10.0)
        x = int(x * self.pageObjLineDist / self.normalLineDist)
        y = int(y * self.pageObjLineDist / self.normalLineDist)
        # y-Referenz ist Basislinie des Textes.
        # <credit-words> haben den unteren Rand von gjpqy als Referenz
        align = pagetext.getAttribute("align")
        # left, center, right in CapXML genau wie in MusicXML
        font = self.getFirstChildElement(pagetext, "font")
        fontheight = font.getAttribute("height")
        fontlower = TextHeightMXML(float(fontheight))/5
        fontface = font.getAttribute("face")
        fontweight = font.getAttribute("weight")
        fontitalic = font.getAttribute("italic")
        fontunderline = font.getAttribute("underline")
        fontcolor = font.getAttribute("color")
        content = self.getFirstChildElement(pagetext, "content")
        for n in content.childNodes:
            if n.nodeType == n.TEXT_NODE:
                self.writeNewline("<credit>\n")
                self.indentlevel += 1
                position = "default-x=\""+str(x + self.pagemarginLeft)+"\" default-y=\""+str(self.pagesizeY - y - self.pagemarginTop - fontlower)+"\""
                if fontface != "":
                    position += " font-family=\""+fontface+"\""
                position += " font-size=\""+fontheight+"\""
                if fontweight != "":
                    if int(fontweight) > 400:
                        position += " font-weight=\"bold\""
                if fontitalic != "":
                    position += " font-style=\"italic\""
                if align != "":
                    position += " justify=\""+align+"\""
                self.writeElementWithAttribute("credit-words", position, str(utf8(n.data)))
                self.indentlevel -= 1
                self.writeNewline("</credit>\n")

    def isTextChunkWhitespace(self, textchunk):
        pureText = self.getPureTextFromChunk(textchunk)
        return '' == pureText.strip()

    def getPureTextFromChunk(self, textchunk):
        indexStart = 0
        indexEnd = textchunk.find(";")
        if indexEnd < 0:
            return ""
        indexStart = indexEnd + 1
        indexEnd = indexStart + textchunk[indexStart:].find(";")
        if indexEnd < 0:
            return ""
        indexStart = indexEnd + 1
        indexEnd = indexStart + textchunk[indexStart:].find(";")
        if indexEnd < 0:
            return ""
        indexStart = indexEnd + 1
        indexEnd = indexStart + textchunk[indexStart:].find(";")
        if indexEnd < 0:
            return ""
        indexStart = indexEnd + 1
        indexEnd = indexStart + textchunk[indexStart:].find(";")
        if indexEnd < 0:
            return ""
        indexStart = indexEnd + 1
        indexEnd = indexStart + textchunk[indexStart:].find(";")
        indexStart = indexEnd + 1
        return textchunk[indexStart:]

    def getTextFromChunk(self, textchunk):
        fontname = ""
        fontsize = 0
        italic = False
        bold = False
        aligntype = "l"
        charset = "0"
        
        indexStart = 0
        indexEnd = textchunk.find(";")
        if indexEnd < 0:
            return "", fontname, fontsize, italic, bold, aligntype, charset
        fontname = textchunk[indexStart:indexEnd]
        indexStart = indexEnd + 1
        indexEnd = indexStart + textchunk[indexStart:].find(";")
        if indexEnd < 0:
            return "", fontname, fontsize, italic, bold, aligntype, charset
        fontsize = int(textchunk[indexStart:indexEnd])
        indexStart = indexEnd + 1
        indexEnd = indexStart + textchunk[indexStart:].find(";")
        if indexEnd < 0:
            return "", fontname, fontsize, italic, bold, aligntype, charset
        if indexStart < indexEnd:
            italic = "i" == textchunk[indexStart:indexEnd]
        indexStart = indexEnd + 1
        indexEnd = indexStart + textchunk[indexStart:].find(";")
        if indexEnd < 0:
            return "", fontname, fontsize, italic, bold, aligntype, charset
        if indexStart < indexEnd:
            bold = "b" == textchunk[indexStart:indexEnd]
        indexStart = indexEnd + 1
        indexEnd = indexStart + textchunk[indexStart:].find(";")
        if indexEnd < 0:
            return "", fontname, fontsize, italic, bold, aligntype, charset
        if indexStart < indexEnd:
            if "c" == textchunk[indexStart:indexEnd]:
                aligntype = "c"
            elif "r" == textchunk[indexStart:indexEnd]:
                aligntype = "r"
        indexStart = indexEnd + 1
        indexEnd = indexStart + textchunk[indexStart:].find(";")
        if indexStart < indexEnd:
            charset = textchunk[indexStart:indexEnd]
        indexStart = indexEnd + 1
        return textchunk[indexStart:], fontname, fontsize, italic, bold, aligntype, charset

    def getMetronomeVals(self, textchunk):
        beatsStr = ""
        parentheseIndex = -1
        PureText = self.getPureTextFromChunk(textchunk)
        i = PureText.find("=")
        if 0 <= i:
            # vorsicht: es kann auch noch eine schließende Klammer folgen...
            parentheseIndex = PureText[i+1:].find(")")
            if parentheseIndex > 0:
                beatsStr = PureText[i+1:i+1+parentheseIndex]
            else:
                beatsStr = PureText[i+1:]
        beatsStr = beatsStr.rstrip()
        stripped = beatsStr.lstrip()
        if stripped.isdigit():
            beats = int(stripped)
        else:
            # hole die erste Integer-Zahl aus dem string.
            lastIndex = len(stripped)
            startIndexNumber = 0
            while startIndexNumber < lastIndex and not stripped[startIndexNumber:startIndexNumber+1].isdigit():
                startIndexNumber += 1
            stopIndexNumber = startIndexNumber
            while stopIndexNumber < lastIndex and stripped[stopIndexNumber:stopIndexNumber+1].isdigit():
                stopIndexNumber += 1
            if stopIndexNumber > startIndexNumber:
                beats = int(stripped[startIndexNumber:stopIndexNumber])
            else:
                beats = 0
        return beats, beatsStr, parentheseIndex >= 0

    def handleMetronome(self, where, x, y, beats, beatsString, NoteType, parentheses):
        self.writeNewline("<direction placement=\""+where+"\">\n")
        self.indentlevel += 1
        self.writeNewline("<direction-type>\n")
        self.indentlevel += 1
        position = "relative-x=\""+str(x)+"\" default-y=\""+str( -y )+"\""
        if parentheses:
            self.writeNewline("<metronome parentheses=\"yes\" "+position+">\n")
        else:
            self.writeNewline("<metronome parentheses=\"no\" "+position+">\n")
        self.indentlevel += 1
        self.writeNewline("<beat-unit>"+NoteType+"</beat-unit>\n")
        self.writeNewline("<per-minute>"+beatsString+"</per-minute>\n")
        self.indentlevel -= 1
        self.writeNewline("</metronome>\n")
        self.indentlevel -= 1
        self.writeNewline("</direction-type>\n")
        if NoteType == "quarter":
            outbeats = beats
        elif NoteType == "half":
            outbeats = beats / 2
        elif NoteType == "whole":
            outbeats = beats /4
        elif NoteType == "eighth":
            outbeats = beats * 2
        elif NoteType == "16th":
            outbeats = beats * 4
        self.writeTempo([outbeats])
        self.indentlevel -= 1
        self.writeNewline("</direction>\n")

    def writeTempo(self,outbeats):
        if len(outbeats) > 1:
            for i in range(len(outbeats)):
                if outbeats[i] > 0:
                    self.writeNewline("<sound tempo=\""+str(outbeats[i])+"\" time-only=\""+str(i+1)+"\"/>\n")
                break # Finale und Finale Notepad kommen höchstens mit einem sound-Element klar (BJ120604)
        else:
            self.writeNewline("<sound tempo=\""+str(outbeats[0])+"\"/>\n")

    def writeTempoDirection(self,outbeats):
        self.writeNewline("<direction>\n")
        self.indentlevel += 1
        self.writeElementWithAttributeAndChild("direction-type","","<other-direction/>")
        self.writeTempo(outbeats)
        self.indentlevel -= 1
        self.writeNewline("</direction>\n")
        
    def getMaxFontsizCurrentLine(self, TxtAndFonts, index):
        # das ist Klassenfunktion nur weil es nicht so weit weg von
        # handleRichText stehen soll.
        maxFontsize = 0
        while index < len(TxtAndFonts):
            textchunk = TxtAndFonts[index]
            index += 1
            if textchunk != "":
                rawText, fontname, fontsize, italic, bold, aligntype, charset = self.getTextFromChunk(textchunk)
                if maxFontsize < fontsize:
                    maxFontsize = fontsize
                if 0 <= rawText.find("\n"):
                    break
        return maxFontsize            
       
    def getRightXoffCurrentLine(self, TxtAndFonts, index):
        # das ist Klassenfunktion nur weil es nicht so weit weg von
        # handleRichText stehen soll.
        length = 0
        charcount = 0
        while index < len(TxtAndFonts):
            textchunk = TxtAndFonts[index]
            index += 1
            if textchunk != "":
                rawText, fontname, fontsize, italic, bold, aligntype, charset = self.getTextFromChunk(textchunk)
                restlength = rawText.find("\n")
                if 0 <= restlength:
                    length += TextWidthMXML(fontsize, bold, rawText[:restlength], charset)
                    charcount += restlength
                    break
                length += TextWidthMXML(fontsize, bold, rawText, charset)
                charcount += len(rawText)
        return length,charcount

    def handleRichText(self, ch, AsCredit, iVoice):
        x = int(float(ch.getAttribute("x"))*10.0)
        y = int(float(ch.getAttribute("y"))*10.0)
        boxwidth = int(float(ch.getAttribute("width"))*10.0)
        if AsCredit:
            x = int(x * self.pageObjLineDist / self.normalLineDist)
            y = int(y * self.pageObjLineDist / self.normalLineDist)
            boxwidth = int(boxwidth * self.pageObjLineDist / self.normalLineDist)
        file = ch.getAttribute("file")
        text = self.zipfile.read(file)
        TxtAndFonts = Rtf2TxtAndFont.getTxt(text)
        xoffset = 0
        yoffset = 0
        countChunks = -1
        fontsizeMax = 0
        CenterAlignPending = False
        for textchunk in TxtAndFonts:
            countChunks += 1
            if textchunk != "":
                rawText, fontname, fontsize, italic, bold, aligntype, charset = self.getTextFromChunk(textchunk)
                # y-Referenz in Capella ist oberer Rand des Kastens. Die
                # Höhe des ersten Strings schätzen wir mit yoffset:
                if fontsizeMax == 0:
                    fontsizeMax = self.getMaxFontsizCurrentLine(TxtAndFonts, countChunks)
                    y += TextHeightMXML(fontsizeMax)/5
                yoffset = TextHeightMXML(fontsizeMax)
                alignstr = "left"
                if aligntype == "c":
                    if not CenterAlignPending:
                        alignstr = "center"
                elif aligntype == "r":
                    alignstr = "right"
                TextContent = self.decodeFromCharset(rawText, charset)
                if aligntype == "l":
                    xwrite = x                    
                elif aligntype == "r":
                    xwrite = x + boxwidth
                    clExtent,clCharcount = self.getRightXoffCurrentLine(TxtAndFonts, countChunks+1)
                    xoffset = -clExtent
                elif aligntype == "c":
                    clExtent,clCharcount = self.getRightXoffCurrentLine(TxtAndFonts, countChunks)
                    cchunkCharcount = len(TextContent)
                    cchunkNLIndex = TextContent.find("\n")
                    if not CenterAlignPending and (cchunkCharcount == clCharcount or cchunkNLIndex >= 0):
                        alignstr = "center"
                        xwrite = x + boxwidth/2
                        xoffset = 0
                    elif not CenterAlignPending:
                        alignstr = "left"
                        xwrite = x + boxwidth/2 - clExtent/2
                        xoffset = 0
                        CenterAlignPending = True
                    elif CenterAlignPending:
                        if cchunkCharcount == clCharcount or cchunkNLIndex >= 0:
                            CenterAlignPending = False
                if y<0:
                    where = "above"
                else:
                    where = "below"
                indexNl = TextContent.find("\n")
                if fontname == "capella3":
                    if indexNl >= 0:
                        TextContentReal = TextContent[:indexNl]
                    else:
                        TextContentReal = TextContent
                    NoteType = isNoteType(TextContentReal)
                    if NoteType != "":
                        # das könnte eine Tempoangabe sein! Wir suchen nach "=" und Zahl.
                        ChunkOffset = 1
                        while len(TxtAndFonts) > countChunks + ChunkOffset:
                            # gelegentlich verbergen sich in RichText-Feldern überflüssige Fontänderungen!
                            if self.isTextChunkWhitespace(TxtAndFonts[countChunks+ChunkOffset]):
                                ChunkOffset +=1
                            else:
                                break                                                          
                        if len(TxtAndFonts) > countChunks+ChunkOffset:
                            beats, beatsStr, parenthese = self.getMetronomeVals(TxtAndFonts[countChunks+ChunkOffset])
                            if beats > 0 and not AsCredit:
                                self.handleMetronome(where, xwrite + xoffset, y + yoffset ,beats, beatsStr, NoteType, parenthese)
                                break
                    if isCodaLarge(TextContentReal) or isCodaSmall(TextContentReal):
                        position = "relative-x=\""+str(xwrite + xoffset)+"\" default-y=\""+str( -y -yoffset )+"\""
                        position += " font-size=\""+str(fontsize)+"\""
                        self.writeDirection(where,"coda", position, "", iVoice)
                        xoffset += 2*TextWidthMXML(fontsize, bold, TextContentReal, charset)
                        continue
                    elif isSegno1(TextContentReal):
                        position = "relative-x=\""+str(xwrite + xoffset)+"\" default-y=\""+str( -y -yoffset )+"\""
                        position += " font-size=\""+str(fontsize)+"\""
                        self.writeDirection(where,"segno", position, "", iVoice)
                        xoffset += 2*TextWidthMXML(fontsize, bold, TextContentReal, charset)
                        continue
                    elif isTranslatable(TextContent):
                        TextContent = Capella3ToText(TextContent)
                        fontname = ""
                    else:
                        TextContent = Capella3ToMaestro(TextContent)
                    # das wird von Dolet light nicht interpretiert: font-family "Maestro" oder "music"
                    # Dolet 3.2 kennt Maestro.
                        fontname = "Maestro"
                while indexNl >= 0:
                    if aligntype == "r":
                        xoffset = 0
                    elif aligntype == "c":
                        pass
                    if indexNl > 0:
                        if AsCredit:
                            self.writeCredit(xwrite + xoffset, y + yoffset,fontname,str(fontsize),bold,italic,alignstr,TextContent[:indexNl])
                        else:
                            self.writeDirectionWords(where,xwrite + xoffset, y + yoffset ,fontname,str(fontsize),bold,italic,alignstr,TextContent[:indexNl], iVoice)
                    y += yoffset
                    TextContent = TextContent[indexNl+1:]
                    indexNl = TextContent.find("\n")
                    if indexNl < 0:
                        fsMax = self.getMaxFontsizCurrentLine(TxtAndFonts, countChunks+1)
                        clExtent,clCharcount = self.getRightXoffCurrentLine(TxtAndFonts, countChunks+1)
                        if aligntype == "r":
                            xoffset = -clExtent
                        elif aligntype == "c":
                            CenterAlignPending = False
                            clExtent,clCharcount = self.getRightXoffCurrentLine(TxtAndFonts, countChunks+1)
                            cchunkCharcount = len(TextContent)
                            xoffset = 0
                            if cchunkCharcount == clCharcount:
                                alignstr = "center"
                                xwrite = x + boxwidth/2
                            elif cchunkCharcount > 0:
                                alignstr = "left"
                                xwrite = x + boxwidth/2 - (clExtent + TextWidthMXML(fontsize, bold, TextContent, charset))/2
                                CenterAlignPending = True
                        if fsMax > fontsize:
                            fontsizeMax = fsMax
                            yoffset = TextHeightMXML(fontsizeMax)
                    else:
                        yoffset = TextHeightMXML(fontsize)
                        if aligntype == "c":
                            alignstr = "center"
                            xwrite = x + boxwidth/2
                            xoffset = 0
                    if aligntype == "l":
                        xoffset = 0
                if len(TextContent) > 0:
                    if AsCredit:
                        self.writeCredit(xwrite + xoffset, y + yoffset,fontname,str(fontsize),bold,italic,alignstr,TextContent)
                    else:
                        parentheseIndex = TextContent.find("(")
                        if parentheseIndex >= 0 and len(TxtAndFonts) >= countChunks+2:
                            # Es könnte eine eingeklammerte Tempoangabe sein!
                            thistext, thisfontname, fontsizeDummy, italicDummy, boldDummy, aligntypeDummy, charsetDummy = self.getTextFromChunk(TxtAndFonts[countChunks+1])
                            if thisfontname == "capella3":
                                NoteType = isNoteType(self.getPureTextFromChunk(TxtAndFonts[countChunks+1]))
                                if NoteType != "":
                                    beats, beatsStr, parenthese = self.getMetronomeVals(TxtAndFonts[countChunks+2])
                                    if beats > 0 and not AsCredit:
                                        self.handleMetronome(where, xwrite + xoffset, y + yoffset ,beats, beatsStr, NoteType, parenthese)
                                        break
                        self.writeDirectionWords(where, xwrite + xoffset, y + yoffset,fontname,str(fontsize),bold,italic,alignstr,TextContent, iVoice)
                    if aligntype == "l":
                        xoffset += TextWidthMXML(fontsize, bold, TextContent, charset)
                    elif aligntype == "c":
                        if CenterAlignPending:
                            xoffset += TextWidthMXML(fontsize, bold, TextContent, charset)

    def writeCredit(self,x,y,fontname,fontsiz,bold,italic,align,content):
        fontlower = TextHeightMXML(float(fontsiz))/5
        # In Finale ist die Referenzposition für Credits die untere Grenze von gjpqy,
        # (für direction-type word jedoch die untere Grenze von abcde)
        position = "default-x=\""+str(x + self.pagemarginLeft)+"\" default-y=\""+str(self.pagesizeY - y - self.pagemarginTop - fontlower)+"\""
        if fontname != "":
            position += " font-family=\""+fontname+"\""
        position += " font-size=\""+fontsiz+"\""
        if bold:
            position += " font-weight=\"bold\""
        if italic:
            position += " font-style=\"italic\""
        if align != "":
            position += " justify=\""+align+"\""
        self.writeNewline("<credit>\n")
        self.indentlevel += 1
        self.writeElementWithAttribute("credit-words",position,utf8(content))
        self.indentlevel -= 1
        self.writeNewline("</credit>\n")

    def getLineCountFromStaffLayout(self, staffLayout):                
        # Schlagzeug?
        lineCount = 5
        notation = self.getFirstChildElement(staffLayout,"notation")
        if notation:
            nlstring = notation.getAttribute("notelines")
            if nlstring == "5":
                lineCount = 5
            elif nlstring == "1":
                lineCount = 1
            else:
                Count = nlstring.count("|")
                if Count == 1:
                    lineCount = Count
        return lineCount

    def handleDistances(self, layout, firstNameStyle, othersNameStyle):
        distances = self.getFirstChildElement(layout, "distances")
        staffLines = self.getFirstChildElement(distances, "staffLines")
        small = staffLines.getAttribute("small")
        normalStr = staffLines.getAttribute("normal")
        self.normalLineDist = float(normalStr)
        pageObj = staffLines.getAttribute("pageObj")
        self.pageObjLineDist = float(pageObj)
        self.writeNewline("<defaults>\n")
        self.indentlevel += 1
        self.writeNewline("<scaling>\n")
        self.indentlevel += 1
        self.writeSimpleElement("millimeters",normalStr)
        zehntel = 10.0
        self.writeSimpleElement("tenths",str(zehntel))
        self.indentlevel -= 1
        self.writeNewline("</scaling>\n")
        self.writeNewline("<page-layout>\n")
        self.indentlevel += 1
        pages = self.getFirstChildElement(layout, "pages")
        left = pages.getAttribute("left")
        right = pages.getAttribute("right")
        top = pages.getAttribute("top")
        bottom = pages.getAttribute("bottom")
        landscape = pages.getAttribute("landscape")
        if landscape == "true":
            y,x = pagesizeMm()
        else:
            x,y = pagesizeMm()
        self.pagesizeY = int(y/self.normalLineDist*zehntel)
        self.pagesizeX = int(x/self.normalLineDist*zehntel)
        self.writeSimpleElement("page-height",str(self.pagesizeY))
        self.writeSimpleElement("page-width",str(self.pagesizeX))
        self.pagemarginLeft = int(float(left)/self.normalLineDist*zehntel)
        self.pagemarginRight = int(float(right)/self.normalLineDist*zehntel)
        self.pagemarginTop = int(float(top)/self.normalLineDist*zehntel)
        self.pagemarginBottom = int(float(bottom)/self.normalLineDist*zehntel)
        self.writeNewline("<page-margins type=\"both\">\n")
        self.indentlevel += 1
        self.writeSimpleElement("left-margin",str(self.pagemarginLeft))
        self.writeSimpleElement("right-margin",str(self.pagemarginRight))
        self.writeSimpleElement("top-margin",str(self.pagemarginTop))
        self.writeSimpleElement("bottom-margin",str(self.pagemarginBottom))
        self.indentlevel -= 1
        self.writeNewline("</page-margins>\n")
        self.indentlevel -= 1
        self.writeNewline("</page-layout>\n")
        # In capella gibt's eine Automatik: der linken Rand der Systeme
        # wird abhängig von der Länge der Stimmenbezeichnung verschoben.
        # Für Finale wird das hier nachempfunden. Das erste System muss
        # separat später in einem <print>-Element positioniert werden.
        instrumentNames = self.getFirstChildElement(layout,"instrumentNames")
        if instrumentNames:
            font = self.getFirstChildElement(instrumentNames, "font")
            fontsizePartName = font.getAttribute("height")
            fontweight = font.getAttribute("weight")
            fontCharSet = font.getAttribute("charSet")
            if fontCharSet == "":
                fontCharSet = "0"
            bold = fontweight != "" and int(fontweight) > 400
        else:
            font = "Times New Roman"
            fontsizePartName = "12"
            fontweight = "0"
            fontCharSet = "0"
            bold = False
        staves = self.getFirstChildElement(layout,"staves")
        instruments = staves.getElementsByTagName("instrument")
        longestPartName = ""
        longestPartAbbrev = ""
        for instrument in instruments:
            name = instrument.getAttribute("name")
            if len(name) > len(longestPartName):
                longestPartName = name
            name = instrument.getAttribute("abbrev")
            if len(name) > len(longestPartAbbrev):
                longestPartAbbrev = name
            name = instrument.getAttribute("pairName")
            if len(name) > len(longestPartName):
                longestPartName = name
            name = instrument.getAttribute("pairAbbrev")
            if len(name) > len(longestPartAbbrev):
                longestPartAbbrev = name
        # Abstand Stimmenbezeichnung - Systemanfang
        if longestPartName != "":
            longestPartName += "____"
        self.marginLong = TextWidthMXML(int(float(fontsizePartName)), bold, longestPartName, fontCharSet)
        stringtomeasure = longestPartName
        if firstNameStyle == "short":
            stringtomeasure = longestPartAbbrev
        elif firstNameStyle == "none":
            stringtomeasure = ""
        self.marginSys1 = TextWidthMXML(int(float(fontsizePartName)), bold, stringtomeasure, fontCharSet)
        if longestPartAbbrev != "":
            longestPartAbbrev += "____"
        self.marginShort = TextWidthMXML(int(float(fontsizePartName)), bold, longestPartAbbrev, fontCharSet)
        stringtomeasure = longestPartName
        if othersNameStyle == "short":
            stringtomeasure = longestPartAbbrev
        elif othersNameStyle == "none":
            stringtomeasure = ""
        margin = TextWidthMXML(int(float(fontsizePartName)), bold, stringtomeasure, fontCharSet)
        systems = self.getFirstChildElement(distances, "systems")
        self.ydistSys1 = 10 * int(systems.getAttribute("top"))
        ydist = int(systems.getAttribute("between"))
        self.DefaultSystemDistance = ydist
        self.writeDefaultSystemLayout(margin, 10 * ydist, self.ydistSys1)
        staffInd = 0
        self.StaffDist = []
        for n in staves.childNodes:
            if n.nodeType == n.ELEMENT_NODE and n.tagName == "staffLayout":
                # Schlagzeug?
                lineCount = self.getLineCountFromStaffLayout(n)
                
                distances = self.getFirstChildElement(n, "distances")
                top = 0
                bottom = 0
                if distances != None:
                    topstaff = distances.getAttribute("top")
                    bottomstaff = distances.getAttribute("bottom")
                    groupstaff = distances.getAttribute("group")
                    # Capella misst Zeilenabstände von der Zeilenmitte,
                    # MusicXML von der äußeren Linie.
                    if topstaff != "":
                        top = int(topstaff) - 2
##                        if lineCount == 1:
##                            top += 4
                    if bottomstaff != "":
                        bottom = int(bottomstaff) - 2
                        if lineCount == 1:
                            bottom += 4
                    if groupstaff != "":
                        bottom += int(groupstaff)
                self.StaffDist.append((top,bottom))
                staffInd += 1

        self.indentlevel -= 1
        self.writeNewline("</defaults>\n")

    def handlePageObjects(self, pageObjects):
        CreditWritten = False
        drawObjects = pageObjects.getElementsByTagName("drawObj")
        for n in drawObjects:
            if self.isElement(n, "drawObj"):
                if self.handleDrawObjAsCredit(n):
                    CreditWritten = True
        return CreditWritten

    def handleVolta(self, n, ch, iVoice):
        # n ist drawObj, ch ist volta.
        basic = self.getFirstChildElement(n,"basic")
        pending = 1
        if basic != None:
            range = basic.getAttribute("noteRange")
            if range != "":
                pending = int(range) + 1
        if self.isCAPX2:
            noteObj = n.parentNode.parentNode
            if noteObj.tagName == "barline":
                pending -= 1
        # Ending kann nur einmal für einen part existieren,
        # obwohl ein volta-DrawObj in jeder Stimme jeder Zeile
        # vorkommen kann, die zu diesem part zugeordnet ist.
        # Das ist zwar i.A. nicht sinnvoll, aber auch nicht
        # ausgeschlossen.
        EndingNumber = ch.getAttribute("firstNumber")
        if EndingNumber == '':
            x1 = ch.getAttribute("x1")
            x2 = ch.getAttribute("x2")
            y = int(10.0 * float(ch.getAttribute("y")))
            # mittlere Linie ist Referenz für CapXML
            if y<0:
                where = "above"
                y += 20
            else:
                where = "below"
                y += 20
            NotRightBent = "false" == ch.getAttribute("rightBent")
            NotLeftBent = "false" == ch.getAttribute("leftBent")
            if "true" == ch.getAttribute("dotted"):
                linetype = " line-type=\"dashed\""
            else:
                linetype = ""
            if "true" == ch.getAttribute("upBent"):
                if NotLeftBent:
                    llineend = "\"none\""
                else:
                    llineend = "\"up\""
                if NotRightBent:
                    rlineend = "\"none\""
                else:
                    rlineend = "\"up\""
            else:
                if NotLeftBent:
                    llineend = "\"none\""
                else:
                    llineend = "\"down\""
                if NotRightBent:
                    rlineend = "\"none\""
                else:
                    rlineend = "\"down\""
            # das ist wahrscheinlich keine Voltenklammer und soll als bracket exportiert werden.
            # Das soll aber nicht gemacht werden, wenn wir aus writeHiddenLine gerufen wurden.
            if len(self.MaxBracketNo) > iVoice:
                self.MaxBracketNo[iVoice] += 1
                BracketLabel = 0
                for tup in self.MaxBracketNo:
                    BracketLabel += tup
                (self.BracketPending[iVoice])[self.MaxBracketNo[iVoice]] = int(pending), BracketLabel, rlineend
                position = "line-end="+llineend+linetype+" type=\"start\" number=\""+str(self.MaxBracketNo[iVoice])+"\" default-y=\""+str(-y)+"\""
                self.writeDirection(where, "bracket", position, "", iVoice)
        else:
            note = n.parentNode.parentNode
            pending = self.checkVoltaOverlap(note, pending)
            Pend = self.EndingPending
            if Pend >= 0:
                Number = self.EndingNumber
                Discontinue = self.EndingDiscontinue
                # Diese Notfallmaßnahme, falls z.B. VoltaOverlap in unterschiedlichen Stimmen einer Zeile vorkommt,
                # kann den Output nicht reparieren - die VoltaStopBarline hätte am Schluss des vorigen Taktes
                # geschrieben werden müssen. Diese Situation ist ja auch in capella eine Fehlersituation.
                # self.writeVoltaStopBarline(Number, Discontinue)
                pass
            self.EndingPending = pending
            self.EndingVoice = iVoice
            self.EndingNumber = EndingNumber
            last = ch.getAttribute("lastNumber")
            if last != "":
                if "true" == ch.getAttribute("allNumbers"):
                    intnext = int(self.EndingNumber) + 1
                    intlast = int(last)
                    while intnext <= intlast:
                        self.EndingNumber += ". "+str(intnext)
                        intnext += 1
                else:
                    self.EndingNumber += ". - "+last
            self.EndingDiscontinue = "false" == ch.getAttribute("rightBent")
            self.writeVoltaStartBarline(self.EndingNumber)

    def getVolta(self, cBarline):
        drawObjects = self.getFirstChildElement(cBarline, "drawObjects")
        if drawObjects != None:
            drawObj = self.getFirstChildElement(drawObjects, "drawObj")
            for ch in drawObj.childNodes:
                if ch.nodeType == drawObj.ELEMENT_NODE:
                    if ch.tagName == "volta":
                        return drawObj

    def handleDrawObj(self, n, iVoice):
        notationsstr = ""
        for ch in n.childNodes:
            if ch.nodeType == n.ELEMENT_NODE:
                if ch.tagName == "text":
                    notationsstr += self.handleNoteObjText(ch, iVoice)
                elif ch.tagName == "richText":
                    self.handleRichText(ch,False, iVoice)
                elif ch.tagName == "wedge":
                    basic = self.getFirstChildElement(n,"basic")
                    self.handleWedge(ch, basic, iVoice)
                elif ch.tagName == "group":
                    notationsstr += self.handleDrawObjects(ch, iVoice)
                elif ch.tagName == "volta":
                    if n.parentNode.parentNode.tagName != "barline":
                        self.handleVolta(n, ch, iVoice)
                elif ch.tagName == "transposable":
                    requiredbase = ch.getAttribute("base");
                    visibletext = None;
                    if self.chordstyle == {}:
                        self.findChordstyle(ch)
                    for instance in ch.childNodes:
                        if instance.nodeType == ch.ELEMENT_NODE and instance.tagName == "drawObj":
                            if requiredbase == instance.getAttribute("base"):
                                visibletext = self.getFirstChildElement(instance,"text");
                                if visibletext != None:
                                    self.handleHarmonyText(visibletext, requiredbase)
                                    break
                                visibleRichtext = self.getFirstChildElement(instance,"richText")
                                if visibleRichtext != None:
                                    self.handleHarmonyRichText(visibleRichtext, requiredbase)
                                    break
                                Group = self.getFirstChildElement(instance,"group")
                                if Group != None:
                                    self.handleHarmonyGroup(Group, requiredbase)
                                    break
                        
                # slur und bracket werden in müssen wir an den einzelnen Köpfen implementieren.
                # vgl. handleSlurTuplet!
        return notationsstr

    def findChordstyle(self,ch):
        american =  ["Fb", "Cb", "Gb", "Db", "Ab","Eb","Bb", "F", "C", "G",  "D", "A", "E", "B", "F#", "C#", "G#", "D#", "A#", "E#", "B#"]
        german1 =   ["Fes","Ces","Ges","Des","As","Es","B",  "F", "C", "G",  "D", "A", "E", "H","Fis","Cis","Gis", "Dis","Ais","Eis","His"]
        # german2 = Scandinavian (Finale)
        german2 =    ["Fb","Cb","Gb","Db", "Ab", "Eb", "B",  "F", "C", "G",  "D", "A", "E", "H", "F#", "C#", "G#",  "D#", "A#", "E#","H#"]
        # Finale kennt folgene:
        solfeggio1 = ["Mi","Ti","Se","Ra", "Le", "Me", "Te", "Fa","Do","So", "Re","La","Mi","Ti","Fi", "Di", "Si",  "Ri", "Li", "Fa","Do"]
        # gebräuchlich in Italien ist auch:
        solfeggio2 = ["Mi","Ti","Se","Reb","Lab","Mib","Sib","Fa","Do","Sol","Re","La","Mi","Si","Fa#","Do#","Sol#","Re#","La#","Fa","Do"]
        # die ganze Liste der möglichen Transpositionen erfassen:
        basecount = 0
        chordlist = []
        for instance in ch.childNodes:
            if instance.nodeType == ch.ELEMENT_NODE and instance.tagName == "drawObj":
                basecount += 1
                visibletext = self.getFirstChildElement(instance,"text");
                if visibletext != None:
                    font = self.getFirstChildElement(visibletext, "font")
                    fontCharSet = font.getAttribute("charSet")
                    content = self.getFirstChildElement(visibletext, "content")
                    for n in content.childNodes:
                        if n.nodeType == n.TEXT_NODE:
                            if fontCharSet != "":
                                text = self.decodeFromCharset(n.data,fontCharSet)
                            else:
                                text = n.data
                            chordlist.append(self.stripBassAndCR(text))
                    continue
                visibleRichtext = self.getFirstChildElement(instance,"richText")
                if visibleRichtext != None:
                    file = visibleRichtext.getAttribute("file")
                    text = self.zipfile.read(file)
                    simpletext, fontsizedummy = self.getSimpleHarmonyFromRichText(text)
                    chordlist.append(self.stripBassAndCR(simpletext))
                    continue
                Group = self.getFirstChildElement(instance,"group")
                if Group != None:
                    wholetext = ""
                    for instance1 in Group.childNodes:
                        if instance1.nodeType == instance1.ELEMENT_NODE and instance1.tagName == "drawObj":
                            visibletext = self.getFirstChildElement(instance1,"text");
                            if visibletext != None:
                                font = self.getFirstChildElement(visibletext, "font")
                                fontCharSet = font.getAttribute("charSet")
                                fontface = font.getAttribute("face")
                                content = self.getFirstChildElement(visibletext, "content")
                                for n in content.childNodes:
                                    if n.nodeType == n.TEXT_NODE:
                                        if fontface == "capella3":
                                            text = Capella3ToText(n.data)
                                        elif fontCharSet != "":
                                            text = self.decodeFromCharset(n.data,fontCharSet)
                                        else:
                                            text = n.data
                                        wholetext += text
                    chordlist.append(self.stripBassAndCR(wholetext))
                    continue
                    
        # sind die enharmonischen Verwechslungen auch bereitgestellt?
        offset = 0
        if basecount == 12:
            offset = 5
        # die am besten passende vordefinierte Basisliste heraussuchen
        americanCount = 0
        german1Count = 0
        german2Count = 0
        solfeggio1Count = 0
        solfeggio2Count = 0
        iChord = 0
        while iChord < basecount:
            if 0 == chordlist[iChord].find(american[iChord+offset]):
                americanCount += 1
            if 0 == chordlist[iChord].lower().find(german1[iChord+offset].lower()):
                german1Count += 1
            if 0 == chordlist[iChord].lower().find(german2[iChord+offset].lower()):
                german2Count += 1
            if 0 == chordlist[iChord].lower().find(solfeggio1[iChord+offset].lower()):
                solfeggio1Count += 1
            if 0 == chordlist[iChord].lower().find(solfeggio2[iChord+offset].lower()):
                solfeggio2Count += 1
            iChord += 1
        style = american
        maxCount = americanCount
        if german1Count > maxCount:
            style = german1
            maxCount = german1Count
        if german2Count > maxCount:
            style = german2
            maxCount = german2Count
        if solfeggio1Count > maxCount:
            style = solfeggio1
            maxCount = solfeggio1Count
        if solfeggio2Count > maxCount:
            style = solfeggio2
            maxCount = solfeggio2Count
        iChord = 0
        while iChord < basecount:
            self.chordstyle[american[iChord+offset]] = style[iChord+offset]
            iChord += 1

    def handleHarmonyText(self, NoteObjText, base):
        xStr = NoteObjText.getAttribute("x")
        x = int(float(xStr) * 10.0)
        yStr = NoteObjText.getAttribute("y")
        y = int(float(yStr) * 10.0)
        # capella zählt ab mittlere Linie, nach oben negativ
        # MusicXML zählt ab obere Linie, nach oben positiv
        if self.staveLineCount != 1:
            y += 20
        if y<0:
            where = "above"
        else:
            where = "below"
        align = NoteObjText.getAttribute("align")
        # left, center, right in CapXML genau wie in MusicXML
        # allerdings 
        font = self.getFirstChildElement(NoteObjText, "font")
        fontheight = font.getAttribute("height")
        fontface = font.getAttribute("face")
        fontweight = font.getAttribute("weight")
        bold = fontweight != "" and int(fontweight) > 400
        fontitalic = font.getAttribute("italic")
        italic = fontitalic != ""
        fontCharSet = font.getAttribute("charSet")
        content = self.getFirstChildElement(NoteObjText, "content")
        for n in content.childNodes:
            if n.nodeType == n.TEXT_NODE:
                if fontCharSet == "capella3":
                    text = Capella3ToText(n.data)
                elif fontCharSet != "":
                    text = self.decodeFromCharset(n.data,fontCharSet)
                else:
                    text = n.data
                # jetzt die position wie in writeDirectionWords
                position = "relative-x=\""+str(x)+"\" default-y=\""+str( -y )+"\""
                if fontface != "":
                    position += " font-family=\""+fontface+"\""
                position += " font-size=\""+fontheight+"\""
                if bold:
                    position += " font-weight=\"bold\""
                if italic:
                    position += " font-style=\"italic\""
                if align != "":
                    position += " justify=\""+align+"\""
                # jetzt schreiben ähnlich wie in writeDirection
                self.writeHarmonyCommon( where, position, base, text)

    def getSimpleHarmonyFromRichText(self, richtext):
        TxtAndFonts = Rtf2TxtAndFont.getTxt(richtext)
        fontsize = 0
        simpletext = ""
        for textchunk in TxtAndFonts:
            if textchunk != "":
                rawText, fontname, fontsiz, italic, bold, aligntype, charset = self.getTextFromChunk(textchunk)
                if fontsize == 0 and fontsiz > 0:
                    fontsize = fontsiz
                if fontname == "capella3":
                    simpletext += Capella3ToText(rawText)
                else:
                    simpletext += rawText
        return simpletext, fontsize

    def stripBassAndCR(self, text):
        bassindex = text.find("/")
        if bassindex > 0:
            text = text[:bassindex]
        return text.strip("\n")

    def handleHarmonyRichText(self, ch, base):
        # MusicXML kennt keine Formatierung des Textes in <kind text="...">
        x = int(float(ch.getAttribute("x"))*10.0)
        y = int(float(ch.getAttribute("y"))*10.0)
        y += 20
        if y<0:
            where = "above"
        else:
            where = "below"
        boxwidth = int(float(ch.getAttribute("width"))*10.0)
        file = ch.getAttribute("file")
        text = self.zipfile.read(file)
        # y-Referenz in Capella ist oberer Rand des Kastens. Wir brauchen hier
        # den unteren Rand des Buchstabens. Deshalb müssen wir die fontsize
        # aus dem RichText herausholen. Wir nehmen die fontsize des ersten
        # Stücks, das ist normalerweise der größte und maßgebliche Buchstabe.
        #
        # Ausserdem müssen wir im Falle von capella3.ttf-Zeichen uminterpretieren.
        simpletext, fontsize = self.getSimpleHarmonyFromRichText(text)
##        TxtAndFonts = Rtf2TxtAndFont.getTxt(text)
##        fontsize = 0
##        simpletext = ""
##        for textchunk in TxtAndFonts:
##            if textchunk != "":
##                rawText, fontname, fontsiz, italic, bold, aligntype, charset = self.getTextFromChunk(textchunk)
##                if fontsize == 0 and fontsiz > 0:
##                    fontsize = fontsiz
##                if fontname == "capella3":
##                    simpletext += Capella3ToText(rawText)
##                else:
##                    simpletext += rawText
        y += TextHeightMXML(fontsize)
        position = "relative-x=\""+str(x)+"\" default-y=\""+str( -y )+"\""
        self.writeHarmonyCommon( where, position, base, simpletext.strip("\n"))

    def handleHarmonyGroup(self, Group, base):
        # Hier werden nur Gruppen von Einfachtexten abgehandelt. Die sind für
        # Akkordsymbole in capella eine schöne Möglichkeit, den wenig schlanken
        # RichText zu vermeiden.
        wholetext = ""
        position = ""
        for instance in Group.childNodes:
            if instance.nodeType == instance.ELEMENT_NODE and instance.tagName == "drawObj":
                NoteObjText = self.getFirstChildElement(instance,"text");
                if NoteObjText != None:
                    font = self.getFirstChildElement(NoteObjText, "font")
                    fontheight = font.getAttribute("height")
                    fontface = font.getAttribute("face")
                    fontweight = font.getAttribute("weight")
                    bold = fontweight != "" and int(fontweight) > 400
                    fontitalic = font.getAttribute("italic")
                    italic = fontitalic != ""
                    fontCharSet = font.getAttribute("charSet")
                    content = self.getFirstChildElement(NoteObjText, "content")
                    for n in content.childNodes:
                        if n.nodeType == n.TEXT_NODE:
                            if fontface == "capella3":
                                text = Capella3ToText(n.data)
                            elif fontCharSet != "":
                                text = self.decodeFromCharset(n.data,fontCharSet)
                            else:
                                text = n.data
                            wholetext += text
                            if position == "":
                                # Nur das erste Gruppenelement definiert die Position!
                                xStr = NoteObjText.getAttribute("x")
                                x = int(float(xStr) * 10.0)
                                yStr = NoteObjText.getAttribute("y")
                                y = int(float(yStr) * 10.0)
                                # capella zählt ab mittlere Linie, nach oben negativ
                                # MusicXML zählt ab obere Linie, nach oben positiv
                                if self.staveLineCount != 1:
                                    y += 20
                                if y<0:
                                    where = "above"
                                else:
                                    where = "below"
                                align = NoteObjText.getAttribute("align")
                                # jetzt die position wie in writeDirectionWords
                                position = "relative-x=\""+str(x)+"\" default-y=\""+str( -y )+"\""
                                if fontface != "":
                                    position += " font-family=\""+fontface+"\""
                                position += " font-size=\""+fontheight+"\""
                                if bold:
                                    position += " font-weight=\"bold\""
                                if italic:
                                    position += " font-style=\"italic\""
                                if align != "":
                                    position += " justify=\""+align+"\""
        # jetzt schreiben ähnlich wie in writeDirection
        self.writeHarmonyCommon( where, position, base, wholetext)

    def writeHarmonyCommon(self, where, position, base, rawtext):
        localizedbase = self.chordstyle[base]
        bassindex = rawtext.find("/")
        if bassindex > 0:
            text = rawtext[:bassindex]
            basstext = rawtext[bassindex+1:]
            # translate local bass to american bass, if possible
            chordstyleitems = self.chordstyle.items()
            for bassbase, localbass in chordstyleitems:
                if localbass == basstext:
                    basstext = bassbase
        else:
            text = rawtext            
        capitalizedtext = text.capitalize()
        # Achtung: capitalize() macht alle folgenden Buchstaben klein!
        if 0 == capitalizedtext.find(localizedbase):
            # das Akkordsymbol entspricht einem der vordefinierten Stile
            suffix = text[len(localizedbase):]
        else:
            # wir nehmen an, dass es ein Versehen oder ein Synomym wie Do = Ut oder Si = Ti ist.
            # Man könnte hier eine Warnung anzeigen.
            suffix = text[len(localizedbase):]
        if 0 > suffix.find("<"):
            # Found Finale Dolet error message:
            # The value of attribute "text" associated with an element type "kind" must
            # not contain the '<' character.
            self.writeNewline("<harmony ")
            if where != "":
                self.f.write(" placement=\""+where+"\" ")
            self.f.write(position)
            self.f.write(">\n")
            self.indentlevel += 1
            self.writeNewline("<root>\n")
            self.indentlevel += 1
            self.writeElementWithAttributeAndChild("root-step","", base[0:1])
            if len(base) > 1:
                if base[1:2] == 'b':
                    self.writeElementWithAttributeAndChild("root-alter","", "-1")
                elif base[1:2] == '#':
                    self.writeElementWithAttributeAndChild("root-alter","", "1")
            self.indentlevel -= 1
            self.writeNewline("</root>\n")
            kind = self.findHarmonyKind(suffix, globalChordCaseMatters and capitalizedtext[0] != text[0])
            self.writeElementWithAttributeAndChild("kind", "text=\""+utf8(suffix)+"\"", kind)
            if bassindex > 0:
                self.writeNewline("<bass>\n")
                self.indentlevel += 1
                self.writeElementWithAttributeAndChild("bass-step","", basstext[0:1])
                if len(basstext) > 1:
                    if basstext[1:2] == 'b':
                        self.writeElementWithAttributeAndChild("bass-alter","", "-1")
                    elif basstext[1:2] == '#':
                        self.writeElementWithAttributeAndChild("bass-alter","", "1")
                self.indentlevel -= 1
                self.writeNewline("</bass>\n")
            self.indentlevel -= 1
            self.writeNewline("</harmony>\n")
        else:
            pass
        
    def findHarmonyKind(self,suffix, islowercase):
        if globalFinaleDolet33:
            # can handle "none" kind except in naked major chords
            if len(suffix) > 0:
                kind = "none"
            else:
                kind = "major"
        else:
            # cannot handle "none" kind. Try primitive chord type parsing.
            kind = "major"
            index7 = suffix.find("7")
            index6 = suffix.find("6")
            if suffix[:1] == "m" or islowercase:
                kind = "minor"
                if index7 >= 0:
                    kind = "minor-seventh"
                elif index6 >= 0:
                    kind = "minor-sixth"
            else:
                if index7 >= 0:
                    kind = "dominant"
                elif index6 >= 0:
                    kind = "major-sixth"
        return kind

    def handleNoteObjText(self, NoteObjText, iVoice):
        notationsstr = ""
        xStr = NoteObjText.getAttribute("x")
        x = int(float(xStr) * 10.0)
        yStr = NoteObjText.getAttribute("y")
        y = int(float(yStr) * 10.0)
        # capella zählt ab mittlere Linie, nach oben negativ
        # MusicXML zählt ab obere Linie, nach oben positiv
        if self.staveLineCount != 1:
            y += 20
        if y<0:
            where = "above"
        else:
            where = "below"
        align = NoteObjText.getAttribute("align")
        # left, center, right in CapXML genau wie in MusicXML
        # allerdings 
        font = self.getFirstChildElement(NoteObjText, "font")
        fontheight = font.getAttribute("height")
        fontface = font.getAttribute("face")
        fontweight = font.getAttribute("weight")
        fontitalic = font.getAttribute("italic")
        fontunderline = font.getAttribute("underline")
        fontcolor = font.getAttribute("color")
        fontCharSet = font.getAttribute("charSet")
        content = self.getFirstChildElement(NoteObjText, "content")
        for n in content.childNodes:
            if n.nodeType == n.TEXT_NODE:
                if fontface == "capella3":
                    placementstr = " placement=\""+where+"\""
                    if isMXMLornament(n.data):
                        pos = "relative-x=\""+str(x)+"\" default-y=\""+str( -y  )+"\""
                        notationsstr += getNtabs(self.indentlevel+2)
                        notationsstr += "<ornaments>\n"
                        notationsstr += getNtabs(self.indentlevel+3)
                        if isTrill(n.data):
                            notationsstr += "<trill-mark "+pos+placementstr+"/>\n"
                        elif isPrallTrill(n.data):
                            notationsstr += "<inverted-mordent "+pos+placementstr+"/>\n"
                        elif isMordent(n.data):
                            notationsstr += "<mordent "+pos+placementstr+"/>\n"
                        elif isDoppelschlag(n.data):
                            if x > 10:
                                notationsstr += "<delayed-turn "+pos+placementstr+"/>\n"
                            else:
                                notationsstr += "<turn "+pos+placementstr+"/>\n"
                        notationsstr += getNtabs(self.indentlevel+2)
                        notationsstr += "</ornaments>\n"
                    elif isFermataAbove(n.data):
                        notationsstr += getNtabs(self.indentlevel+2)
                        notationsstr += "<fermata type=\"upright\"/>\n"
                    elif isFermataBelow(n.data):
                        notationsstr += getNtabs(self.indentlevel+2)
                        notationsstr += "<fermata type=\"inverted\"/>\n"
                    elif isAufstrich(n.data):
                        notationsstr += getNtabs(self.indentlevel+2)
                        notationsstr += "<technical>\n"
                        notationsstr += getNtabs(self.indentlevel+3)
                        notationsstr += "<up-bow/>\n"
                        notationsstr += getNtabs(self.indentlevel+2)
                        notationsstr += "</technical>\n"
                    elif isAbstrich(n.data):
                        notationsstr += getNtabs(self.indentlevel+2)
                        notationsstr += "<technical>\n"
                        notationsstr += getNtabs(self.indentlevel+3)
                        notationsstr += "<down-bow/>\n"
                        notationsstr += getNtabs(self.indentlevel+2)
                        notationsstr += "</technical>\n"
                    else:
                        position = "relative-x=\""+str(x)+"\" default-y=\""+str( -y )+"\""
                        position += " font-size=\""+fontheight+"\""
                        dynamicStr = getMXMLdynamics(n.data)
                        if dynamicStr != "":
                            self.writeDirection(where, "dynamics", position, dynamicStr, iVoice)
                        else:
                            if isPedalStart(n.data):
                                self.writeDirection(where,"pedal", position + " line=\"no\" type=\"start\"","", iVoice)
    #                            self.writeNewline("<pedal "+ position + " line=\"no\" type=\"start\"")
                            elif isPedalStop(n.data):
                                self.writeDirection(where,"pedal", position + " line=\"no\" type=\"stop\"","", iVoice)
                            elif isSegno1(n.data):
                                # capella segno Referenz-y-Position ist in der Mitte
                                poscorrsegno = TextHeightMXML(float(fontheight))/3
                                position = "relative-x=\""+str(x)+"\" default-y=\""+str( -y -poscorrsegno )+"\""
                                position += " font-size=\""+fontheight+"\""
                                self.writeDirection(where,"segno", position, "", iVoice)
                            elif isSegno2(n.data):
                                self.writeDirection(where,"segno", position, "", iVoice)
                            elif isCodaLarge(n.data):
                                poscorrkopf = TextHeightMXML(float(fontheight))/4
                                position = "relative-x=\""+str(x)+"\" default-y=\""+str( -y -poscorrkopf )+"\""
                                position += " font-size=\""+fontheight+"\""
                                self.writeDirection(where,"coda", position, "", iVoice)
                            elif isCodaSmall(n.data):
                                self.writeDirection(where,"coda", position, "", iVoice)
                else:
                    # int(fontheight) ist nicht genau; wir bräuchten die Höhe des Buchstabenrechtecks
                    # in Zehnteln.
                    if fontCharSet != "":
                        text = self.decodeFromCharset(n.data,fontCharSet)
                    else:
                        text = n.data
                    if globalFinaleDolet33:
                        self.writeDirectionWords(where,x,y,fontface,fontheight,fontweight != "" and int(fontweight) > 400, fontitalic != "", align,text, iVoice)
                    else:
                        self.writeDirectionWordsMultiline(where,x,y,fontface,fontheight,fontweight != "" and int(fontweight) > 400, fontitalic != "", align,text, iVoice)
        return notationsstr

    def writeDirectionWords(self,where,x,y,fontname,fontsiz,bold,italic,align,content, iVoice):
        position = "relative-x=\""+str(x)+"\" default-y=\""+str( -y )+"\""
        if fontname != "":
            position += " font-family=\""+fontname+"\""
        position += " font-size=\""+fontsiz+"\""
        if bold:
            position += " font-weight=\"bold\""
        if italic:
            position += " font-style=\"italic\""
        if align != "":
            position += " justify=\""+align+"\""
        self.writeDirection(where,"words", position, str(utf8(content)), iVoice)
                    
    def writeDirectionWordsMultiline(self,where,x,y,fontname,fontsiz,bold,italic,align,content, iVoice):
        position1 = "relative-x=\""+str(x)+"\" default-y=\""
        position2 = "\""
        if fontname != "":
            position2 += " font-family=\""+fontname+"\""
        position2 += " font-size=\""+fontsiz+"\""
        if bold:
            position2 += " font-weight=\"bold\""
        if italic:
            position2 += " font-style=\"italic\""
        if align != "":
            position2 += " justify=\""+align+"\""
        self.writeDirectionMultiline(where,"words", position1, position2, str(utf8(content)), iVoice, -y , TextHeightMXML(float(fontsiz)))
        
    def handleDrawObjectsSlur(self, drawObjectsElement, iVoice):
        retstring = ""
        drawObjects = drawObjectsElement.getElementsByTagName("drawObj")
        for n in drawObjects:
            if self.isElement(n, "drawObj"):
                retstring += self.handleSlur(n, iVoice)
        return retstring

    def handleDrawObjectsTuplet(self, drawObjectsElement, iVoice):
        retstring = ""
        drawObjects = drawObjectsElement.getElementsByTagName("drawObj")
        for n in drawObjects:
            if self.isElement(n, "drawObj"):
                retstring += self.handleTuplet(n, iVoice)
        return retstring

    def getShortestBase(self, note, noteRange):
        if note.tagName != "chord" and note.tagName != "rest":
            # hängt vielleicht an Taktstrich
            return 0,0
        durationBase, dots = self.getDurationBase(note)
        note = self.nextSiblingElement(note)
        while noteRange > 0 and note != None and (note.tagName == "chord" or note.tagName == "rest"):
            noteRange -= 1
            durationBase1, dots1 = self.getDurationBase(note)
            if durationBase1 > durationBase:
                durationBase = durationBase1
                dots = dots1
            note = self.nextSiblingElement(note)
        return durationBase, dots
    
    def handleSlur(self, n, iVoice):
        retstring = ""
        basic = self.getFirstChildElement(n,"basic")
        if basic != None:
            noteRangeStr = basic.getAttribute("noteRange")
            if noteRangeStr == "":
                noteRange = 0
            else:
                noteRange = int(noteRangeStr)
            for ch in n.childNodes:
                if ch.nodeType == n.ELEMENT_NODE:
                    if ch.tagName == "slur":
                        y1 = float(ch.getAttribute("y1"))
                        y2 = float(ch.getAttribute("y2"))
                        y3 = float(ch.getAttribute("y3"))
                        y4 = float(ch.getAttribute("y4"))
                        if y2 > y1 and y3 > y4:
                            placement = " placement=\"below\""
                        elif y2 < y1 and y3 < y4:
                            placement = " placement=\"above\""
                        else:
                            placement = ""
                        self.MaxSlurNo[iVoice] += 1
                        SlurLabel = 0
                        for sl in self.MaxSlurNo:
                            SlurLabel += sl
                        (self.SlurPending[iVoice])[self.MaxSlurNo[iVoice]] = noteRange, SlurLabel
                        for i in range(self.indentlevel+1):
                            retstring += "\t"
                        retstring += "<slur number=\""+str(SlurLabel)+"\""+placement+" type=\"start\"/>\n"
                    # text, richText, wedge, group, volta werden in handleDrawObj abgehandelt!
        return retstring

    def handleTuplet(self, n, iVoice):
        retstring = ""
        basic = self.getFirstChildElement(n,"basic")
        if basic != None:
            noteRangeStr = basic.getAttribute("noteRange")
            if noteRangeStr == "":
                noteRange = 0
            else:
                noteRange = int(noteRangeStr)
            for ch in n.childNodes:
                if ch.nodeType == n.ELEMENT_NODE:
                    if ch.tagName == "bracket":
                        # hier wird nur die Verwendung als N-Tolen-Klammer unterstützt.
                        shortestBase, dots = self.getShortestBase(n.parentNode.parentNode, noteRange)
                        if shortestBase > 0:
                            self.MaxTupletNo[iVoice] += 1
                            TupletLabel = 0
                            for tup in self.MaxTupletNo:
                                TupletLabel += tup
                            (self.TupletPending[iVoice])[self.MaxTupletNo[iVoice]] = noteRange,TupletLabel,shortestBase,dots
                            for i in range(self.indentlevel+1):
                                retstring += "\t"
                            retstring += "<tuplet number=\""+str(TupletLabel)+"\" type=\"start\"/>\n"
                    # text, richText, wedge, group, volta werden in handleDrawObj abgehandelt!
        return retstring

    def checkSlurEnd(self, iVoice):
        retstring = ""
        slurNo = 1
        while slurNo <= self.MaxSlurNo[iVoice]:
            if (self.SlurPending[iVoice]).has_key(slurNo):
                noteRange, SlurLabel = (self.SlurPending[iVoice])[slurNo]
                if noteRange > 0:
                    noteRange -= 1
                    (self.SlurPending[iVoice])[slurNo] = noteRange, SlurLabel
                    if noteRange == 0:
                        for i in range(self.indentlevel+1):
                            retstring += "\t"
                        retstring += "<slur number=\""+str(SlurLabel)+"\" type=\"stop\"/>\n"
                        # MaxSlurNo ggfls. dekrementieren
                        while slurNo == self.MaxSlurNo[iVoice] and slurNo > 0:
                            if (self.SlurPending[iVoice]).has_key(slurNo):
                                noteRange, SlurLabel = (self.SlurPending[iVoice])[slurNo]
                                if noteRange == 0:
                                    del (self.SlurPending[iVoice])[slurNo]
                                    self.MaxSlurNo[iVoice] -= 1
                            else:
                                break
                            slurNo -= 1
            slurNo += 1
        return retstring

    def checkTupletEnd(self, iVoice):
        retstring = ""
        TupletNo = 1
        while TupletNo <= self.MaxTupletNo[iVoice]:
            if (self.TupletPending[iVoice]).has_key(TupletNo):
                noteRange, TupletLabel, shortestBase, dots = (self.TupletPending[iVoice])[TupletNo]
                if noteRange > 0:
                    noteRange -= 1
                    (self.TupletPending[iVoice])[TupletNo] = noteRange, TupletLabel, shortestBase, dots
                    if noteRange == 0:
                        for i in range(self.indentlevel+1):
                            retstring += "\t"
                        retstring += "<tuplet number=\""+str(TupletLabel)+"\" type=\"stop\"/>\n"
                        # MaxTupletNo ggfls. dekrementieren
                        while TupletNo == self.MaxTupletNo[iVoice] and TupletNo > 0:
                            if (self.TupletPending[iVoice]).has_key(TupletNo):
                                noteRange, TupletLabel, shortestBase, dots = (self.TupletPending[iVoice])[TupletNo]
                                if noteRange == 0:
                                    del (self.TupletPending[iVoice])[TupletNo]
                                    self.MaxTupletNo[iVoice] -= 1
                            else:
                                break
                            TupletNo -= 1
            TupletNo += 1
        return retstring

    def getPendingTupletDurationBase(self, iVoice):
        TupletNo = 1
        while TupletNo <= self.MaxTupletNo[iVoice]:
            if (self.TupletPending[iVoice]).has_key(TupletNo):
                noteRange, TupletLabel, shortestBase, dots = (self.TupletPending[iVoice])[TupletNo]
                return shortestBase, dots
            TupletNo += 1
        return 0,0
    
    def handleWedge(self, ch, basic, iVoice):
        noteRange = 0
        if basic != None:
            noteRangeStr = basic.getAttribute("noteRange")
            if noteRangeStr != "":
                noteRange = int(noteRangeStr)
##            horizAlign = basic.getAttribute("horizAlign")
##            vertAlign = basic.getAttribute("vertAlign")
##            visibilityFlags = basic.getAttribute("visibilityFlags")
        y1 = int(10.0 * float(ch.getAttribute("y1")))
        y2 = int(10.0 * float(ch.getAttribute("y2")))
        # mittlere Linie ist Referenz für CapXML
        if y1<0:
            where = "above"
            y1 += 20
            y2 += 20
        else:
            where = "below"
            y1 += 20
            y2 += 20
        self.MaxWedgeNo[iVoice] += 1
        position = "number=\""+str(self.MaxWedgeNo[iVoice])+"\" default-y=\""+str( -y1) + "\""
        spread = 10 # capella-default = 1.0
        spreadStr = ch.getAttribute("span")
        if spreadStr != "":
            spread = 10.0 * float(spreadStr)
        if "true" == ch.getAttribute("decrescendo"):
            position += " type=\"diminuendo\" spread=\""+str(spread)+"\""
            spread = 0
        else:
            position += " type=\"crescendo\""
        self.writeDirection(where, "wedge", position, "", iVoice)
        # checkWedgeEnd wird bei derselben Note noch einmal gerufen, deshalb noteRange+1!
        (self.WedgePending[iVoice])[self.MaxWedgeNo[iVoice]] = (noteRange+1,spread, y2)

    def checkWedgeEnd(self, iVoice):
        WedgeNo = 1
        while WedgeNo <= self.MaxWedgeNo[iVoice]:
            if (self.WedgePending[iVoice]).has_key(WedgeNo):
                noteRange,spread,y2 = (self.WedgePending[iVoice])[WedgeNo]
                if noteRange > 0:
                    noteRange -= 1
                    (self.WedgePending[iVoice])[WedgeNo] = noteRange,spread,y2
                    if noteRange == 0:
                        position = "number=\""+str(WedgeNo)+"\" default-y=\""+str(-y2)+"\" type=\"stop\""
                        if spread > 0:
                            position += " spread=\""+str(spread)+"\""
                        self.writeDirection("","wedge", position, "", iVoice)
                        # In das "direction"-Element hinter dem "direction-type"-Element evtl. noch ein
                        # offset-element einfügen, denn ein auf diese Weise eingetragener
                        # wedge wird bis exakt an den Anfang der folgenden Note bzw. Pause gemalt, das ist
                        # eigentlich zu weit rechts.
                        #
                        # MaxWedgeNo ggfls. dekrementieren
                        while WedgeNo == self.MaxWedgeNo[iVoice] and WedgeNo > 0:
                            if (self.WedgePending[iVoice]).has_key(WedgeNo):
                                noteRange,spread,y2 = (self.WedgePending[iVoice])[WedgeNo]
                                if noteRange == 0:
                                    del (self.WedgePending[iVoice])[WedgeNo]
                                    self.MaxWedgeNo[iVoice] -= 1
                            else:
                                break
                            WedgeNo -= 1
            WedgeNo += 1
        BracketNo = 1
        while BracketNo <= self.MaxBracketNo[iVoice]:
            if (self.BracketPending[iVoice]).has_key(BracketNo):
                noteRange, BracketLabel, lineend = (self.BracketPending[iVoice])[BracketNo]
                if noteRange > 0:
                    noteRange -= 1
                    (self.BracketPending[iVoice])[BracketNo] = noteRange, BracketLabel, lineend
                    if noteRange == 0:
                        position = "number=\""+str(BracketLabel)+"\" type=\"stop\" line-end="+lineend
                        self.writeDirection("","bracket", position, "", iVoice)
                        # MaxBracketNo ggfls. dekrementieren
                        while BracketNo == self.MaxBracketNo[iVoice] and BracketNo > 0:
                            if (self.BracketPending[iVoice]).has_key(BracketNo):
                                noteRange, BracketLabel, lineend = (self.BracketPending[iVoice])[BracketNo]
                                if noteRange == 0:
                                    del (self.BracketPending[iVoice])[BracketNo]
                                    self.MaxBracketNo[iVoice] -= 1
                            else:
                                break
                            BracketNo -= 1
            BracketNo += 1

    def checkEndingEnd(self, iVoice):
        if self.EndingPending > 0 and iVoice == self.EndingVoice:
            self.EndingPending -= 1
        
    def handleDrawObjects(self, drawObjectsElement, iVoice):
        # Capella-DrawObjects können in MXML Directions oder Notations werden.
        # Directions können sofort geschrieben werden, Notations erst später bei
        # den einzelnen Notenköpfen. Notations sammeln wir also in notationsstr.
        notationsstr = ""
#        drawObjects = drawObjectsElement.getElementsByTagName("drawObj")
        drawObjects = self.getDirectChildrenByTagName(drawObjectsElement, "drawObj")
        for n in drawObjects:
            if self.isElement(n, "drawObj"):
                notationsstr += self.handleDrawObj(n, iVoice)
        return notationsstr

    def handleLyric(self, lyric, iVoice):
        for verse in lyric.childNodes:
            if self.isElement(verse, 'verse'):
                No = verse.getAttribute("i")
                hyphen = verse.getAttribute("hyphen")
#               die Liedtextpositionierung über default-y funktioniert nicht, auch nicht bei den
#               Beispieldateien von Recordare - erst ab Dolet 3.0
                # capella zählt ab Mittellinie, MusicXML ab äußerer Linie, deshalb +2.0
                defY = int(10 * (-self.lyricsFirstLine[iVoice]-2.0))
                verseNumber = verse.getAttribute("i")
                for text in verse.childNodes:
                    if text.nodeType == text.TEXT_NODE:
                        if hyphen == "true":
                            if self.hyphenPending:
                                element = "middle"
                            else:
                                element = "begin"
                                self.hyphenPending = True
                        else:
                            if self.hyphenPending:
                                element = "end"
                                self.hyphenPending = False
                            else:
                                element = "single"
                        lyricdata = self.decodeFromCharset(text.data, self.LyricsCharset)
                        if verseNumber != "":
                            defY -= int(verseNumber) * int(10*self.lyricsDist[iVoice])
                        outstr = utf8(lyricdata)
                        if outstr != "":
                            self.writeNewline("<lyric default-y=\""+str(defY)+"\" number=\"" + str(int(No)+1) + "\">\n")
                            self.indentlevel += 1
                            self.writeElementWithAttributeAndChild("syllabic","",element)
                            self.writeElementWithAttributeAndChild("text","",outstr)
                            self.indentlevel -= 1
                            self.writeNewline("</lyric>\n")
                        break

    def writeAttributes(self, iVoice):
        self.writeNewline("<attributes>\n")
        self.indentlevel += 1
        if not self.DivisionsWritten:
            self.writeSimpleElement("divisions",str(self.arDivisions[self.iSystem]))
            self.Divisions = self.arDivisions[self.iSystem]
            self.DivisionsWritten = True
        if self.changedAttrKey:
            self.writeNewline("<key>\n")
            self.indentlevel += 1
            fifths,mode = self.Key
            self.writeSimpleElement("fifths",fifths)
            self.writeSimpleElement("mode",mode)
            self.indentlevel -= 1
            self.writeNewline("</key>\n")
            self.changedAttrKey = False
        beats,beatType,symbol = self.Time[self.staffIndSystem[iVoice]]
        if self.changedAttrTime:
            if symbol == "senza-misura":
                self.writeNewline("<time>\n")
                self.indentlevel += 1
                self.writeNewline("<senza-misura/>\n")
                self.indentlevel -= 1
                self.writeNewline("</time>\n")
            else:
                if symbol != "":
                    self.writeNewline("<time symbol=\""+symbol+"\">\n")
                else:
                    self.writeNewline("<time>\n")
                self.indentlevel += 1
                self.writeSimpleElement("beats",beats)
                self.writeSimpleElement("beat-type",beatType)
                self.indentlevel -= 1
                self.writeNewline("</time>\n")
            self.changedAttrTime = False
        if len(self.layoutsInPart) > 1:
            self.writeSimpleElement("staves",str(len(self.layoutsInPart)))
        if self.changedAttrClef[self.staffNoPart[iVoice] - 1]:
            staffInd = 0
            for Clef in self.Clefs:
                staffInd += 1
                if len(self.Clefs) > 1:
                    if self.staffNoPart[iVoice] == staffInd:
                        self.writeNewline("<clef number=\"")
                        self.f.write(str(staffInd))
                        self.f.write("\">\n")
                    else:
                        continue
                else:
                    self.writeNewline("<clef>\n")
                self.indentlevel += 1
                sign,line,octaveChange = Clef
                self.writeSimpleElement("sign",sign)
                self.writeSimpleElement("line",line)
                if octaveChange != "0":
                    self.writeSimpleElement("clef-octave-change",octaveChange)
                self.indentlevel -= 1
                self.writeNewline("</clef>\n")
            self.changedAttrClef[self.staffNoPart[iVoice] - 1] = False
        staffind = 0
        staffLinesElement = ""
        if self.staveLineCountNotWritten != 0:
            staffLinesElement = "<staff-lines>%i</staff-lines>"%self.staveLineCountNotWritten
            self.staveLineCountNotWritten = 0;
            pass
        while staffind < len(self.layoutsInPart):
            printstr = "yes"
            if self.stavesmissing[staffind] != "":
                printstr = "no"
            if self.lastPrintState[staffind] != printstr:
                # lastPrintState wird nur hier und in writeHiddenLine geändert.
                # Deshalb braucht es nicht in reset, save und restore behandelt zu werden.
                staffDetailsAttributes = "number=\""+str(staffind+1)+"\" print-object=\""+printstr+"\""
                if staffLinesElement != "":
                    self.writeElementWithAttributeAndChild("staff-details", staffDetailsAttributes,staffLinesElement)
                else:
                    self.writeNewline("<staff-details " + staffDetailsAttributes + "/>\n")
#                self.writeNewline("<staff-details number=\""+str(staffind+1)+"\" print-object=\""+printstr+"\"/>\n")
                self.lastPrintState[staffind] = printstr
            elif staffLinesElement != "":
                self.writeElementWithAttributeAndChild("staff-details", "", staffLinesElement)
                
            staffind +=1
        if self.transposeNotWritten != 0:
            self.writeElementWithAttributeAndChild("transpose","","<chromatic>%i</chromatic>"%self.transposeNotWritten)
            self.transposeNotWritten = 0
        if self.measureRepeatPending == 1:
            self.measureRepeatPending = 0
            self.writeElementWithAttributeAndChild("measure-style","","<measure-repeat type=\"stop\"/>")
        if self.measureRepeat > 0:
            self.writeElementWithAttributeAndChild("measure-style","","<measure-repeat type=\"start\">%i</measure-repeat>"%self.measureRepeat)
            self.measureRepeatPending = self.measureRepeat+1
            self.measureRepeat = 0

        self.indentlevel -= 1
        self.writeNewline("</attributes>\n")

    def areAttributesWritten(self, iVoice):
        if self.changedAttrClef[self.staffNoPart[iVoice] - 1] or self.changedAttrTime or self.changedAttrKey \
           or not self.DivisionsWritten or self.measureRepeat or self.measureRepeatPending == 1:
            return False
        return True

    def resetAttributesWritten(self):
        for cAC in self.changedAttrClef:
            cAC = True
        self.changedAttrTime = True
        self.changedAttrKey = True
        self.DivisionsWritten = False
        self.measureRepeat = 0
        self.measureRepeatPending = 0

    def saveAttributesWritten(self):
        Attributes = {}
        Attributes["caClef"] = self.changedAttrClef
        Attributes["caTime"] = copy.deepcopy((self.changedAttrTime, self.Time))
        Attributes["caKey"] = self.changedAttrKey
        Attributes["divWr"] = self.DivisionsWritten
        Attributes["measRep"] = self.measureRepeat
        Attributes["measRepP"] = self.measureRepeatPending
        return Attributes

    def restoreAttributesWritten(self, Attributes):
        self.changedAttrClef = Attributes["caClef"]
        self.changedAttrTime, self.Time = copy.deepcopy(Attributes["caTime"])        
        self.changedAttrKey = Attributes["caKey"]
        self.DivisionsWritten = Attributes["divWr"]
        self.measureRepeat = Attributes["measRep"]
        self.measureRepeatPending = Attributes["measRepP"]

    def writeNewMeasureStart(self, nonControlling, toHandleIsDefault, multipleRest):
        self.MeasureCount += 1
        if self.measureRepeatPending > 1:
            self.measureRepeatPending -= 1
        if nonControlling:
            mn = "%d"%(self.MeasureNumber + self.MeasureNumberSysOffset)
            if not toHandleIsDefault:
                # wenn es zur gleichen MeasureNumber schon ein Suffix gegeben hat,
                # nehmen wir ein neues.
                Number, Suffix = self.nonControllingSuffix
                suffixes = "ABCDEFG"
                # gibt's keine bequemere Art für alpha-Inkrement?
                if Suffix != "":
                    if Number == self.MeasureNumber:
                        SuffixIndex = suffixes.find(Suffix)
                        if SuffixIndex >= 0 and SuffixIndex+1 < len(suffixes):
                            Suffix = suffixes[SuffixIndex+1:SuffixIndex+2]
                        else:
                            Suffix = suffixes[0:1] # das darf eigentlich nicht vorkommen.
                    else:
                        Suffix = suffixes[0:1]
                else:
                    Suffix = suffixes[0:1]
                self.nonControllingSuffix = self.MeasureNumber, Suffix
                mn += Suffix
            else:
                self.nonControllingSuffix = (0,"")
        else:
            self.nonControllingSuffix = (0,"")
            self.MeasureNumber += 1
            mn = str(self.MeasureNumber + self.MeasureNumberSysOffset)
        self.writeNewline("<measure number=\"")
        self.f.write(mn)
        if nonControlling:
            self.f.write("\" non-controlling=\"yes")
#   dies ist nicht das Mittel der Wahl, um die Anzeige der Taktnumerierung auszuschalten. 
#   Es dient allenfalls dazu, einzelne Takte aus der Numerierung herauszuhalten.
#        if self.HideBarCount:
#            self.f.write("\" implicit=\"yes")
        if multipleRest:
            self.f.write("\" width=\"0")
        self.f.write("\">\n")
        self.indentlevel += 1
        if self.VoltaSaved != None:
            self.handleVolta(self.VoltaSaved, self.getFirstChildElement(self.VoltaSaved, "volta"), self.VoltaSaved_iVoice)
            self.VoltaSaved = None
        iV = 0
        maxDurationsum = 0
        while iV < self.nVoices:
            if self.durationsum[iV] > maxDurationsum:
                maxDurationsum = self.durationsum[iV]
            self.durationsum[iV] = 0
            self.alterPending[iV] = {}
            self.explicitBar[iV] = False
            iV += 1
        self.durationsumSystem += maxDurationsum

    def writeNewMeasureEnd(self):
        if self.EndingPending == 0:
            self.writeVoltaStopBarline(self.EndingNumber, self.EndingDiscontinue)
            self.EndingPending = -1
            self.EndingVoice = -1
        self.indentlevel -= 1
        self.writeNewline("</measure>\n")

    def writeNewMeasure(self, nonControlling, toHandleIsDefault):
        self.writeNewMeasureEnd()
        self.writeNewMeasureStart(nonControlling, toHandleIsDefault, False)

    def writeRestMeasure(self):
        self.writeNewMeasureEnd()
        self.writeNewMeasureStart(False, False, True)

    def writeSystem1Margin(self):
        self.writeNewline("<print>\n")
        self.indentlevel += 1
        self.writeSystem1Layout( self.marginSys1)
        self.indentlevel -= 1
        self.writeNewline("</print>\n")
        self.System1MarginWritten = True
        
    def writeDefaultSystemLayout(self, margin, ydist, topydist):
        self.writeNewline("<system-layout>\n")
        self.indentlevel += 1
        self.writeNewline("<system-margins>\n")
        self.indentlevel += 1
        self.writeSimpleElement("left-margin",str(margin))
        self.writeSimpleElement("right-margin","0")
        self.indentlevel -= 1
        self.writeNewline("</system-margins>\n")
        self.writeSimpleElement("system-distance", str(ydist))
        self.writeSimpleElement("top-system-distance", str(topydist))
        self.indentlevel -= 1
        self.writeNewline("</system-layout>\n")

    def writeSystem1Layout(self, margin):
        self.writeNewline("<system-layout>\n")
        self.indentlevel += 1
        self.writeNewline("<system-margins>\n")
        self.indentlevel += 1
        # Automatische Berechnung des Einzugs berücksichtigt die Länge
        # der Stimmenbezeichnung.
        if self.leftIndent != -1:
            margin = self.leftIndent * 10
        self.writeSimpleElement("left-margin",str(margin))
        margin = 0
        if self.rightIndent != -1:
            margin = self.rightIndent * 10
        self.writeSimpleElement("right-margin",str(margin))
        self.indentlevel -= 1
        self.writeNewline("</system-margins>\n")
        # nur die Hälfte von getSystemDist:
        tiStaff0 = self.TemplateIndex[self.StavesInSystem[0]]
        top,bottom = self.StaffDist[tiStaff0]
        topOff, bottomOff = self.StaffDistOffset[0]
        sysdist = top + topOff
        self.writeSimpleElement("top-system-distance", str(10 * sysdist + self.ydistSys1))
        self.indentlevel -= 1
        self.writeNewline("</system-layout>\n")
        if len(self.StavesInSystem) > 1:
            self.writeStaffLayouts()

    def writeLineBreak(self):
        self.writeLineBreakLayout()
#        if self.partIndex == 0:
#            self.writeLineBreakLayout()
#        else:
#            self.writeNewline("<print new-system=\"yes\"/>\n")
        
    def writeLineBreakLayout(self):
##      print new-page hat im capella-Import Nachbesserungen gefordert.
##      Deshalb wollen wir es hier zunächst noch inaktiv lassen. bj160114
##        if self.pageBreakPending:
##            self.writeNewline("<print new-page=\"yes\">\n")
##            self.pageBreakPending = False
##        else:
##            self.writeNewline("<print new-system=\"yes\">\n")
        self.writeNewline("<print new-system=\"yes\">\n")
        
        self.indentlevel += 1
        self.writeNewline("<system-layout>\n")
        self.indentlevel += 1

        self.writeNewline("<system-margins>\n")
        self.indentlevel += 1
        if self.leftIndent == -1:
            # Automatik: Stimmenbezeichnung berücksichtigen!
            if self.instrNotation == "short":
                self.writeSimpleElement("left-margin", str(self.marginShort))
            elif self.instrNotation == "long":
                self.writeSimpleElement("left-margin", str(self.marginLong))
            else:
                self.writeSimpleElement("left-margin", str(0))
        else:
            self.writeSimpleElement("left-margin", str(self.leftIndent * 10))
        rightMargin = 0
        if not self.SystemJustified:
#            todo: Eine Schätzung für den Platzbedarf des Systems durchführen
#            SystemExtent = ???
#            rightMargin = self.pagesizeX - self.pagemarginRight - self.pagemarginLeft - SystemExtent
            pass
        elif self.rightIndent != -1:
            rightMargin = self.rightIndent * 10
        self.writeSimpleElement("right-margin", str(rightMargin))
        self.indentlevel -= 1
        self.writeNewline("</system-margins>\n")
        
        ydist = self.getSystemDist() + self.DefaultSystemDistance
        self.writeSimpleElement("system-distance", str(10 * ydist))
        self.indentlevel -= 1
        self.writeNewline("</system-layout>\n")
        if len(self.StavesInSystem) > 1:
            self.writeStaffLayouts()
        self.indentlevel -= 1
        self.writeNewline("</print>\n")

    def getSystemDist(self):
        tiStaff0 = self.TemplateIndex[self.StavesInSystem[0]]
        tiStaffLast = self.TemplateIndex[self.StavesInSystem[-1]]
        top,bottom = self.StaffDist[tiStaff0]
        topL,bottomL = self.StaffDist[tiStaffLast]
        topOff, bottomOff = self.StaffDistOffset[0]
        if not self.isFirstSystem:  # sollte immer erfüllt sein.
            topOffL, bottomOffL = self.previousStaffDistOffset[-1]
        else:
            topOffL = 0
            bottomOffL = 0
        return top + topOff + bottomL + bottomOffL

    def writeStaffLayouts(self):
        StaffNo = 0
        CapStaffInd = -1
        for layoutstring in self.StavesInSystem:
            CapStaffInd += 1
            if layoutstring in self.layoutsInPart:
                top, bottom = self.StaffDist[self.TemplateIndex[layoutstring]]
                topOff, bottomOff = self.StaffDistOffset[CapStaffInd]
                if CapStaffInd > 0:
                    prevTopOff, prevBottomOff = self.StaffDistOffset[CapStaffInd-1]
                    topOff += prevBottomOff
                    prevTop, prevBottom = self.StaffDist[self.TemplateIndex[self.StavesInSystem[CapStaffInd-1]]]
                    topOff += prevBottom
                StaffNo += 1
                self.writeStaffLayout(StaffNo, 10 * (top+topOff))

    def writeStaffLayout(self, staffNo, ydist):
        self.writeNewline("<staff-layout number=\""+str(staffNo)+"\">\n")
        self.indentlevel += 1
        if ydist <= 0:
            # wenn Finale hier 0 vorfindet, nimmt es seinen Default.
            ydist = 1
        self.writeSimpleElement("staff-distance", str(ydist))
        self.indentlevel -= 1
        self.writeNewline("</staff-layout>\n")

    def writeBackup(self, duration):
        self.writeNewline("<backup>\n")
        self.indentlevel += 1
        self.writeSimpleElement("duration",str(int(duration)))
        self.indentlevel -= 1
        self.writeNewline("</backup>\n")
        
    def writeForward(self, duration, iVoice):
        self.writeNewline("<forward>\n")
        self.indentlevel += 1
        self.writeSimpleElement("duration",str(int(duration)))
        self.writeSimpleElement("voice",str(self.DestVoice[iVoice]))
        if self.StavesInPart > 1:
            self.writeSimpleElement("staff",str(self.staffNoPart[iVoice]))
        self.indentlevel -= 1
        self.writeNewline("</forward>\n")
        
    def isMeasureComplete(self, iVoice):
        if self.explicitBar[iVoice]:
            return True
        beats, beatType, symbol = self.Time[self.staffIndSystem[iVoice]]
        if symbol == "senza-misura":
            return False
        measurelength = self.arDivisions[self.iSystem] * int(beats) * 4 / int(beatType)
        return self.durationsum[iVoice] >= measurelength

    def fitsInMeasure(self, iVoice, duration):
        beats, beatType, symbol = self.Time[self.staffIndSystem[iVoice]]
        measurelength = self.arDivisions[self.iSystem] * int(beats) * 4 / int(beatType)
        return self.durationsum[iVoice] + duration <= measurelength
    
    def getMXMLaccidental(self, note, alter, iVoice):
        fifths, mode = self.Key
        alterstr, alteroffs = self.accidentalRules(note[0:1], alter, int(fifths))
#        alterstr, alteroffs = self.AccidentalRules[(note[0:1], alter, int(fifths))]
        if (self.alterPending[iVoice]).has_key(note):
            # in diesem Takt gab es schon ein Vorzeichen
            alterP = (self.alterPending[iVoice])[note]
            if alterP != 0:
                if alterP == alteroffs:
                    return ""
                elif (alterP == 1 or alterP == -1) and alteroffs == 0:
                    del (self.alterPending[iVoice])[note]
                    if alter == 0:
                        return "natural"
                    elif alter == -1:
                        return "flat"
                    elif alter == 1:
                        return "sharp"
                elif alteroffs == 1 and alterP == -1:
                    (self.alterPending[iVoice])[note] = alteroffs
                    return "natural-sharp"
                elif alteroffs == -1 and alterP == 1:
                    (self.alterPending[iVoice])[note] = alteroffs
                    return "natural-flat"
                elif alteroffs == -2 and alterP == -1:
                    (self.alterPending[iVoice])[note] = alteroffs
                    return "flat-flat"
                elif alteroffs == 2 and alterP == 1:
                    (self.alterPending[iVoice])[note] = alteroffs
                    return "sharp-sharp"
                elif alteroffs == 1 and alterP == 2:
                    (self.alterPending[iVoice])[note] = alteroffs
                    return "sharp"
                elif alteroffs == -1 and alterP == -2:
                    (self.alterPending[iVoice])[note] = alteroffs
                    return "flat"
        if alteroffs != 0:
            (self.alterPending[iVoice])[note] = alteroffs
        return alterstr
    
    def getMXMLarticulation(self, text):
        retstring = ""
        articulationString = ""
        if text == "staccato":
            articulationString = "<staccato/>"
        elif text == "tenuto":
            articulationString = "<tenuto/>"
        elif text == "staccatissimo":
            articulationString = "<staccatissimo/>"
        elif text == "normalAccent":
            articulationString = "<accent/>"
        elif text == "strongAccent":
            articulationString = "<strong-accent/>"
        elif text == "weakBeat":
            pass
        elif text == "strongBeat":
            pass
        elif text == "staccato tenuto":
            articulationString = "<staccato/>\n"
            for i in range(self.indentlevel+3):
                articulationString += "\t"
            articulationString += "<tenuto/>"
        if articulationString != "":
            for i in range(self.indentlevel+2):
                retstring += "\t"
            retstring += "<articulations>\n"
            for i in range(self.indentlevel+3):
                retstring += "\t"
            retstring += articulationString + "\n"
            for i in range(self.indentlevel+2):
                retstring += "\t"
            retstring += "</articulations>\n"
        return retstring

    def accidentalRules(self, chSrc, iAlter, iKey):
        # self.accidentalRules("F",0,-6) liefert: "",0, d.h.
        # Ein F, das in Ges Dur in capella mit alter="0" gemeldet wird,
        # kriegt in MusicXML kein Vorzeichen und keine Tonhöhenänderung gegenüber
        # dem nach Tonart normalen Ton.
        defaultAlter = self.AlterDefault[(chSrc, iKey)]
        AlterShift = iAlter - defaultAlter
        if AlterShift == 0:
            return "",0
        return self.AlterOutput[(defaultAlter, AlterShift)], AlterShift
        

    def startScore(self, score):
        self.CodecFromCharset = {}
        self.CodecFromCharset["238"] = "cp1250" # east europe
        self.CodecFromCharset["204"] = "cp1251" # cyrillic
        self.CodecFromCharset["0"] = "cp1252"   # ansi
        self.CodecFromCharset["1"] = "cp1252"   # default
        self.CodecFromCharset["2"] = "cp1252"   # symbol map to default
        self.CodecFromCharset["161"] = "cp1253" # greek
        self.CodecFromCharset["162"] = "cp1254" # turkish
        self.CodecFromCharset["177"] = "cp1255" # hebrew
        self.CodecFromCharset["178"] = "cp1256" # arabic
        self.CodecFromCharset["186"] = "cp1257" # baltic
        self.CodecFromCharset["128"] = "cp1252"   # map to default
        self.CodecFromCharset["129"] = "cp1252"   # map to default
        self.CodecFromCharset["134"] = "cp1252"   # SimSun map to default
        self.CodecFromCharset["136"] = "cp1252"   # map to default

        self.AlterDefault = {\
("F",0):0, ("F",1):1, ("F",2):1, ("F",3):1, ("F",4):1, ("F",5):1, ("F",6):1, ("F",7):1,\
("F",-1):0, ("F",-2):0, ("F",-3):0, ("F",-4):0, ("F",-5):0, ("F",-6):0, ("F",-7):-1,\
("C",0):0, ("C",1):0, ("C",2):1, ("C",3):1, ("C",4):1, ("C",5):1, ("C",6):1, ("C",7):1,\
("C",-1):0, ("C",-2):0, ("C",-3):0, ("C",-4):0, ("C",-5):0, ("C",-6):-1, ("C",-7):-1,\
("G",0):0, ("G",1):0, ("G",2):0, ("G",3):1, ("G",4):1, ("G",5):1, ("G",6):1, ("G",7):1,\
("G",-1):0, ("G",-2):0, ("G",-3):0, ("G",-4):0, ("G",-5):-1, ("G",-6):-1, ("G",-7):-1,\
("D",0):0, ("D",1):0, ("D",2):0, ("D",3):0, ("D",4):1, ("D",5):1, ("D",6):1, ("D",7):1,\
("D",-1):0, ("D",-2):0, ("D",-3):0, ("D",-4):-1, ("D",-5):-1, ("D",-6):-1, ("D",-7):-1,\
("A",0):0, ("A",1):0, ("A",2):0, ("A",3):0, ("A",4):0, ("A",5):1, ("A",6):1, ("A",7):1,\
("A",-1):0, ("A",-2):0, ("A",-3):-1, ("A",-4):-1, ("A",-5):-1, ("A",-6):-1, ("A",-7):-1,\
("E",0):0, ("E",1):0, ("E",2):0, ("E",3):0, ("E",4):0, ("E",5):0, ("E",6):1, ("E",7):1,\
("E",-1):0, ("E",-2):-1, ("E",-3):-1, ("E",-4):-1, ("E",-5):-1, ("E",-6):-1, ("E",-7):-1,\
("B",0):0, ("B",1):0, ("B",2):0, ("B",3):0, ("B",4):0, ("B",5):0, ("B",6):0, ("B",7):1,\
("B",-1):-1, ("B",-2):-1, ("B",-3):-1, ("B",-4):-1, ("B",-5):-1, ("B",-6):-1, ("B",-7):-1}
        # (defaultAlter, AlterShift)
        self.AlterOutput = {(0,1):"sharp", (0,2):"double-sharp", (0,-1):"flat", (0,-2):"double-flat",
                            (1,1):"double-sharp", (1,-1):"natural", (1,-2):"natural-flat",
                            (-1,1):"natural", (-1,-1):"double-flat", (-1,2):"natural-sharp"}
                            


        self.ExportLineBreak = True
        self.indentlevel = 0;
        self.writeNewline("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n")
        self.writeNewline("<!--MusicXML 2.0 file created by "+globalVersionString+"-->\n")
        self.writeNewline("<!DOCTYPE score-partwise PUBLIC \"-//Recordare//DTD MusicXML 2.0 Partwise//EN\" \"http://www.musicxml.org/dtds/partwise.dtd\">\n")
        self.writeNewline("<score-partwise version=\"2.0\">\n")
        self.indentlevel += 1
        self.writeNewline("<identification>\n")
        self.indentlevel += 1
        self.writeNewline("<encoding>\n")
        self.indentlevel += 1
        self.writeNewline("<encoding-date>")
        self.f.write(time.strftime("%Y-%m-%d",time.localtime()))
        self.f.write("</encoding-date>\n")
        self.writeNewline("<software>")
        self.f.write(globalVersionString)
        self.f.write("</software>\n")
        global globalFinaleDolet33
        global globalChordCaseMatters
        global globalExportToSibelius
        optionstring = "Options: FinaleDolet33=%d, ChordCaseMatters=%d, ExportToSibelius=%d"\
                       %(globalFinaleDolet33,globalChordCaseMatters,globalExportToSibelius)
        self.writeSimpleElement("encoding-description", optionstring)
        self.indentlevel -= 1
        self.writeNewline("</encoding>\n")
        self.indentlevel -= 1
        self.writeNewline("</identification>\n")
        layout = self.getFirstChildElement(score,"layout")
        barCount = self.getFirstChildElement(score,"barCount")
        self.HideBarCount = barCount == None
        # grafische Aspekte des Mustersystems
        systems = self.getFirstChildElement(score,"systems")
        firstNameStyle, othersNameStyle = self.peekInstrNotations(systems)
        self.handleDistances(layout, firstNameStyle, othersNameStyle)
        pageObjects = self.getFirstChildElement(score,"pageObjects")
        CreditWritten = False
        if pageObjects != None:
            CreditWritten = self.handlePageObjects(pageObjects)
        if not CreditWritten:
            # Leeres credit-Feld vermeidet "[Titel]" und "[Komponist]"
            self.writeNewline("<credit>\n")
            self.indentlevel += 1
            self.writeNewline("<credit-words></credit-words>\n")
            self.indentlevel -= 1
            self.writeNewline("</credit>\n")
# CapXML-Layout -> MusicXML-part-list
        self.handleLayout(layout, firstNameStyle, othersNameStyle)

    def peekInstrNotations(self, systems):
        first = ""
        # falls mindestens ein System ausführlichere Bezeichnungen
        # trägt, übernehmen wir sie für Finale - dort können wir
        # nur zwischen erstem und den anderen Systemen differenzieren
        others = "none"
        for system in systems.childNodes:
            if self.isElement(system, 'system'):
                instrNotation = system.getAttribute("instrNotation")
                if instrNotation == "":
                    instrNotation = "none"
                if first == "":
                    first = instrNotation
                else:
                    if instrNotation == "long":
                        others = instrNotation
                        break
                    elif instrNotation == "short":
                        if others == "none":
                            others = instrNotation
        return first,others

    def maxVoices(self, score, layoutstr):
        # zählt in jedem System der Partitur die Stimmen in der Zeile mit description layoutstr
        # und liefert das Maximum zurück
        maxVoicecount = 0
        for n in score.childNodes:
            if self.isElement(n, 'systems'):
                for s in n.childNodes:
                    if self.isElement(s, 'system'):
                        for vs in s.childNodes:
                            if self.isElement(vs, 'staves'):
                                for v in vs.childNodes:
                                    if self.isElement(v, 'staff') and v.getAttribute('layout') == layoutstr:
                                        voicecount = 0
                                        for cs in v.childNodes:
                                            if self.isElement(cs, 'voices'):
                                                for c in cs.childNodes:
                                                    if self.isElement(c, 'voice'):
                                                        voicecount += 1
                                        if maxVoicecount < voicecount:
                                            maxVoicecount = voicecount
        return maxVoicecount
    
    def handleScore(self, score):
        self.DefaultDurationsDict = {}
        self.chordstyle = {}
        self.startScore(score)
        self.BarlineSaved = None
        self.VoltaSaved = None
        self.VoltaSaved_iVoice = -1
        self.partIndex = 0
        self.arDivisions = []
        # index 0 nicht genutzt, wird mit self.iSystem indiziert 1,2,...!
        self.arDivisions.append(0)
        for (self.layoutsInPart, id, transpose, self.staveLineCount) in self.ScorePartIDs:
            self.writeNewline("<part id=\"")
            self.f.write(id)
            self.f.write("\">\n")
            self.indentlevel += 1
            self.Key = ("0","major")
            self.Clefs = []
            self.changedAttrClef = []
            self.lastPrintState = []
            self.firstVoiceIndex = {}
            VoiceIndex = 1;
            for layoutstr in self.layoutsInPart:
                self.lastPrintState.append("yes")
                self.changedAttrClef.append(True)
                self.Clefs.append(("G","2","0"))
                self.firstVoiceIndex[layoutstr] = VoiceIndex
                if len(self.layoutsInPart) > 1:
                    VoiceIndex += self.maxVoices(score, layoutstr)
            self.stavesmissing = []
            self.changedAttrKey = True
            self.changedAttrTime = True
            self.measureRepeat = 0
            self.measureRepeatPending = 0
            self.MeasureCount = -1
            self.MeasureNumber = -1
            self.MeasureNumberSysOffset = 0
            self.nVoices = 0
            self.isFirstSystem = True
            self.System1MarginWritten = False
            self.hyphenPending = False
            self.iSystem = 0
            self.Divisions = 0
            self.DivisionsWritten = False
            self.transposeNotWritten = transpose
            self.oldtempo = []
            self.TimeLastSystem = []
            if self.staveLineCount != 5:
                self.staveLineCountNotWritten = self.staveLineCount
            else:
                self.staveLineCountNotWritten = 0
            for n in score.childNodes:
                if self.isElement(n, 'systems'):
                    for s in n.childNodes:
                        if self.isElement(s, 'system'):
                            self.iSystem += 1
                            if len(self.arDivisions) == self.iSystem:
                                # wir bauen self.arDivisions nur im ersten part auf.
                                self.arDivisions.append(480)
                            self.Time = []
                            self.handleSystem(s)
                            self.TimeLastSystem = self.Time
                            self.isFirstSystem = False
            self.indentlevel -= 1
            self.writeNewline("</part>\n")
            self.partIndex += 1

        self.finishScore(score)

    def finishScore(self, score):
        self.f.write("</score-partwise>\n")

    def startSystem(self, sys):
        self.beamGrouping = 3
        strGrouping = sys.getAttribute("beamGrouping")
        if strGrouping != "":
            self.beamGrouping = int(strGrouping)
        self.SystemJustified = False
        if "true" == sys.getAttribute("justified"):
            self.SystemJustified = True
            # for now, we don't implement support for unjustified systems
            # self.SystemJustified = False
        self.leftIndent = -1
        indentstr = sys.getAttribute("leftIndent")
        if indentstr != "":
            self.leftIndent = int(indentstr)
        self.rightIndent = -1
        if self.SystemJustified:
            indentstr = sys.getAttribute("rightIndent")
            if indentstr != "":
                self.rightIndent = int(indentstr)
        self.instrNotation = sys.getAttribute("instrNotation")   # "none", "short", "long"
        tempostr = sys.getAttribute("tempo")
        self.tempo = self.mkTempo(tempostr)
##        if len(self.oldtempo) == 0:
##            self.oldtempo = self.tempo
##        self.pageBreakPending = False
##        if "true" == sys.getAttribute("pageBreak"):
##            self.pageBreakPending = True

    def mkTempo(self,tempostr):
        tempo = []
        if tempostr != "":
            iEnd1=tempostr.find(' ')
            while iEnd1 >= 0:
                tempo.append(int(float(tempostr[0:iEnd1])))
                tempostr = tempostr[iEnd1+1:]
                iEnd1=tempostr.find(' ')
            tempo.append(int(float(tempostr)))
        return tempo

    def getStavesToHandle(self, sys, stavesToHandle):
        self.StavesInPart = 0
        countStavesInSystem = 0
        if not self.isFirstSystem:
            self.previousStaffDistOffset = self.StaffDistOffset
        self.StaffDistOffset = []
        for n in sys.childNodes:
            if self.isElement(n, 'staves'):
                for s in n.childNodes:
                    if self.isElement(s, 'staff'):
                        layoutstr = s.getAttribute("layout")
                        if layoutstr in self.layoutsInPart:
                            stavesToHandle.append((s,countStavesInSystem))
                            self.StavesInPart += 1
#                        myMessage("getStavesToHandle",str(self.Clefs))
                        countStavesInSystem += 1
                        self.StavesInSystem.append(layoutstr)
                        defaultTime = s.getAttribute("defaultTime")
                        changedAttrTime = self.changedAttrTime # Seiteneffekt von handleTimeString kompensieren
##                        AttributesWrittenSav = self.saveAttributesWritten()  # Seiteneffekt von handleTimeString kompensieren
                        self.Time.append(("","",""))
                        self.handleTimeString(defaultTime,-1, False)
                        self.changedAttrTime = changedAttrTime # Seiteneffekt von handleTimeString kompensieren
##                        self.restoreAttributesWritten(AttributesWrittenSav)
                        if len(self.TimeLastSystem) >= countStavesInSystem:
                            if self.TimeLastSystem[countStavesInSystem-1] != self.Time[countStavesInSystem-1]:
                                self.changedAttrTime = True
                        extraD = self.getFirstChildElement(s,"extraDistance")
                        top = 0
                        bottom = 0
                        if extraD != None:
                            topstr = extraD.getAttribute("top")
                            if topstr != "":
                                top = int(topstr)
                            bottomstr = extraD.getAttribute("bottom")
                            if bottomstr != "":
                                bottom = int(bottomstr)
                        self.StaffDistOffset.append((top,bottom))

    def handleSystem(self, sys):
        self.startSystem(sys)
        self.nonControllingSuffix = (0,"")
        stavesToHandle = [] # (staff, staffIndexSystem)
        self.StavesInSystem = []
        self.getStavesToHandle(sys, stavesToHandle)
        DefaultDurations = []
        DefaultDurationsToHandle = []
        isControllingToHandle = []
        SpecialBars = []
        DefaultDurationsumSystem, toHandleIsDefault = self.getDefaultInfo(sys, stavesToHandle,
                        DefaultDurations, DefaultDurationsToHandle, isControllingToHandle, SpecialBars)
        if self.Divisions != self.arDivisions[self.iSystem]:
            # wir schreiben die divisions nur wenn nötig
            self.DivisionsWritten = False
        self.MeasureNumberSysOffset += self.MeasureNumber+1
        self.MeasureCount = -1
        self.MeasureNumber = -1
        barCount = self.getFirstChildElement(sys,"barCount")
        if barCount != None and self.MeasureNumberSysOffset == 0:
            # Dolet erlaubt nur einfach ansteigende Taktnummern in einer Datei.
            # Nur im ersten System kann eine Startnummer interpretiert werden.
            nAdd = barCount.getAttribute("nAdd")
            if nAdd != "" and int(nAdd) > 0:
                self.MeasureNumberSysOffset += int(nAdd)
            reset = barCount.getAttribute("reset")
            if reset == "true":
                self.MeasureNumberSysOffset = 0
        if self.MeasureNumberSysOffset == 0:
            if len(DefaultDurationsToHandle) > 1 and DefaultDurationsToHandle[0] >= DefaultDurationsToHandle[1]:
                # Auftakt hat in Finale üblicherweise Nummer 0
                self.MeasureNumberSysOffset += 1
        self.durationsumSystem = 0.0
        self.handleSelectedStaves(stavesToHandle, DefaultDurations, DefaultDurationsToHandle,
                DefaultDurationsumSystem, isControllingToHandle, toHandleIsDefault, SpecialBars)
        self.finishSystem(sys)

    def getDefaultInfo(self, sys, stavesToHandle, DefaultDurations, DefaultDurationsToHandle, isControllingToHandle, SpecialBars):
        # durations für jeden Takt sammeln falls Mustersystemzeile im System fehlt
        # Dabei die Metren in den Zeilen vergleichen. Im Falle von Ungleichheiten
        # werden DefaultDurations und DefaultDurationsToHandle unterschiedlich, und
        # isControllingToHandle notiert diejenigen Taktstriche, die für alle Zeilen
        # gelten.
        self.durationsum = []
        self.explicitBar = []
        DefaultDurationss = []
        DurationSums = []
        RestsOnly = []
        SpecialBarss = []
        noteObjectss = []
        noteObjectssToHandle = []
        self.staffIndSystem = []
        staves = self.getFirstChildElement(sys, 'staves')
        staffs = self.getDirectChildrenByTagName(staves, 'staff')
        staffcount = -1
        for staff in staffs:
            staffcount += 1
            # ist diese Zeile gerade zu bearbeiten?
            toHandle = False
            for staffToHandle, iSt in stavesToHandle:
                if staffToHandle == staff:
                    toHandle = True
            voices = self.getFirstChildElement(staff, 'voices')
            vs = self.getDirectChildrenByTagName(voices, 'voice')
            for voice in vs:
                noteObjects = self.getFirstChildElement(voice, 'noteObjects')
                noteObjectssToHandle.append(toHandle)
                noteObjectss.append(noteObjects)
                self.staffIndSystem.append(staffcount)
        # getDefaultDurations kostet deutlich Zeit! Deswegen rufen wir es nur 1mal pro System und halten
        # die Ergebnisse für die anderen parts in einem Dictionary zur Verfügung.
        if self.DefaultDurationsDict.has_key(self.iSystem):
            DefaultDurationss, DurationSums, RestsOnly, SpecialBarss = self.DefaultDurationsDict[self.iSystem]
        else:
            self.getDefaultDurations(noteObjectss, DefaultDurationss, DurationSums, RestsOnly, SpecialBarss)
            self.DefaultDurationsDict[self.iSystem] = DefaultDurationss, DurationSums, RestsOnly, SpecialBarss
        # Finde aus DefaultDurationss die am weitesten besetzte Stimme heraus
        # Falls im System mehrere gleichweit besetzte Stimmen mit unterschiedlichen
        # Metren existieren, wähle eine derjenigen mit den längsten Takten.
        minlen = 0
        countMinlen = 0
        bestDDIndex = -1
        bestDurationSumIndex = -1
        bestDurationSum = 0
        bestOwnDurationSum = 0
        bestOwnIndex = -1
        for i in range(len(DefaultDurationss)):
            DD = DefaultDurationss[i]
            DurationSum = DurationSums[i]
            thisLen = len(DD)
            if thisLen > 0 and DurationSum >= bestDurationSum:
                if DurationSum > bestDurationSum or minlen > thisLen and not RestsOnly[i]:
                    # beim Erkennen von ungleichen Metren wollen wir uns nicht durch falsch
                    # aufgefüllte Leerzeilen narren lassen (2_Volkslieder.cap)
                    bestDDIndex = i
                    minlen = thisLen
                    bestDurationSumIndex = i
                    bestDurationSum = DurationSum
            if noteObjectssToHandle[i]:
                if bestOwnDurationSum < DurationSum:
                    bestOwnDurationSum = DurationSum
                    bestOwnIndex = i                
        if bestDDIndex >= 0:
            bestDD = DefaultDurationss[bestDDIndex]
            for duration in bestDD:
                DefaultDurations.append(duration)
            # Feste Taktstriche für nicht vorhandene Zeilen:
            # aus der am besten gefüllten Stimme übernehmen
            # Voltenklammern aus der obersten vorhandenen Stimme übernehmen.
            for iMeas in range(len(SpecialBarss[bestDDIndex])):
                bardict = SpecialBarss[bestDDIndex][iMeas]
                if len(SpecialBarss[0]) > iMeas:
                    bardictV = SpecialBarss[0][iMeas]
                    if bardictV.has_key("volta"):
                        bardict["volta"] = bardictV["volta"]
                SpecialBars.append(bardict)
        if len(DefaultDurations) == 0:
            DefaultDurations.append(0)
        if bestOwnIndex >= 0:
            bestDD = DefaultDurationss[bestOwnIndex]
            for duration in bestDD:
                DefaultDurationsToHandle.append(duration)
        if bestOwnDurationSum < bestDurationSum:
            # was ist, wenn die zu bearbeitenden Stimmen insgesamt unvollständig sind -
            # d.h. kürzer als die längste Stimme des Systems?
            # Falls die Takte gleichlang sind, ist es kein Problem - wir füllen einfach von
            # der längsten Stimme auf.
            sameMetrum = True
            if len(DefaultDurationsToHandle) > 0:
                for i in range(len(DefaultDurationsToHandle)):
                    if i >= len(DefaultDurations) or DefaultDurationsToHandle[i] != DefaultDurations[i]:
                        sameMetrum = False
            if sameMetrum:
                for i in range(len(DefaultDurationsToHandle), len(DefaultDurations)):
                    DefaultDurationsToHandle.append(DefaultDurations[i])
            else:
                # bei unterschiedlichen Metren füllen wir mit dem längsten bisher vorgekommenen Metrum auf.
                # Das ist Code für Partituren wie DonGiov195, wenn sie unvollständig wären.
                maxDuration = 0
                for duration in DefaultDurationsToHandle:
                    if maxDuration < duration:
                        maxDuration = duration
                if DefaultDurationsToHandle[-1] < maxDuration:
                    addToLast = maxDuration - DefaultDurationsToHandle[-1]
                    if bestOwnDurationSum + addToLast > bestDurationSum:
                        addToLast = bestDurationSum - bestOwnDurationSum
                    DefaultDurationsToHandle[-1] += addToLast
                    bestOwnDurationSum += addToLast
                while bestOwnDurationSum < bestDurationSum:
                    addMeasure = maxDuration
                    if bestOwnDurationSum + addMeasure > bestDurationSum:
                        addMeasure = bestDurationSum - bestOwnDurationSum
                    DefaultDurationsToHandle.append(addMeasure)
                    bestOwnDurationSum += addMeasure
        # nur wenn alle Taktstriche übereinander sind, ist der Taktstrich "controlling"
        OwnDurationSum = 0.0
        for duration in DefaultDurationsToHandle:
            isControllingToHandle.append(True)
            OwnDurationSum += duration
            for i in range(len(DefaultDurationss)):
                if OwnDurationSum < DurationSums[i]:
                    DD = DefaultDurationss[i]
                    DurationSum = 0.0
                    found = False
                    for dur in DD:
                        DurationSum += dur
                        if DurationSum == OwnDurationSum:
                            found = True
                            break;
                        elif DurationSum > OwnDurationSum:
                            break;
                    if not found:
                        isControllingToHandle[-1] = False
                        break;
        # Wenn ein part die DefaultDurations hat, kann man sich bei der
        # Taktnummer das "A" sparen
        sameMetrum = len(DefaultDurationsToHandle) == len(DefaultDurations)
        if sameMetrum:
            for i in range(len(DefaultDurationsToHandle)):
                if DefaultDurationsToHandle[i] != DefaultDurations[i]:
                    sameMetrum = False
                    break        
        return bestDurationSum, sameMetrum
            
    def getDefaultDurations(self, noteObjectss, DefaultDurationss, DurationSums, RestsOnly, SpecialBarss):
        AttributesWrittenSav = self.saveAttributesWritten()   # Seiteneffekt von handleTimeString kompensieren
        self.durationsum = []
        self.explicitBar = []
        # Zunächst erfassen wir alle capella-durations des Systems, um die MusicXML divisions
        # zu bestimmen.
        kgv = KGV()
        durationRatD = {}
        for noteObjects in noteObjectss:
            VoiceIndex = noteObjectss.index(noteObjects)
            staff = noteObjects.parentNode.parentNode.parentNode
            if staff != None:
                defaultTime = staff.getAttribute("defaultTime")
                if defaultTime != "":
                    self.handleTimeString(defaultTime, self.staffIndSystem[VoiceIndex], False)
            for n in noteObjects.childNodes:
                if self.isElement(n):
                    isChord = self.isElement(n, 'chord')
                    if isChord or self.isElement(n, 'rest'):
                        if isChord:
                            durationRat,numerator,denominator = self.duration(n)
                        else:
                            durationRat,numerator,denominator,wholeMeasure = self.durationRest(n, VoiceIndex)
                        durationRatD[n] = durationRat
                        kgv.add(durationRat.q)
                    elif self.isElement(n, 'timeSign'):
                        self.handleTimeSign(n, self.staffIndSystem[VoiceIndex])
            divisions4 = kgv.getkgv()
            # hier können wir noch die alte (bis Version 1.10) divisions von pauschal 480
            # aktivieren.
            useVariableDivisions = True
            if useVariableDivisions and 480 % divisions4 != 0:
                if divisions4 % 4 != 0:
                    # falls nur ganze oder halbe Noten/Pausen da waren
                    self.arDivisions[self.iSystem] = divisions4
                else:
                    self.arDivisions[self.iSystem] = divisions4 / 4
        # Jetzt können wir die divisions so setzen, dass jedes Tupel aufgeht.
        self.restoreAttributesWritten(AttributesWrittenSav)  # Seiteneffekt von handleTimeString kompensieren
        for noteObjects in noteObjectss:
            VoiceIndex = noteObjectss.index(noteObjects)
            staff = noteObjects.parentNode.parentNode.parentNode
            if staff != None:
                defaultTime = staff.getAttribute("defaultTime")
                if defaultTime != "":
                    self.handleTimeString(defaultTime, self.staffIndSystem[VoiceIndex], False)
            self.durationsum.append(0)
            self.explicitBar.append(False)
            DefaultDurationss.append([])
            DurationSums.append(0.0)
            RestsOnly.append(True)
            SpecialBarss.append([])
            SpecialBarsThisMeasure = {}
            afterTime = False
            afterKey = False
            for n in noteObjects.childNodes:
                if self.isElement(n):
                    isChord = self.isElement(n, 'chord')
                    if isChord or self.isElement(n, 'rest'):
                        durationBase, dots = self.getDurationBase(n)
##                        if isChord:
##                            durationRat,numerator,denominator,wholeMeasure = self.duration(n)
##                        else:
##                            durationRat,numerator,denominator,wholeMeasure = self.durationRest(n, VoiceIndex)
                        durationRat = durationRatD[n]
                        duration = durationRat.__float__() * 4 * self.arDivisions[self.iSystem]
                        self.durationsum[VoiceIndex] += duration
                        DurationSums[VoiceIndex] += duration
                        if isChord:
                            RestsOnly[VoiceIndex] = False
                        # ist ein volta-Element dran?
                        drawObjects = self.getFirstChildElement(n, "drawObjects")
                        if drawObjects != None:
                            drawObjs = self.getDirectChildrenByTagName(drawObjects, "drawObj")
                            volta = None
                            for dO in drawObjs:
                                volta = self.getFirstChildElement(dO, "volta")
                                if volta != None:
                                    SpecialBarsThisMeasure["volta"] = volta,dO
                    elif self.isElement(n, 'barline'):
                        if self.durationsum[VoiceIndex] > 0:
                            self.explicitBar[VoiceIndex] = True
                            SpecialBarsThisMeasure["right"] = n
                        else:
                            SpecialBarsThisMeasure["left"] = n
                    elif self.isElement(n, 'timeSign'):
                        self.handleTimeSign(n, self.staffIndSystem[VoiceIndex])
                        if SpecialBarsThisMeasure.has_key("left"):
                            afterTime = True
                    elif self.isElement(n, 'keySign'):
                        if SpecialBarsThisMeasure.has_key("left"):
                            afterKey = True
                    if self.isMeasureComplete(VoiceIndex):
                        afterTime = False
                        afterKey = False
                        DefaultDurationss[VoiceIndex].append(self.durationsum[VoiceIndex])
                        self.durationsum[VoiceIndex] = 0
                        self.explicitBar[VoiceIndex] = False
                        SpecialBarss[VoiceIndex].append(SpecialBarsThisMeasure)
                        SpecialBarsThisMeasure = {}
            if self.durationsum[VoiceIndex] > 0 or (afterTime or afterKey) and SpecialBarsThisMeasure.has_key("left"):
                DefaultDurationss[VoiceIndex].append(self.durationsum[VoiceIndex])
                SpecialBarss[VoiceIndex].append(SpecialBarsThisMeasure)
        self.restoreAttributesWritten(AttributesWrittenSav)  # Seiteneffekt von handleTimeString kompensieren
        
    def startSelectedStaves(self, staves):
        self.nVoice = 0
        self.outbeamStarted = 0

    def handleSelectedStaves(self, stavesToHandle, DefaultDurations,
                             DefaultDurationsToHandle, DefaultDurationsumSystem, isControllingToHandle,
                             toHandleIsDefault, SpecialBars):
        self.startSelectedStaves(stavesToHandle)
        voices = []
        self.nVoices = 0
        self.durationsum = []
        self.alterPending = []
        self.explicitBar = []
        self.lyricsFirstLine = []
        self.lyricsDist = []
        self.stemDir = []
        self.MaxSlurNo = []
        self.SlurPending = []
        self.MaxTupletNo = []
        self.TupletPending = []
        self.MaxWedgeNo = []
        self.WedgePending = []
        self.MaxBracketNo = []
        self.BracketPending = []
        self.EndingPending = -1
        self.EndingNumber = ""
        self.EndingVoice = -1
        # alle voices der zu bearbeitenden Zeilen kommen in das voices-Array.
        # Das gleichlange staffIndSystem-Array hält die zugehörigen staff-Indices.
        self.staffIndSystem = []
        self.staffNoPart = []
        self.DestVoice = []
        staffcount = 0
        destVoice = 1
        for staff, staffIndexSystem in stavesToHandle:
            staffcount += 1
            defaultTime = staff.getAttribute("defaultTime")
            if defaultTime != "":
                self.handleTimeString(defaultTime, staffIndexSystem, False)
            for n in staff.childNodes:
                if self.isElement(n, 'voices'):
                    for s in n.childNodes:
                        if self.isElement(s, 'voice'):
                            voices.append(s)
                            self.durationsum.append(0)
                            self.alterPending.append({})
                            self.explicitBar.append(False)
                            self.staffIndSystem.append(staffIndexSystem)
                            self.staffNoPart.append(staffcount)
                            if destVoice < self.firstVoiceIndex[staff.getAttribute("layout")]:
                                destVoice = self.firstVoiceIndex[staff.getAttribute("layout")]
                            self.DestVoice.append(destVoice)
                            destVoice += 1
                            self.MaxSlurNo.append(0)
                            self.SlurPending.append({})
                            self.MaxTupletNo.append(0)
                            self.TupletPending.append({})
                            self.MaxWedgeNo.append(0)
                            self.WedgePending.append({})
                            self.MaxBracketNo.append(0)
                            self.BracketPending.append({})
                            stemDir = s.getAttribute("stemDir")
                            if stemDir == "none":
                                stemDir = ""
                            self.stemDir.append(stemDir)
                            lyricsSettings = self.getFirstChildElement(s, "lyricsSettings")
                            if lyricsSettings != None:
                                firstLine = lyricsSettings.getAttribute("firstLine")
                                lineDist = lyricsSettings.getAttribute("lineDist")
                                self.lyricsFirstLine.append(float(firstLine))
                                self.lyricsDist.append(float(lineDist))
                                lyricsFont = self.getFirstChildElement(lyricsSettings, "font")
                                if lyricsFont != None:
                                    charSet = lyricsFont.getAttribute("charSet")
                                    if charSet != "":
                                        self.LyricsCharset = charSet
                                    else:
                                        self.LyricsCharset = "0"
                            else:
                                self.lyricsFirstLine.append(1.5)
                                self.lyricsDist.append(10.0)
                            self.nVoices += 1
        self.handleVoices(voices, DefaultDurations, DefaultDurationsToHandle,
                          DefaultDurationsumSystem, isControllingToHandle, toHandleIsDefault, SpecialBars)
        self.finishSelectedStaves(stavesToHandle)

    def finishSelectedStaves(self, staves):
        pass
        
    def finishSystem(self, sys):
#        self.f.write('\n')
        pass

    def startVoice(self, vc):
        pass

    def chooseNextVoice(self, s):
        chosen = -1
        iV = 0
        while iV < self.nVoices:
            if not self.isMeasureComplete(iV):
                chosen = iV
                break
            
            iV += 1
        return chosen

    def allVoicesDone(self, s):
        for obj in s:
            if obj != None:
                return False
        return True

    def handleVoiceFirstMeasure(self, voice, iVoice):
        for n in voice.childNodes:
            if self.isElement(n, 'noteObjects'):
                for s in n.childNodes:
                    if self.isElement(s):
                        self.handleNoteObj(s, iVoice)
                        n.removeChild(s)
                        s.unlink()
                        if self.isMeasureComplete(iVoice):
                            return
    
    def iterateOnce(self, iters, objs, iVhandle):
        objs[iVhandle] = myNext(iters[iVhandle])
        while not objs[iVhandle] == None and not self.isElement(objs[iVhandle]):
            objs[iVhandle] = myNext(iters[iVhandle])

    def nextSiblingElement(self, node):
        while node.nextSibling != None:
            if self.isElement(node.nextSibling):
                return node.nextSibling
            else:
                node = node.nextSibling
        return None

    def nothingleftButKeyTimeClefBar(self, node):
        while node != None:
            node = self.nextSiblingElement(node)
            if node == None:
                return True
            if self.isElement(node,"clefSign") or self.isElement(node,"keySign") or self.isElement(node,"timeSign") or self.isElement(node,"barline"):
                pass
            else:
                return False
                

    def handleVoices(self, voices, DefaultDurations, DefaultDurationsToHandle,
                     DefaultDurationsumSystem, isControllingToHandle, toHandleIsDefault, SpecialBars):
        self.writeNewMeasureStart(False, True, False)
        if self.ExportLineBreak and not self.isFirstSystem:
            self.writeLineBreak()
        if len(self.tempo) > 0 and self.tempo != self.oldtempo:
            self.writeTempoDirection(self.tempo)
            self.oldtempo = self.tempo
        if self.nVoices > 0:
            iVoice = 0
            iters = []
            # für jede Stimme, die noteObjects enthält, einen Iterator bereitstellen
            while iVoice < self.nVoices:
                for n in voices[iVoice].childNodes:
                    if self.isElement(n, 'noteObjects'):
                        iters.append(iter(n.childNodes))
                        break
                iVoice += 1
            # für jede im aktuellen System fehlende Zeile des Mustersystems
            # das print-object="no"-Attribut schreiben
            stavesmissing = []
            for layoutstr in self.layoutsInPart:
                stavesmissing.append(layoutstr)
            for voice in voices:
                myVoices = voice.parentNode
                myStaff = myVoices.parentNode
                myLayout = myStaff.getAttribute("layout")
                for i in range(len(stavesmissing)):
                    if myLayout == stavesmissing[i]:
                        stavesmissing[i] = ""
            if str(self.stavesmissing) != str(stavesmissing):
                self.resetAttributesWritten()
                self.stavesmissing = stavesmissing
                
            iVoice = 0
            objs = []
            # Für jede Stimme das erste Notenobjekt holen. Eins auswählen.
            while iVoice < self.nVoices:
                objs.append(myNext(iters[iVoice]))
                while objs[iVoice] != None and not self.isElement(objs[iVoice]):
                    objs[iVoice] = myNext(iters[iVoice])
                iVoice += 1
            iVhandle = 0
            # Schleife: Ausgewähltes Notenobjekt exportieren, Nachfolger in
            # seiner Stimme holen, wieder Notenobjekt auswählen.
            keyAmEnd = None
            self.TimeSignSaved = None
            self.timeAmEndStaff = -1
            while iVhandle < self.nVoices:
                if objs[iVhandle] != None:
                    wasMultipleRest = self.handleNoteObj(objs[iVhandle], iVhandle)
                    self.iterateOnce(iters, objs, iVhandle)
                    if self.isMeasureComplete(iVhandle):
                        if objs[iVhandle] != None:
                            # in jeder Stimme prüfen wir auf Nachschlag
                            if self.isElement(objs[iVhandle],"chord"):
                                duration = self.getFirstChildElement(objs[iVhandle],"duration")
                                if "true" == duration.getAttribute("noDuration"):
                                    display = self.getFirstChildElement(objs[iVhandle],"display")
                                    if display != None:
                                        if "true" == display.getAttribute("postGrace"):
                                            self.handleChord(objs[iVhandle], iVhandle)
                                            self.iterateOnce(iters, objs, iVhandle)
                            # in jeder Stimme prüfen wir auf Schlüsselwechsel
                            if objs[iVhandle] != None and self.isElement(objs[iVhandle],"clefSign"):
                               # if self.nothingleftButKeyTimeClefBar(objs[iVhandle]):
                                    # am Ende der Zeile!
                                    self.handleClefSign(objs[iVhandle], iVhandle)
                                    self.iterateOnce(iters, objs, iVhandle)
                            # in jeder Stimme prüfen wir auf Tonartwechsel. Den dürfen wir aber erst
                            # mit handleKeySign zu Schreiben anmelden, wenn der Takt in allen Stimmen
                            # fertiggeschrieben ist.
                            # in capella braucht jede Stimme ihr keySign (Designschwäche!),
                            # in MusicXML gilt ein Key für einen ganzen Part (ggfls. mehrere staves!)
                            if objs[iVhandle] != None and self.isElement(objs[iVhandle],"keySign"):
                               # if self.nothingleftButKeyTimeClefBar(objs[iVhandle]):
                                    keyAmEnd = objs[iVhandle]
                                    self.iterateOnce(iters, objs, iVhandle)
                            # in jeder Stimme prüfen wir auf Taktwechsel
                            if objs[iVhandle] != None and self.isElement(objs[iVhandle],"timeSign"):
                               # if self.nothingleftButKeyTimeClefBar(objs[iVhandle]):
                                    # am Ende der Zeile!
                                    self.TimeSignSaved = objs[iVhandle]
                                    self.timeAmEndStaff = self.staffIndSystem[iVhandle]
                                    self.iterateOnce(iters, objs, iVhandle)
                            # in der letzten Stimme prüfen wir auf expliziten Taktstrich
                            if objs[iVhandle] != None and self.isElement(objs[iVhandle],"barline"):
                                if self.isCAPX2:
                                    self.checkEndingEnd(iVhandle)
                                    NotationsFromDrawObjectsString = ""
                                    drawObjects = self.getFirstChildElement(objs[iVhandle], "drawObjects")
                                    if drawObjects != None:
                                        NotationsFromDrawObjectsString = self.handleDrawObjects(drawObjects, iVhandle)
                                        drawObj = self.getVolta(objs[iVhandle])
                                        if drawObj != None:
                                            self.VoltaSaved = drawObj
                                            self.VoltaSaved_iVoice = iVhandle
                                if iVhandle == self.nVoices - 1:
                                    if self.isLeftBarline(objs[iVhandle]):
                                        self.BarlineSaved = objs[iVhandle]
                                    if self.isRightBarline(objs[iVhandle]):
                                        self.handleBarline(objs[iVhandle], self.durationsum[iVhandle] == 0)
                                        type = objs[iVhandle].getAttribute("type")
                                        if type == "repEnd" or type == "repEndBegin":
                                            # darf in diesem Fall nicht im Folgetakt wirksam werden.
                                            keyAmEnd = None
                                            self.TimeSignSaved = None
                                self.iterateOnce(iters, objs, iVhandle)
                        if iVhandle + 1 < self.nVoices:
                            if not self.areAttributesWritten(iVhandle):
                                # hier wollen wir (Tonart, Schlüssel, Metrum)-Wechsel schreiben, wenn sie
                                # am Ende der Zeile auftraten. Vorsicht: Diese Dinge sind in capella
                                # eigentlich nur für eine Stimme definiert, gelten in MusicXML und
                                # im gedruckten Notenbild aber für eine Zeile!
                                self.writeAttributes(iVhandle)
                            if self.durationsum[iVhandle] > 0 and not wasMultipleRest:
                                self.writeBackup(self.durationsum[iVhandle])
                            iVhandle = iVhandle + 1
                        elif len(DefaultDurationsToHandle) > self.MeasureCount and (not self.allVoicesDone(objs) or DefaultDurationsumSystem > self.durationsumSystem + self.durationsum[iVhandle]):
                            if isControllingToHandle[self.MeasureCount]:
                                self.writeNewMeasure(False, toHandleIsDefault)
                            else:
                                self.MeasureNumber = LastIndexLowerEqual(DefaultDurations, self.durationsum[iVhandle]+self.durationsumSystem)
                                self.writeNewMeasure(True, toHandleIsDefault)
                            iVhandle = 0
                            if self.TimeSignSaved != None or keyAmEnd != None:
                                if keyAmEnd != None:
                                    self.handleKeySign(keyAmEnd)
                                    keyAmEnd = None
                                if self.TimeSignSaved != None:
                                    self.handleTimeSign(self.TimeSignSaved, self.timeAmEndStaff)
                                    self.TimeSignSaved = None
                                if not self.areAttributesWritten(iVhandle): # ist das hier nötig?
                                    changedAttrTime = self.changedAttrTime
                                    if not self.System1MarginWritten and iVhandle == 0:
                                        self.writeSystem1Margin()
                                    self.writeAttributes(iVhandle)
                                    self.changedAttrTime = changedAttrTime
                                
                    elif objs[iVhandle] == None:
                        # unvollständiger Takt in dieser Stimme
                        if not self.ExportLineBreak:
                            # unvollständigen Takt aus nächstem System vervollständigen
                            nextSystemVoice = self.peekNextSystemVoice(voices[iVhandle],iVhandle)
                            if nextSystemVoice != None:
                                self.handleVoiceFirstMeasure(nextSystemVoice, iVhandle)
                            if iVhandle + 1 < self.nVoices:
                                if self.durationsum[iVhandle] > 0:
                                    self.writeBackup(self.durationsum[iVhandle])
                                iVhandle += 1
                        else:
                            if len(DefaultDurationsToHandle) > self.MeasureCount:
                                thisDuration = DefaultDurationsToHandle[self.MeasureCount] - self.durationsum[iVhandle]
                            else:
                                thisDuration = 0
                            if thisDuration > 0:
                                # in einer anderen Stimme dieses Systems gibt's noch mehr. Auffüllen.
                                if not self.areAttributesWritten(iVhandle):
                                    if not self.System1MarginWritten and iVhandle == 0:
                                        self.writeSystem1Margin()
                                    self.writeAttributes(iVhandle)
                                self.writeForward(thisDuration, iVhandle)
                                self.durationsum[iVhandle] += int(thisDuration)
                            if iVhandle + 1 < self.nVoices:
                                if self.durationsum[iVhandle] > 0:
                                    self.writeBackup(self.durationsum[iVhandle])
                                iVhandle = iVhandle + 1
                            elif len(DefaultDurationsToHandle) > self.MeasureCount and (not self.allVoicesDone(objs) or DefaultDurationsumSystem > self.durationsumSystem + self.durationsum[iVhandle]):
                                if isControllingToHandle[self.MeasureCount]:
                                    self.writeNewMeasure(False, toHandleIsDefault)
                                else:
                                    self.MeasureNumber = LastIndexLowerEqual(DefaultDurations, self.durationsum[iVhandle]+self.durationsumSystem)
                                    self.writeNewMeasure(True, toHandleIsDefault)
                                iVhandle = 0
                else:
                    if self.TimeSignSaved != None or keyAmEnd != None:
                        if keyAmEnd != None:
                            self.handleKeySign(keyAmEnd)
                            keyAmEnd = None
                        if self.TimeSignSaved != None:
                            self.handleTimeSign(self.TimeSignSaved, self.timeAmEndStaff)
                            self.TimeSignSaved = None
                        if self.durationsum[iVhandle] > 0:
                            if not self.areAttributesWritten(iVhandle):
                                changedAttrTime = self.changedAttrTime
                                if not self.System1MarginWritten and iVhandle == 0:
                                    self.writeSystem1Margin()
                                self.writeAttributes(iVhandle)
                                self.changedAttrTime = changedAttrTime
                    if self.durationsum[iVhandle] > 0:
                        # wir sind fertig mit dem System
                        iVhandle += 1
                    else:
                        # leerer Takt in dieser Stimme
                        if not self.areAttributesWritten(iVhandle):
                            changedAttrTime = self.changedAttrTime
                            if not self.System1MarginWritten and iVhandle == 0:
                                self.writeSystem1Margin()
                            self.writeAttributes(iVhandle)
                            self.changedAttrTime = changedAttrTime
                        if len(DefaultDurationsToHandle) > self.MeasureCount:
                            # eine andere Stimme des Systems hat irgendwelche Objekte
                            thisDuration = DefaultDurationsToHandle[self.MeasureCount]
                        else:
                            thisDuration = 0                        
                        if thisDuration == 0 and globalExportToSibelius:
                            thisDuration = self.arDivisions[self.iSystem]        # Lex Sibelius experimental
                        if thisDuration > 0:
                            self.writeForward(thisDuration, iVhandle)
                            self.durationsum[iVhandle] += int(thisDuration)
                        if iVhandle + 1 < self.nVoices:
                            if self.durationsum[iVhandle] > 0:
                                self.writeBackup(self.durationsum[iVhandle])
                            iVhandle = iVhandle + 1
                        elif iVhandle+1 == self.nVoices and len(DefaultDurationsToHandle) > self.MeasureCount and (not self.allVoicesDone(objs) or DefaultDurationsumSystem > self.durationsumSystem + self.durationsum[iVhandle]):
                            if isControllingToHandle[self.MeasureCount]:
                                self.writeNewMeasure(False, toHandleIsDefault)
                            else:
                                self.MeasureNumber = LastIndexLowerEqual(DefaultDurations, self.durationsum[iVhandle]+self.durationsumSystem)
                                self.writeNewMeasure(True, toHandleIsDefault)
                            iVhandle = 0
                        else:
                            iVhandle += 1
            # für Noten ohne Takt den in Finale+Sibelius leider notwendigen Taktstrich unsichtbar machen!
            beats,beatType,symbol = self.Time[self.staffIndSystem[-1]]
            if symbol == "senza-misura":
                self.writeElementWithAttributeAndChild("barline","location=\"right\"","<bar-style>none</bar-style>")
        else:
            self.writeHiddenLine(DefaultDurationsToHandle, isControllingToHandle, SpecialBars)
        self.writeNewMeasureEnd()

    def writeHiddenLine(self, DefaultDurations, isControlling, SpecialBars):
        measureCount = 0
        self.writeNewline("<attributes>\n")
        self.indentlevel += 1
        self.writeSimpleElement("divisions",str(self.arDivisions[self.iSystem]))
        beats,beatType,symbol = self.Time[0]
        if symbol == "senza-misura":
            self.writeElementWithAttributeAndChild("time","","<senza-misura/>")
        else:
            if symbol != "":
                self.writeNewline("<time symbol=\""+symbol+"\">\n")
            else:
                self.writeNewline("<time>\n")
            self.indentlevel += 1
            self.writeSimpleElement("beats",beats)
            self.writeSimpleElement("beat-type",beatType)
            self.indentlevel -= 1
            self.writeNewline("</time>\n")
        if self.StavesInPart > 1:
            self.writeSimpleElement("staves",str(self.StavesInPart))
        staffind = 0
        while staffind < len(self.layoutsInPart):
            self.lastPrintState[staffind] = "no"
            staffind +=1
            self.writeNewline("<staff-details number=\""+str(staffind)+"\" print-object=\"no\"/>\n")
        self.indentlevel -= 1
        self.writeNewline("</attributes>\n")
        savedBarline = None
        voltaDurationPending = 0
        while measureCount < len(DefaultDurations): # == len(SpecialBars)
            sp = SpecialBars[measureCount]
            if sp.has_key("left"):
                barline = sp["left"]
            else:
                barline = savedBarline
            savedBarline = None
            if barline != None and self.isLeftBarline(barline):
                self.handleBarline(barline, True)
            if sp.has_key("volta"):
                volta, drawObj = sp["volta"]
                self.handleVolta(drawObj, volta, 0)
                # um die Klammer wieder schließen zu können, müssen wir aus der ursprünglichen
                # Stimme die Duration der geklammerten Notenobjekte bestimmen. Das sind self.EndingPending
                # Stück.
                countVoltaChordRest = self.EndingPending
                thischordrest = drawObj.parentNode.parentNode
                noteObjects = thischordrest.parentNode
                chordrests = self.getDirectChildrenBy2TagNames(noteObjects, "chord","rest")
                found = False
                voltaDurationPending = 0
                for chordrest in chordrests:
                    if found:
                        if self.isElement(chordrest, "chord"):
                            voltaDurationPending += self.duration(chordrest)[0]
                            countVoltaChordRest -= 1
                        elif self.isElement(chordrest, "rest"):
                            voltaDurationPending += self.durationRest(chordrest,0)[0]
                            countVoltaChordRest -= 1
                        if countVoltaChordRest <= 0:
                            break
                    if chordrest == thischordrest:
                        found = True
            duration = DefaultDurations[measureCount]
            measureCount += 1
            self.writeNewline("<note>\n")
            self.indentlevel += 1
            self.writeNewline("<rest/>\n")
            self.writeSimpleElement("duration",str(int(duration)))
            self.indentlevel -= 1
            self.writeNewline("</note>\n")
            voltaDurationPending -= duration
            if voltaDurationPending <= 0 and self.EndingPending > 0:
                self.EndingPending = 0  # Mechanismus in writeNewMeasureEnd() anstoßen!
            if sp.has_key("right"):
                barline = sp["right"]
                if self.isRightBarline(barline):
                    self.handleBarline(barline, False)
            if measureCount < len(DefaultDurations):
                if SpecialBars[measureCount].has_key("left"):
                    barline = SpecialBars[measureCount]["left"]
                    if self.isRightBarline(barline):
                        self.handleBarline(barline, False)
                    if self.isLeftBarline(barline):
                        savedBarline = barline
                self.writeNewMeasure(not isControlling[measureCount-1], True)
        self.resetAttributesWritten()
        
    def finishVoice(self, voice):
#        self.f.write('\n')
        pass

    def handleNoteObj(self, obj, iVoice):
        wasMultipleRest = False
        self.startNoteObj(obj)
        if self.isElement(obj, 'chord'):
            if self.BarlineSaved != None:
                self.handleBarline(self.BarlineSaved, self.durationsum[iVoice] == 0)
                self.BarlineSaved = None
            if not self.areAttributesWritten(iVoice):
                if not self.System1MarginWritten and iVoice == 0:
                    self.writeSystem1Margin()
                self.writeAttributes(iVoice)
            self.handleChord(obj, iVoice)
            self.checkEndingEnd(iVoice)
        elif self.isElement(obj, 'rest'):
            if self.isFullMeasureRest(obj):
                hasR, nR = self.hasRepeat(obj)
                if hasR:
                    self.measureRepeat = nR
                elif self.isCAPX2:
                    nextObj = self.nextSiblingElement(obj)
                    if nextObj and self.isElement(nextObj, 'barline'):
                        hasR, nR = self.hasRepeat(nextObj)
                        if hasR:
                            self.measureRepeat = nR
                        
            if self.BarlineSaved != None:
                self.handleBarline(self.BarlineSaved, self.durationsum[iVoice] == 0)
                self.BarlineSaved = None
            if not self.areAttributesWritten(iVoice):
                if not self.System1MarginWritten and iVoice == 0:
                    self.writeSystem1Margin()
                self.writeAttributes(iVoice)
            wasMultipleRest = self.handleRest(obj, iVoice)
            self.checkEndingEnd(iVoice)
        elif self.isElement(obj, 'barline'):
            if self.durationsum[iVoice] > 0:
                self.explicitBar[iVoice] = True
            if self.isCAPX2:
                self.checkEndingEnd(iVoice)
                NotationsFromDrawObjectsString = ""
                drawObjects = self.getFirstChildElement(obj, "drawObjects")
                if drawObjects != None:
                    NotationsFromDrawObjectsString = self.handleDrawObjects(drawObjects, iVoice)
                    drawObj = self.getVolta(obj)
                    if drawObj != None:
                        self.VoltaSaved = drawObj
                        self.VoltaSaved_iVoice = iVoice
            if iVoice == self.nVoices - 1:
                if self.isLeftBarline(obj):
                    self.BarlineSaved = obj
                if self.isRightBarline(obj):
                    self.handleBarline(obj, self.durationsum[iVoice] == 0)
        elif self.isElement(obj, 'clefSign'):
            if iVoice == 0 or self.staffIndSystem[iVoice] != self.staffIndSystem[iVoice-1]:
                self.handleClefSign(obj, iVoice)
                ## in capella braucht jede Stimme ihren clef (Designschwäche!).
                ## Wir nehmen nur den aus der ersten Stimme jeder Zeile.
        elif self.isElement(obj, 'keySign'):
            if  iVoice == 0:
                if self.durationsum[iVoice] > 0:
                    self.writeBarline( "double", False)
                    self.explicitBar[iVoice] = True
                self.handleKeySign(obj)
                # in capella braucht jede Stimme ihr keySign (Designschwäche!),
                # in MusicXML gilt ein Key für einen ganzen Part (ggfls. mehrere staves!)
        elif self.isElement(obj, 'timeSign'):
            nextObj = self.nextSiblingElement(obj)
            if nextObj == None:
                self.TimeSignSaved = obj
                self.timeAmEndStaff = self.staffIndSystem[iVoice]
            else:
                self.handleTimeSign(obj, self.staffIndSystem[iVoice])
        return wasMultipleRest

    def isLeftBarline(self, obj):
        type = obj.getAttribute("type")
        if type == "repBegin" or type == "repEndBegin":
            return True
        return False

    def isRightBarline(self, obj):
        type = obj.getAttribute("type")
        if type == "repBegin":
            return False
        return True

    def handleBarline(self, obj, left):
        # nur ein barline Element pro Part an jedem Taktrand
        type = obj.getAttribute("type")
        self.writeBarline(type, left)

    def writeBarline(self, type, left):
        # nur ein barline Element pro Part an jedem Taktrand
        if left:
            self.writeNewline("<barline location=\"left\">\n")
        else:
            self.writeNewline("<barline>\n")
        self.indentlevel += 1
        if type == "single":
            typeMXML = "regular"
        elif type == "double":
            typeMXML = "light-light"
        elif type == "end":
            typeMXML = "light-heavy"
        elif type == "repEnd":
            typeMXML = "light-heavy"
        elif type == "repBegin":
            typeMXML = "heavy-light"
        elif type == "repEndBegin":
            typeMXML = "heavy-heavy"
        else:
            typeMXML = "regular"
        self.writeElementWithAttributeAndChild("bar-style","",typeMXML)
        # EndingPending == 0 bedeutet: bei nächster barline oder am Schluss
        # des Taktes VoltaStop schreiben
        Pend = self.EndingPending
        if self.EndingPending == 0:
            self.writeVoltaStop(self.EndingNumber, self.EndingDiscontinue)
            self.EndingPending = -1
            self.EndingVoice = -1
        if type == "repBegin":
            self.writeNewline("<repeat direction=\"forward\"/>\n")
        elif type == "repEnd":
            self.writeNewline("<repeat direction=\"backward\"/>\n")
        elif type == "repEndBegin":
            if left:
                self.writeNewline("<repeat direction=\"forward\"/>\n")
            else:
                self.writeNewline("<repeat direction=\"backward\"/>\n")
        self.indentlevel -= 1
        self.writeNewline("</barline>\n")
        
    def startChord(self, chord):
        pass

    def isFullMeasureRest(self, obj):
        durat = self.getFirstChildElement(obj, 'duration')
        return 0 > durat.getAttribute('base').find('/')

    def hasRepeat(self, obj):
        drawObjects = self.getFirstChildElement(obj, "drawObjects")
        if drawObjects:
            textList = drawObjects.getElementsByTagName("text")
            for n in textList:
                if self.isElement(n, "text"):
                    font = self.getFirstChildElement(n, "font")
                    fontface = font.getAttribute("face")
                    if fontface == "capella3":
                        content = self.getFirstChildElement(n, "content")
                        for tn in content.childNodes:
                            if tn.nodeType == n.TEXT_NODE:
                                if isFaulenzer1(tn.data):
                                    return True, 1
                                elif isFaulenzer2(tn.data):
                                    return True, 2                
        return False, 0

    def getDurationBase(self, obj):
        durat = self.getFirstChildElement(obj, 'duration')
        n = Rational(str(durat.getAttribute('base')))
        dots = durat.getAttribute('dots')
        ndots = 0
        if dots != "":
            if dots == "1":
                ndots = 1
            elif dots == "2":
                ndots = 2
        return n.q,ndots      

    def handleChord(self, obj, iVoice):
        NotationsFromDrawObjectsString = ""
        drawObjects = self.getFirstChildElement(obj, "drawObjects")
        if drawObjects != None:
            NotationsFromDrawObjectsString = self.handleDrawObjects(drawObjects, iVoice)
        tempo = self.mkTempo(obj.getAttribute("tempo"))
        if len(tempo) > 0:
            self.writeTempoDirection(tempo)
            self.tempo = tempo
        durationBase, dots = self.getDurationBase(obj)
        durationRat,numerator,denominator = self.duration(obj)
        duration = durationRat.__float__() * 4 * self.arDivisions[self.iSystem]
        self.durationsum[iVoice] += duration
        display = self.getFirstChildElement(obj, "display")
        articulationString = ""
        articulation = self.getFirstChildElement(obj, "articulation")
        if articulation != None:
            articulationType = articulation.getAttribute("type")
            articulationString = self.getMXMLarticulation(articulationType)
        otherstaff = ""
        invisible = False
        postgrace = False
        if display != None:
            invisible = "true" == display.getAttribute("invisible")
            otherstaff = display.getAttribute("notationStaff")
            if otherstaff == "none":
                otherstaff = ""
            postgrace = "true" == display.getAttribute("postGrace")
        if invisible:
            invistring = " print-object=\"no\""
        else:
            invistring = ""
        if invisible and globalExportToSibelius:
            # Sibelius 4 unterstützt keine Unsichtbaren Noten
            self.writeForward(duration, iVoice)
        else:
            stem = self.getFirstChildElement(obj, "stem")
            if stem != None:
                dir = stem.getAttribute("dir")
            elif self.stemDir[iVoice] != "":
                dir = self.stemDir[iVoice]
            else:
                dir = ""
            small = False
            display = self.getFirstChildElement(obj, "display")
            if display != None:
                if "true" == display.getAttribute("small"):
                    small = True
            isFirstHead = True
            for n in obj.childNodes:
                if self.isElement(n, 'heads'):
                    for s in n.childNodes:
                        if self.isElement(s, 'head'):
                            self.writeNewline("<note%s>\n" % invistring)
                            self.indentlevel += 1
                            tie = self.getFirstChildElement(s, "tie")
                            if duration == 0 and not small:
                                # capella erlaubt auch normalgroße Noten ohne Wert. MusicXML nicht.
                                small = True
                            if small:
                                if duration == 0:
                                    bars = ""
                                    if stem != None:
                                        bars = stem.getAttribute("tremoloBars")
                                    self.writeNewline("<grace slash=\"")
                                    if bars == "":
                                        self.f.write("no\"")
                                    else:
                                        self.f.write("yes\"")
                                    if postgrace:
                                        self.f.write(" steal-time-previous=\"2\"")
                                    self.f.write("/>\n")
                                elif tie == None:
                                    self.writeNewline("<cue/>\n")
                                else:
                                    # "cue notes have no ties" (note.dtd)
                                    # wir rechnen mit einem Erkennungsfehler und unterdrücken die small-Eigenschaft.
                                    pass
                            if not isFirstHead:
                                self.writeNewline("<chord/>\n")
                            self.writeNewline("<pitch>\n")
                            self.indentlevel += 1
                            # write pitch element
                            strng = s.getAttribute("pitch")
                            self.writeSimpleElement("step",strng[0:1])
                            alter = self.getFirstChildElement(s, "alter")
                            if alter != None:
                                step = alter.getAttribute("step")
                                if step != "":
                                    self.writeSimpleElement("alter",step)
                                display = alter.getAttribute("display")
                            else:
                                display = ""                                
                            octave = int(strng[1:2]) - 1
                            self.writeSimpleElement("octave",str(octave))
                            self.indentlevel -= 1
                            self.writeNewline("</pitch>\n")
                            # 
                            if duration != 0:
                                self.writeSimpleElement("duration",str(int(duration)))
                            # Haltebogen klingen lassen
                            if tie != None:
                                tieStart = "true" == tie.getAttribute("begin")
                                tieStop = "true" == tie.getAttribute("end")
                                if tieStart:
                                    self.writeNewline("<tie type=\"start\"/>\n")
                                if tieStop:
                                    self.writeNewline("<tie type=\"stop\"/>\n")
                            else:
                                # merken fürs Malen
                                tieStart = False
                                tieStop = False
                            # Stimmenzugehörigkeit wird bei Import über dolet light nicht ausgewertet
                            self.writeSimpleElement("voice",str(self.DestVoice[iVoice]))
                            typestring = typeFromDenominator(durationRat, durationBase)
                            if typestring != "":
                                self.writeSimpleElement("type", typestring)
                            while dots > 0:
                                self.writeNewline("<dot/>\n")
                                dots -= 1
                            # Vorzeichen
                            if alter == None or step == "":
                                intalter = 0
                            else:
                                intalter = int(step)
                            accstring = self.getMXMLaccidental(str(strng), intalter, iVoice)
                            if accstring != "" and not tieStop:
                                if display != "suppress":
                                    self.writeSimpleElement("accidental", accstring)
                            elif (display == "force" or display == "parenth") and intalter >= -1 and intalter <= 1:
                                if intalter == -1:
                                    accstring = "flat"
                                elif intalter == 0:
                                    accstring = "natural"
                                elif intalter == 1:
                                    accstring = "sharp"
                                if display == "force":
                                    self.writeElementWithAttribute("accidental", "cautionary=\"yes\"", accstring)
                                elif display == "parenth":
                                    self.writeElementWithAttribute("accidental", "editorial=\"yes\"", accstring)
                            # Bindebogen, Xtolen?
                            SlurTupletEndstring = ""
                            SlurTupletStartstring = ""
                            if isFirstHead:
                                SlurTupletEndstring += self.checkSlurEnd(iVoice)
                                if drawObjects != None:
                                    SlurTupletStartstring = self.handleDrawObjectsSlur(drawObjects, iVoice)
                                    SlurTupletStartstring += self.handleDrawObjectsTuplet(drawObjects, iVoice)
                            # Tupel braucht time-modification
                            if numerator != 0:
                                self.writeNewline("<time-modification>\n")
                                self.indentlevel += 1
                                self.writeSimpleElement("actual-notes",str(denominator))
                                self.writeSimpleElement("normal-notes",str(numerator))
                                durBase, dts = self.getPendingTupletDurationBase(iVoice)
                                if durBase != 0 and (durBase != durationBase or dts != dots):
                                    self.writeSimpleElement("normal-type",typeFromDenominator(Rational(1,1),durBase))
                                    if dts > 0:
                                        self.writeSimpleElement("normal-dot","")
                                self.indentlevel -= 1
                                self.writeNewline("</time-modification>\n")
                            if isFirstHead and -1 == SlurTupletStartstring.find("tuplet"):
                                SlurTupletEndstring += self.checkTupletEnd(iVoice)
                            # Halsrichtung
                            if dir != "":
                                self.writeSimpleElement("stem", dir)
                            # Besondere Kopfform
                            shape = s.getAttribute("shape")
                            if shape != "":
                                mshape, filled = cShape2mShape(shape, durationBase <= 2)
                                self.writeElementWithAttributeAndChild("notehead",filled,mshape)
                            # Notation in Nachbarzeile
                            if self.StavesInPart > 1:
                                if otherstaff == "up" and self.staffNoPart[iVoice] > 1:
                                    self.writeSimpleElement("staff",str(self.staffNoPart[iVoice]-1))
                                elif otherstaff == "down" and self.staffNoPart[iVoice] < self.StavesInPart:
                                    self.writeSimpleElement("staff",str(self.staffNoPart[iVoice]+1))
                                else:
                                    self.writeSimpleElement("staff",str(self.staffNoPart[iVoice]))
                            # Balken
                            if durationBase >= 8:
                                self.handleBeam(obj, iVoice, durationBase)
                            # MXML notations:
                            # Akzente?
                            
                            if tieStart or tieStop or SlurTupletEndstring != "" or SlurTupletStartstring != "" or articulationString != "" or NotationsFromDrawObjectsString != "":
                                # MXML notations eintragen:
                                self.writeNewline("<notations>\n")
                                self.indentlevel += 1
                                # Akzente
                                if isFirstHead and articulationString != "":
                                    self.f.write(articulationString)
                                # Haltebogen malen
                                if tieStart:
                                    self.writeNewline("<tied type=\"start\"/>\n")
                                if tieStop:
                                    self.writeNewline("<tied type=\"stop\"/>\n")
                                # Bindebogen und Xtolen
                                if isFirstHead and SlurTupletEndstring != "":
                                    self.f.write(SlurTupletEndstring)
                                if isFirstHead and SlurTupletStartstring != "":
                                    self.f.write(SlurTupletStartstring)
                                # Fermaten
                                if isFirstHead and NotationsFromDrawObjectsString != "":
                                    self.f.write(NotationsFromDrawObjectsString)
                                self.indentlevel -= 1
                                self.writeNewline("</notations>\n")
                            # Liedtext
                            if isFirstHead:
                                isFirstHead = False
                                lyric = self.getFirstChildElement(obj, "lyric")
                                if lyric != None:
                                    self.handleLyric(lyric, iVoice)
                            self.indentlevel -= 1
                            self.writeNewline("</note>\n")
        self.checkWedgeEnd(iVoice)

    def peekNextChord(self, thischord):
        # Balken nicht über feste Taktstriche, Pausen, Tonartwechsel und Schlüssel hinaus!
        NextChord = thischord
        while NextChord != None:
            NextChord = self.nextSiblingElement(NextChord)
            if NextChord == None:
                break
            elif self.isElement(NextChord, 'chord'):
                break
            elif self.isElement(NextChord, 'barline') or self.isElement(NextChord, 'rest'):
                return None
            elif self.isElement(NextChord, 'keySign') or self.isElement(NextChord, 'clefSign'):
                return None
        return NextChord

    def checkVoltaOverlap(self, thischord, pending):
        # Falls Voltenklammern überlappen (kann hier nur in der gleichen Stimme bemerkt werden)
        n = thischord
        pend = pending
        while pend > 0:
            n = self.nextSiblingElement(n)
            if n == None:
                return pending - pend
            drawObjects = n.getElementsByTagName("drawObj")
            if len(drawObjects) > 0:
                for drawObj in drawObjects:
                    volta = self.getFirstChildElement(drawObj, "volta")
                    if volta != None:
                        return pending - pend
            if n == None:
                break
            elif self.isElement(n, 'chord') or self.isElement(n, 'rest'):
                pend -= 1
            elif self.isCAPX2 and self.isElement(n, 'barline'):
                pend -= 1
        return pending

    def peekVolta(self, thisNoteObj):
        # schaut nach, ob an das nachfolgende Notenobjekt eine Voltenklammer angebunden ist
        noteObjects = thisNoteObj.parentNode
        found = False
        for n in noteObjects.childNodes:
            if self.isElement(n, "barline") or self.isElement(n, "chord") or self.isElement(n,"rest"):
                if found:
                    drawObjects = n.getElementsByTagName("drawObj")
                    if len(drawObjects) > 0:
                        for drawObj in drawObjects:
                            volta = self.getFirstChildElement(drawObj, "volta")
                            if volta != None:
                                basic = self.getFirstChildElement(drawObj, "basic")
                                return volta, basic # basic kann auch None sein.
                    return None, None
                if n == thisNoteObj:
                    found = True
        return None, None
    
    def writeVoltaStartBarline(self,number):
        # Wenn ein VoltaStart mit einem festen Taktstrich zusammentrifft,
        # haben wir ein Problem. Nur eine barline wird beim Import gelesen.
        self.writeNewline("<barline location=\"left\">\n")
        self.indentlevel += 1
        self.writeVoltaStart(number)
        self.indentlevel -= 1
        self.writeNewline("</barline>\n")

    def voltaNumberFromString(self, NumberString):
        # Das Attribut number soll nur Integer oder durch Komma getrennte Integer "1, 2, 3" enthalten.
        # MusicXML hat keine Beschreibungsmittel für '1.-3.'
        # Deshalb wird der Effektivstring in den Text geschrieben.
        # Finale, MuseScore und capella verstehen das. PriMus 1.1.1448 und Sibelius 8.0.1 nicht.       
        i2 = NumberString.find("-")
        if i2 > 0:
            iFrom = int(NumberString[:NumberString.find(".")])
            iTo = int(NumberString[i2+1:])
            NumberNoDots = str(iFrom)
            while iFrom < iTo:
                iFrom += 1
                NumberNoDots += ","+str(iFrom)
        else:
            NumberNoDots = NumberString.replace(".", ",")
        return NumberNoDots
        
    def writeVoltaStart(self,firstNumber):
        # nur als Element von barline zugelassen!
        firstNumberNoDots = self.voltaNumberFromString(firstNumber)
        self.writeElementWithAttributeAndChild("ending", "type=\"start\" number=\""+firstNumberNoDots+"\"", firstNumber+".")

    def writeVoltaStopBarline(self, number, discontinue):
        self.writeNewline("<barline>\n")
        self.indentlevel += 1
        self.writeVoltaStop(number, discontinue)
        self.indentlevel -= 1
        self.writeNewline("</barline>\n")

    def writeVoltaStop(self, number, discontinue):
        numberNoDots = self.voltaNumberFromString(number)
        if discontinue:
            self.writeNewline("<ending type=\"discontinue\" number=\""+numberNoDots+"\"/>\n")
        else:
            self.writeNewline("<ending type=\"stop\" number=\""+numberNoDots+"\"/>\n")

    def peekBarline(self, thischordrest):
        Barline = None
        noteObjects = thischordrest.parentNode
        chordrests = self.getDirectChildrenBy2TagNames(noteObjects, "chord","rest")
        found = False
        for chordrest in chordrests:
            if found:
                if self.isElement(chordrest, "chord") or self.isElement(chordrest, "rest"):
                    Barline = None
                    break
                Barline = chordrest
                break
            if chordrest == thischordrest:
                found = True
        return Barline

    def findVoice(self, NextSystem, iVoice):
        voice = None
        StavesToHandle = []
        self.getStavesToHandle(NextSystem, StavesToHandle)
        i = -1
        for staff in StavesToHandle:
            for n in staff.childNodes:
                if self.isElement(n, 'voices'):
                    for s in n.childNodes:
                        if self.isElement(s, 'voice'):
                            i += 1
                            if i == iVoice:
                                voice = s
                                break
        return voice

    def peekNextSystemVoice(self, voice, iVoice):
        NextVoice = None
        myVoices = voice.parentNode
        myStaff = myVoices.parentNode
        myStaves = myStaff.parentNode
        mySystem = myStaves.parentNode
        Systems = mySystem.parentNode
        # Hier ist getElementsByTagName wahrscheinlich ineffektiv, man bräuchte nur die
        # unmittelbaren Kinder zu untersuchen:
        allsystems = self.getDirectChildrenByTagName(Systems, "system")
        found = False
        for system in allsystems:
            if found:
                NextSystem = system
                NextVoice = self.findVoice(NextSystem, iVoice)
                break
            if system == mySystem:
                found = True
        return NextVoice

    def getToConnect(self, iVoice):
        # Implementation der beamGrouping-Regeln
        toConnect = 0
        beats, beatType, symbol = self.Time[self.staffIndSystem[iVoice]]
        intbeats = int(beats)
        if self.beamGrouping == 1:
            toConnect = 1
        elif self.beamGrouping == 2:
            if intbeats <= 4:
                toConnect = 1
            else:
                toConnect = 2
            # noch nicht vollständig detailliert!
        elif self.beamGrouping == 3:
            if intbeats == 2 or intbeats == 4:
                toConnect = 2
            elif intbeats == 3 or intbeats == 6 or intbeats == 9 or intbeats == 12:
                toConnect = 3
        return toConnect
            
    def fitsInCurrentBeamGroup(self, iVoice, duration):
        beats, beatType, symbol = self.Time[self.staffIndSystem[iVoice]]
        BeamGroupLength = self.arDivisions[self.iSystem] * int(self.getToConnect(iVoice)) * 4 / int(beatType)
        if BeamGroupLength == 0:
            return False
        durationSinceGroupStart = self.durationsum[iVoice]
        while durationSinceGroupStart > BeamGroupLength:
            durationSinceGroupStart -= BeamGroupLength
        dur = duration * 4 * self.arDivisions[self.iSystem]
        return durationSinceGroupStart + dur <= BeamGroupLength
    
    def handleBeam(self, chord, iVoice, durationBase):
        outbeamStart = 0
        outbeamContinue = 0
        outbeamStop = 0
        outbeamSelf = beamsFromDenominator(durationBase)
        # zuerst die manuell geforderten Balken bzw. Balkenverbote
        forbidden = False
        divide = False
        beam = self.getFirstChildElement(chord,"beam")
        if beam != None:
            beamattr = beam.getAttribute("group")
            if "force" == beamattr:
                if self.outbeamStarted > 0:
                    outbeamContinue = outbeamSelf
                else:
                    outbeamStart = outbeamSelf
            elif "split" == beamattr:
                forbidden = True
            elif "divide" == beamattr:
                divide = True
        # jetzt die automatisch zu schreibenden Balken
        neighborChord = self.peekNextChord(chord)
        if not forbidden and neighborChord != None:
            durbase,dot = self.getDurationBase(neighborChord)
            neighborDuration,nnumerator,ndenominator = self.duration(neighborChord)
            if durbase >= 8 and self.fitsInCurrentBeamGroup(iVoice, neighborDuration.__float__()):
#            if durbase >= 8 and self.fitsInCurrentBeamGroup(iVoice, 1.0/durbase):
                if outbeamSelf >= self.outbeamStarted:
                    outbeamStart = outbeamSelf - self.outbeamStarted
                    outbeamContinue = self.outbeamStarted
                else:
                    outbeamStop = self.outbeamStarted - outbeamSelf
                    outbeamContinue = outbeamSelf
            else:
                outbeamStop = self.outbeamStarted
        else:
            outbeamStop = self.outbeamStarted
        if divide:
            outbeamStop += (outbeamContinue - 1)
            outbeamContinue = 1
        # fürs nächste Mal Status merken
        self.outbeamStarted = outbeamStart + outbeamContinue
        # jetzt die Balken schreiben
        outbeamWritten = 0
        while outbeamContinue > 0:
            outbeamWritten += 1
            self.writeElementWithAttribute("beam","number=\""+str(outbeamWritten)+"\"","continue")
            outbeamContinue -= 1
        while outbeamStart > 0:
            outbeamWritten += 1
            self.writeElementWithAttribute("beam","number=\""+str(outbeamWritten)+"\"","begin")
            outbeamStart -= 1
        while outbeamStop > 0:
            outbeamWritten += 1
            self.writeElementWithAttribute("beam","number=\""+str(outbeamWritten)+"\"","end")
            outbeamStop -= 1
            
            
        

    def finishChord(self, chord):
        pass

    def handleRest(self, obj, iVoice):
        wasMultipleRest = False
        NotationsFromDrawObjectsString = ""
        SlurTupletEndstring = ""
        SlurTupletStartstring = ""
        drawObjects = self.getFirstChildElement(obj, "drawObjects")
        if drawObjects != None:
            NotationsFromDrawObjectsString = self.handleDrawObjects(drawObjects, iVoice)
        tempo = self.mkTempo(obj.getAttribute("tempo"))
        if len(tempo) > 0:
            self.writeTempoDirection(tempo)
            self.tempo = tempo
        durationBase, dots = self.getDurationBase(obj)
        durationRat,numerator,denominator,wholeMeasure = self.durationRest(obj, iVoice)
        duration = durationRat.__float__() * 4 * self.arDivisions[self.iSystem]
        if duration == 0:
            # capella erlaubt Pausen ohne Wert. MusicXML nicht.
            # Wenn daran drawObjects angehängt wären, werden die nicht exportiert!
            return wasMultipleRest
        self.durationsum[iVoice] += duration

        plainDurationBase = self.getFirstChildElement(obj, 'duration').getAttribute('base')
        if wholeMeasure and int(plainDurationBase) > 1:
            wasMultipleRest = True
            if iVoice == 0:
                # in MusicXML gibt's nur eine Mehrtaktpause für alle Stimmen
                self.writeMultipleRest(int(plainDurationBase), iVoice, duration)
            return wasMultipleRest

        invisible = False
        display = self.getFirstChildElement(obj, "display")
        centered = False
        verticalPos = self.getFirstChildElement(obj, "verticalPos")
        if verticalPos != None:
            centered = "true" == verticalPos.getAttribute("centered")
        if display != None:
            invisible = "true" == display.getAttribute("invisible")
        if invisible:
            invistring = " print-object=\"no\""
        else:
            invistring = ""
        if invisible and globalExportToSibelius:
            # Sibelius 4 unterstützt keine Unsichtbaren Pausen
            self.writeForward(duration, iVoice)
        else:
            self.writeNewline("<note%s>\n"%invistring)
            self.indentlevel += 1
            if centered:
                if wholeMeasure:
                    self.writeNewline("<rest measure=\"yes\">\n")
                else:
                    self.writeNewline("<rest>\n")
                self.indentlevel += 1
                # der Finale-Default ist für alle Pausen gleich, aber vom Schlüssel abhängig
                clefIndex = self.staffNoPart[iVoice] - 1
                clef, line, octaveChange = self.Clefs[clefIndex]
                if clef == "G" and line == "2":
                    self.writeSimpleElement("display-step","B")
                    if octaveChange == "-1":
                        self.writeSimpleElement("display-octave","3")
                    else:
                        self.writeSimpleElement("display-octave","4")
                elif clef == "F" and line == "4":
                    self.writeSimpleElement("display-step","D")
                    self.writeSimpleElement("display-octave","3")
                elif clef == "C" and line == "3":
                    self.writeSimpleElement("display-step","C")
                    self.writeSimpleElement("display-octave","4")
                elif clef == "C" and line == "4":
                    self.writeSimpleElement("display-step","A")
                    self.writeSimpleElement("display-octave","3")
                self.indentlevel -= 1
                self.writeNewline("</rest>\n")
            else:
                if wholeMeasure:
                    self.writeNewline("<rest measure=\"yes\"/>\n")
                else:
                    self.writeNewline("<rest/>\n")
            self.writeSimpleElement("duration",str(int(duration)))
            self.writeSimpleElement("voice",str(self.DestVoice[iVoice]))
            typestring = typeFromDenominator(durationRat, durationBase)
            if typestring != "":
                self.writeSimpleElement("type", typestring)
            while dots > 0:
                self.writeNewline("<dot/>\n")
                dots -= 1
            SlurTupletEndstring = self.checkSlurEnd(iVoice)
            if drawObjects != None:
                SlurTupletStartstring = self.handleDrawObjectsSlur(drawObjects, iVoice)
                SlurTupletStartstring += self.handleDrawObjectsTuplet(drawObjects, iVoice)

            if numerator != 0:
                self.writeNewline("<time-modification>\n")
                self.indentlevel += 1
                self.writeSimpleElement("actual-notes",str(denominator))
                self.writeSimpleElement("normal-notes",str(numerator))
                durBase, dts = self.getPendingTupletDurationBase(iVoice)
                if durBase != 0 and (durBase != durationBase or dts != dots):
                    self.writeSimpleElement("normal-type",typeFromDenominator(Rational(1,1),durBase))
                    if dts > 0:
                        self.writeSimpleElement("normal-dot","")
                self.indentlevel -= 1
                self.writeNewline("</time-modification>\n")
            if -1 == SlurTupletStartstring.find("tuplet"):
                SlurTupletEndstring += self.checkTupletEnd(iVoice)
            if self.StavesInPart > 1:
                self.writeSimpleElement("staff",str(self.staffNoPart[iVoice]))
            # Gibt's Notations?
            if SlurTupletEndstring != "" or SlurTupletStartstring != "" or NotationsFromDrawObjectsString != "":
                self.writeNewline("<notations>\n")
                self.indentlevel += 1
                if SlurTupletEndstring != "":
                    self.f.write(SlurTupletEndstring)
                if SlurTupletStartstring != "":
                    self.f.write(SlurTupletStartstring)
                if NotationsFromDrawObjectsString != "":
                    self.f.write(NotationsFromDrawObjectsString)
                self.indentlevel -= 1
                self.writeNewline("</notations>\n")
            self.indentlevel -= 1
            self.writeNewline("</note>\n")
            self.checkWedgeEnd(iVoice)
        return wasMultipleRest

    def writeMultipleRest(self, n, iVoice, duration):
        duration1Meas = duration/n
        self.writeNewline("<attributes>\n")
        self.indentlevel += 1
        self.writeElementWithAttributeAndChild("measure-style","", "<multiple-rest>"+str(n)+"</multiple-rest>")
        self.indentlevel -= 1
        self.writeNewline("</attributes>\n")
        # zur Sicherheit (auch für MusicToCap bis Version 1.15, 2015) noch eine Pause reinschreiben
        self.writeNewline("<note>\n")
        self.indentlevel += 1
        self.writeNewline("<rest measure=\"yes\"/>\n")
        self.writeSimpleElement("duration",str(int(duration1Meas)))
        self.writeSimpleElement("voice",str(self.DestVoice[iVoice]))
        self.indentlevel -= 1
        self.writeNewline("</note>\n")
        n -= 1
        while n > 0:
            self.writeRestMeasure() # mit width="0"
            # zur Sicherheit (auch für MusicToCap bis Version 1.15, 2015) noch eine Pause reinschreiben
            self.writeNewline("<note>\n")
            self.indentlevel += 1
            self.writeNewline("<rest measure=\"yes\"/>\n")
            self.writeSimpleElement("duration",str(int(duration1Meas)))
            self.writeSimpleElement("voice",str(self.DestVoice[iVoice]))
            self.indentlevel -= 1
            self.writeNewline("</note>\n")
            self.durationsum[iVoice] += duration1Meas
            
            self.MeasureCount -= 1 # MeasureCount soll diese Takte nicht zählen
            n -= 1                                                   

    def handleClefSign(self, clefSign, iVoice):
        clefIndex = self.staffNoPart[iVoice] - 1
        clef = clefSign.getAttribute('clef')
        if clef == "treble":
            if self.Clefs[clefIndex] != ("G","2","0"):
                self.Clefs[clefIndex] = ("G","2","0")
                self.changedAttrClef[clefIndex] = True
        elif clef == "bass":
            if self.Clefs[clefIndex] != ("F","4","0"):
                self.Clefs[clefIndex] = ("F","4","0")
                self.changedAttrClef[clefIndex] = True
        elif clef == "alto":
            if self.Clefs[clefIndex] != ("C","3","0"):
                self.Clefs[clefIndex] = ("C","3","0")
                self.changedAttrClef[clefIndex] = True
        elif clef == "tenor":
            if self.Clefs[clefIndex] != ("C","4","0"):
                self.Clefs[clefIndex] = ("C","4","0")
                self.changedAttrClef[clefIndex] = True
        elif len(clef) > 1 and (clef[0] == "C" or clef[0]== "F" or clef[0] == "G"):
            if clef[1] == "2" or clef[1] == "3" or clef[1] == "4":
                if len(clef) > 2:
                    if clef[2] == "+":
                        self.Clefs[clefIndex] = (clef[0], clef[1], "1")
                        self.changedAttrClef[clefIndex] = True
                    elif clef[2] == "-":
                        self.Clefs[clefIndex] = (clef[0], clef[1], "-1")
                        self.changedAttrClef[clefIndex] = True
                else:
                    self.Clefs[clefIndex] = (clef[0], clef[1], "0")
                    self.changedAttrClef[clefIndex] = True
        elif clef[0] == "P":
            self.Clefs[clefIndex] = ("percussion", "3", "0")
            self.changedAttrClef[clefIndex] = True
            
        
        
    def handleKeySign(self, keySign):
        fifths = int(keySign.getAttribute('fifths'))
        if self.Key != (str(fifths),"major"):
            self.Key = str(fifths),"major"
            self.changedAttrKey = True

    def handleTimeSign(self, timeSign, iStaff):
        time = timeSign.getAttribute('time')
        self.handleTimeString(time, iStaff, True)

    def handleTimeString(self, time, iStaff, check_C):
        # check_C: unterscheide 2/2 und "allaBreve"
        # sowie 4/4 und "C"
        if time == "allaBreve":
            if check_C:
                if self.Time[iStaff] != ("2","2","cut"):
                    self.Time[iStaff] = "2","2","cut"
                    self.changedAttrTime = True
            else:
                if self.Time[iStaff] != ("2","2","cut") and self.Time[iStaff] != ("2","2",""):
                    self.Time[iStaff] = "2","2","cut"
                    self.changedAttrTime = True
        elif time == "longAllaBreve":
            if check_C:
                if self.Time[iStaff] != ("4","2","cut"):
                    self.Time[iStaff] = "4","2","cut"
                    self.changedAttrTime = True
            else:
                if self.Time[iStaff] != ("4","2","cut") and self.Time[iStaff] != ("4","2",""):
                    self.Time[iStaff] = "4","2","cut"
                    self.changedAttrTime = True
        elif time == "C":
            if check_C:
                if self.Time[iStaff] != ("4","4","common"):
                    self.Time[iStaff] = "4","4","common"
                    self.changedAttrTime = True
            else:
                if self.Time[iStaff] != ("4","4","common") and self.Time[iStaff] != ("4","4",""):
                    self.Time[iStaff] = "4","4","common"
                    self.changedAttrTime = True
        elif time == "infinite":
            if self.Time[iStaff] != ("1","1","senza-misura"):
                self.Time[iStaff] = "1","1","senza-misura"
                self.changedAttrTime = True
        else:
            t = time.split('/')
            if check_C:
                if self.Time[iStaff] != (t[0],t[1],""):
                    self.Time[iStaff] = t[0],t[1],""
                    self.changedAttrTime = True
            else:
                if self.Time[iStaff] == ("2","2","cut") and t[0] == "2" and t[1] == "2":
                    pass
                elif self.Time[iStaff] == ("4","4","common") and t[0] == "4" and t[1] == "4":
                    pass
                elif self.Time[iStaff] != (t[0],t[1],""):
                    if float(t[0]) / float(t[1]) < 1.0/128.0:
                        # es gab schon mal defaultTime von 32768 - unklar wie das entstanden ist
                        if self.Time[iStaff] != ("1","1","senza-misura"):
                            self.Time[iStaff] = "1","1","senza-misura"
                            self.changedAttrTime = True
                    else:
                        self.Time[iStaff] = t[0],t[1],""
                    self.changedAttrTime = True
        
    def startNoteObj(self, n):
        pass

    def finishNoteObj(self, n):
        pass



    def handleHead(self, head):
        return ""

    def durationRest(self, note, iVoice):
        """ Dauer einer Pause als rationale Zahl in ganzen Noten
            <note> muss ein 'rest'-Knoten sein
        """
        
        duration = self.getFirstChildElement(note, 'duration')
        if duration.getAttribute('noDuration') == 'true':
            rat0 = Rational(0,1)
            return rat0,0,1,False
        durbase = str(duration.getAttribute('base'))
        wholeMeasure = -1 == durbase.find("/")
        n = Rational(durbase)

        # Ganztaktpause?
        if wholeMeasure:
            beats,beatType,symbol = self.Time[self.staffIndSystem[iVoice]]
            intbeats = int(beats)
            intbeatType = int(beatType)
            if intbeats != intbeatType:
                ret = Rational(intbeats,intbeatType)
                ret *= n.p
                return ret,0,0,wholeMeasure

        dots = duration.getAttribute('dots')
        if dots != '':
            if int(dots) == 1: n = (3 * n) / 2
            else: n = (7 * n) / 4

        tuplet = duration.getElementsByTagName('tuplet')
        if len(tuplet) > 0 and "" != tuplet[0].getAttribute('count'):
            denominator = Rational(int(tuplet[0].getAttribute('count')))
            numerator = Rational(1)
            if tuplet[0].getAttribute('tripartite') == 'true':
               numerator = Rational(3,2)
            while numerator < denominator: numerator *= 2
            if tuplet[0].getAttribute('prolong') != 'true':
               numerator /= 2
            n = (n * numerator) / denominator
        else:
            numerator = Rational(0)
            denominator = Rational(0)

        return n,numerator,denominator,wholeMeasure
        
    def duration(self, note):
        """ Dauer einer Note als rationale Zahl in ganzen Noten
            <note> muss ein 'chord'- Knoten sein
        """
        duration = self.getFirstChildElement(note, 'duration')
        if duration.getAttribute('noDuration') == 'true':
            rat0 = Rational(0,1)
            return rat0,0,1
        n = Rational(str(duration.getAttribute('base')))

        dots = duration.getAttribute('dots')
        if dots != '':
            if int(dots) == 1: n = (3 * n) / 2
            else: n = (7 * n) / 4

        tuplet = duration.getElementsByTagName('tuplet')
        if len(tuplet) > 0 and "" != tuplet[0].getAttribute('count'):
            denominator = Rational(int(tuplet[0].getAttribute('count')))
            numerator = Rational(1)
            if tuplet[0].getAttribute('tripartite') == 'true':
               numerator = Rational(3,2)
            while numerator < denominator: numerator *= 2
            if tuplet[0].getAttribute('prolong') != 'true':
               numerator /= 2
            n = (n * numerator) / denominator
        else:
            numerator = Rational(0)
            denominator = Rational(0)

        return n,numerator,denominator

#-----------------------------------------------------------------------
def languageFromLocale():
# L_German=1,L_Finnish=3,L_Swedish=4,L_EnglishUS=5,L_EnglishGB=6,L_Dutch=7,
#         L_Czech=8,L_Polish=9,L_International=10,L_French=11
        l = getLocale()
        langIDs = {"de":1, "fi":3, "en":5, "nl":7, "fr":11}
        if langIDs.has_key(l[:2]):
            lang = langIDs[l[:2]]
        else:
            lang = 1
        return lang

def writeMusicXMLRel():
    """ schreibt die aktuelle Partitur unter gleichem Namen mit Erweiterung '.xml'
    """
    if activeScore():
        outFile = os.path.splitext(activeScore().pathName())[0] + '.xml'
        fd = FileDialog(False)
        language = languageFromLocale()
        fillTextbasket(language)
        filter = myTr("fileFilter")
        fd.addFilter(filter, "*.xml")
        fd.setDefaultExt("xml")
        fd.setStartFile(outFile)
        if fd.run():
            time1 = time.time()
            tmpFile = tempfile.mktemp('.capx')
            outFile = fd.filePath()
            activeScore().write(tmpFile)
            time2 = time.time()
            ME = MusicXMLExport(unicode(tmpFile), unicode(outFile))
            if ME.success == 5:
                myMessage(myTr("errorConversionTitle"), myTr("errorConversionLocation") % (ME.iSystem,ME.MeasureCount+1,ME.layoutsInPart[0]))
            elif ME.success == 4:
                myMessage(myTr("errorReadingTitle"), myTr("errorReadingDetails") % (tmpFile, ME.parseError))
            os.remove(tmpFile)
    
def writeMusicXMLStandalone(filename, outfilename):
    sourcefile = filename + ".capx"
    outFile = outfilename + ".xml"
    ME = MusicXMLExport(unicode(sourcefile),unicode(outFile))
    if ME.success >= 3:
        global language
        if ME.success == 4:
            myMessage(myTr("errorReadingTitle"), myTr("errorReadingDetails") % (sourcefile,ME.parseError))
        elif ME.success == 5:
            myMessage(myTr("errorConversionTitle"), myTr("errorConversionLocation") % (ME.iSystem,ME.MeasureCount+1,ME.layoutsInPart[0]))
        sys.tracebacklimit = 0
        str2raise = myTr("truncated")
        raise str2raise

import getopt,sys

def usage():
    global language
    errstring = myTr("errorCommandline")
    usagestring = myTr("usage")
    print >> sys.stderr, errstring
    print >> sys.stderr, usagestring
# folgende MessagBox kann bei aktiviert werden, falls stderr nicht zugänglich ist
#    tkMessageBox.showinfo("CapToMusic",errstring+'\n'+usagestring)
    
def doitFromCommandLine():  # or from the Python shell
    global language
    try:
        opts, args = getopt.getopt(sys.argv[1:], "hl:f:o:", ["help", "language=", "filename=", "destination="])
    except getopt.GetoptError:
        usage()
        str2raise = myTr("truncated")
        raise str2raise
    filename = None
    outfilename = None
    noBreakExport = False
#    myMessage("CapToMusic","opts %s, args %s" % (str(opts),str(args)))
    for f,a in opts:
        if f == "-n":
            noBreakExport = True
        elif f in ("-h", "--help"):
            print >> sys.stderr, __doc__
            sys.exit()
        elif f == "-f":
            filename = a
        elif f == "-o":
            outfilename = a
        elif f == "-l":
            if a == "1" or a == "5" or a == "11":
                language = int(a)
    fillTextbasket(language)
    if filename == None or filename == "":
        # gerufen von python shell
        setting = locale.getlocale()
        if setting[0] == 'de_DE':
            language = 1
        fillTextbasket(language)
        global doitwithTK
        if doitwithTK:
            filenameWithExt=tkFileDialog.askopenfilename(filetypes=[(myTr("srcFileType"), "capx")])
            filename = filenameWithExt[:-5]
        else:
            print >> sys.stdout, "input file path without extension"
            filename = raw_input("->")
    else:
        # gerufen von Kommandozeile (typisch durch capscan.exe)
        doitwithTK = False
    if filename != None and filename != "":
        if outfilename == None or outfilename == "":
            outfilename = filename    
        writeMusicXMLStandalone(filename, outfilename)
    else:
        print >> sys.stderr, "cancelled"


def myMessage(title,text):
    global globalStandalone
    if globalStandalone:
        global doitwithTK
        if doitwithTK:
            tkMessageBox.showinfo(title,text)
        else:
            print >> sys.stderr, title+"\n"+text
    else:
        try:
            messageBox(title,text)
        except:
            messageBox(title,latin1simple(text))


def doit():
    global globalStandalone
    global globalFinaleDolet33
    global globalChordCaseMatters
    global globalExportToSibelius
    global globalPagesizeXmm
    global globalPagesizeYmm
    globalStandalone = False
    active = None
    try:
        active = activeScore()
    except:
        # activeScore ist undefiniert, wir sind nicht aus capella gestartet.
        globalStandalone = True
        pass
    # Persistente Optionen
    o = MyScriptOptions()
    opts = o.get()
    if opts.has_key("FinaleDolet33"):
        globalFinaleDolet33 = 0 != int(opts["FinaleDolet33"])
    if opts.has_key("ChordCaseMatters"):
        globalChordCaseMatters = 0 != int(opts["ChordCaseMatters"])
    if opts.has_key("ExportToSibelius"):
        globalExportToSibelius = 0 != int(opts["ExportToSibelius"])
    if opts.has_key("PagesizeX(mm)"):
        globalPagesizeXmm = float(opts["PagesizeX(mm)"])
    if opts.has_key("PagesizeY(mm)"):
        globalPagesizeYmm = float(opts["PagesizeY(mm)"])
    if active != None:
        writeMusicXMLRel()
    else:
        doitFromCommandLine()



globalStandalone = False
if __name__ == "__main__":
    doit()
